/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// CommunicationSettings.cpp : implementation file
//
#include "stdafx.h"
#include "ui.h"
#include "CommunicationSettings.h"
#include "pim/ClientSettings.h"

#include "PushSettings.h"
#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "pim/Maincpp.h"
#include "tpcshell.h"  // for SHSendBackToFocusWindow
#include "pim/SettingFunctions.h"
#include "base/startcmd.h"
#include "processUtils.h"
#include "pim/account.h"
#include "pim/utils.h"
#include "notify/timed_msgbox.h"
// CCommunicationSettings dialog

IMPLEMENT_DYNAMIC(CCommunicationSettings, CDialog)

CCommunicationSettings::CCommunicationSettings(CWnd* pParent /*=NULL*/)
    : CDialog(CCommunicationSettings::IDD, pParent)
{

}

CCommunicationSettings::~CCommunicationSettings()
{
#if defined(WIN32_PLATFORM_PSPC)
    if (FUNOPT_SHOW_URL) {
        editUrl.Detach();
    }
    editUser.Detach();
    editPassword.Detach();
#endif
}

void CCommunicationSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    if (FUNOPT_SHOW_URL) {
        DDX_Control(pDX, IDC_COMM_EDIT_URL, editUrl);
    }
    DDX_Control(pDX, IDC_COMM_EDIT_USER, editUser);
    DDX_Control(pDX, IDC_COMM_EDIT_PASS, editPassword);
}

void CCommunicationSettings::showMessage(CString message){

    alertMessage = message;

}
BOOL CCommunicationSettings::OnInitDialog()
{
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_COMMUNICATION_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    if (FUNOPT_SHOW_URL) {
        editUrl.SetLimitText(EDIT_MAXLENGTH);
    }
    else {
        // If the URL is not requested, hide it and re-position
        // the other elements.
        GetDlgItem(IDC_COMM_EDIT_URL)->ShowWindow(SW_HIDE);
        CWnd *user = GetDlgItem(IDC_COMM_EDIT_USER);
        CWnd *pass = GetDlgItem(IDC_COMM_EDIT_PASS);

        modifyWindow(GetDlgItem(IDC_COMM_USER), 0, -62);
        modifyWindow(user, 0, -62);
        modifyWindow(GetDlgItem(IDC_COMM_PASS), 0, -52);
        modifyWindow(pass, 0, -52);
        modifyWindow(GetDlgItem(IDC_ALERT), 0, -5);

        //adjustToParent(user, ADJUST_WIDTH, 10, 20);
        //adjustToParent(pass, ADJUST_WIDTH, 10, 20);

    }
    editUser.SetLimitText(EDIT_MAXLENGTH);
    editPassword.SetLimitText(EDIT_MAXLENGTH);

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    if (FUNOPT_SHOW_URL) {
        GetDlgItem(IDC_COMM_SERVURL)->SetFont(&fontBold);
    }
    else {
        GetDlgItem(IDC_COMM_SERVURL)->ShowWindow(SW_HIDE);
    }
    GetDlgItem(IDC_COMM_USER)->SetFont(&fontBold);
    GetDlgItem(IDC_COMM_PASS)->SetFont(&fontBold);
    GetDlgItem(IDC_ALERT)->SetFont(&fontBold);

    if (FUNOPT_SHOW_URL){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SERVURL);
        SetDlgItemText(IDC_COMM_SERVURL, s1);
    }
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_USER);
    SetDlgItemText(IDC_COMM_USER, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_PASS);
    SetDlgItemText(IDC_COMM_PASS, s1);

    SetDlgItemText(IDC_ALERT, escapeAmpString(alertMessage));

    if(alertMessage.IsEmpty()){
        GetDlgItem(IDC_ALERT)->ShowWindow(SW_HIDE);
    }

#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_COMMUNICATION_SETTINGS);
    s1 = escapeAmpString(s1);
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif

    // read Registry settings
    ClientSettings* cs = getRegConfig();
    CString s3;

    if (FUNOPT_SHOW_URL) {
        s3 = cs->getAccessConfig().getSyncURL();
        SetDlgItemText(IDC_COMM_EDIT_URL, s3);
    }
    s3 = cs->getAccessConfig().getUsername();
    SetDlgItemText(IDC_COMM_EDIT_USER, s3);
    s3 = cs->getAccessConfig().getPassword();
    SetDlgItemText(IDC_COMM_EDIT_PASS, s3);

    if(checkStartSync() != 0)
    {
        // another sync is in progress, disable controls
        if (FUNOPT_SHOW_URL) {
            editUrl.EnableWindow(FALSE);
        }
        editUser.EnableWindow(FALSE);
        editPassword.EnableWindow(FALSE);
    }
    else
    {
        #ifdef WIN32_PLATFORM_WFSP
         // override back key
         LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
	            SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
         ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
        #endif
    }

    // set a large int as current selection, so the cursor will be at end
    if (FUNOPT_SHOW_URL) {
        editUrl.SetSel(LARGE_INT ,LARGE_INT);
    }
    editUser.SetSel(LARGE_INT ,LARGE_INT);
    editPassword.SetSel(LARGE_INT ,LARGE_INT);

    // fix for wrong title problem, we get & set the title for the main window
    CString title;
    AfxGetMainWnd()->GetWindowText(title);
    AfxGetMainWnd()->SetWindowText(title);

    if (FUNOPT_SHOW_URL) {
        editUrl.SetFocus();
    } else {
        editUser.SetFocus();
    }
    return FALSE;
}

BEGIN_MESSAGE_MAP(CCommunicationSettings, CDialog)
#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CCommunicationSettings::OnHotkey)
#endif
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


void CCommunicationSettings::OnOK()
{
    if(checkStartSync()){
        CDialog::OnOK();
    }

    // save values in registry
    ClientSettings* cs = getRegConfig();
    BOOL dirtyFlag=FALSE;

    // Save old values to check changes
    CString oldUrl, oldUsername, oldPassword;
    oldUrl = cs->getAccessConfig().getSyncURL();
    oldUsername = cs->getAccessConfig().getUsername();
    oldPassword = cs->getAccessConfig().getPassword();

    CString s1, url, username, password;

    if (FUNOPT_SHOW_URL) {
        GetDlgItemText(IDC_COMM_EDIT_URL, url);
    }
    else {
        url = oldUrl;
    }
    GetDlgItemText(IDC_COMM_EDIT_USER, username);
    GetDlgItemText(IDC_COMM_EDIT_PASS, password);

    if(!isInputValid(url, username, password)) {
        return;
    }

    char* tmp = NULL;

    if (FUNOPT_SHOW_URL) {
        tmp = toMultibyte(url.GetBuffer());

        if(strcmp(tmp,cs->getAccessConfig().getSyncURL()) != 0){
            cs->getAccessConfig().setSyncURL(tmp);
            dirtyFlag=TRUE;
        }
        if (tmp) {
            delete [] tmp; tmp = NULL;
        }
    }

    tmp = toMultibyte(username.GetBuffer());
    if(strcmp(tmp,cs->getAccessConfig().getUsername()) != 0){
        cs->getAccessConfig().setUsername(tmp);
        dirtyFlag=TRUE;
        changeMailSettingsInfo(username);
    }
    if (tmp) {
        delete [] tmp; tmp = NULL;
    }

    tmp = toMultibyte(password.GetBuffer());
    if(strcmp(tmp,cs->getAccessConfig().getPassword()) != 0){
        cs->getAccessConfig().setPassword(tmp);
        dirtyFlag=TRUE;
    }
    if (tmp) {
        delete [] tmp; tmp = NULL;
    }

    const char* devId = cs->getDeviceConfig().getDevID();
    if(strcmp(devId, "") == 0){
        wstring s1;
        #if defined(WIN32_PLATFORM_WFSP)
            s1 = getIMEI();
        #endif
        #if defined(WIN32_PLATFORM_PSPC)
            s1 = GetSerialNumberFromKernelIoControl();
        #endif

        tmp = toMultibyte(s1.c_str());
        if(strcmp(tmp,cs->getDeviceConfig().getDevID()) != 0){
            cs->getDeviceConfig().setDevID(tmp);
            cs->setDirty(CS_DIRTY_DEVICE_CONFIG);
        }
        if (tmp) {
            delete [] tmp; tmp = NULL;
        }
    }


    // If something changed and push is active, stop push and restart push
    // (if ctp is running, we need to restart it with updated credentials / url)
    if (cs->getPush() != "0") {
        if ( (oldUrl != url) || (oldUsername != username) || (oldPassword != password) ) {
            stopPushService();
            Sleep(500);
            startPushService();
        }
    }


    if(dirtyFlag){
        getRegConfig()->setDirty(CS_DIRTY_ACCESS_CONFIG);
    }
    getRegConfig()->saveDirty();


    // TODO: maybe here would be needed to save it to Registry, and not only to memory
    CDialog::OnOK();
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CCommunicationSettings::OnHotkey(WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == VK_TBACK)
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    return 0;
}
#endif

bool CCommunicationSettings::isInputValid(CString url, CString username, CString password){
    CString s1;
    CString lowerUrl(url);
    lowerUrl.MakeLower();

    if(url.IsEmpty()){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_EMPTY_URL);
        TimedMessageBox(NULL, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                             MB_OK | MB_ICONERROR | MB_SETFOREGROUND, INFINITE);

        //AfxMessageBox(s1);
        return false;
    };
    if(username.IsEmpty()){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_EMPTY_USERNAME);
        TimedMessageBox(NULL, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                             MB_OK | MB_ICONERROR | MB_SETFOREGROUND, INFINITE);

        //AfxMessageBox(s1);
        return false;
    };
    if(password.IsEmpty()){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_EMPTY_PASSWORD);
        TimedMessageBox(NULL, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                             MB_OK | MB_ICONERROR | MB_SETFOREGROUND, INFINITE);
        //AfxMessageBox(s1);
        return false;
    };
    if(username.Find(' ') != -1){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_USERNAME_CONTAINS_BLANKS);
        TimedMessageBox(NULL, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                             MB_OK | MB_ICONERROR | MB_SETFOREGROUND, INFINITE);
        //AfxMessageBox(s1);
        return false;
    }

    if (FUNOPT_SHOW_URL) {
        CString lowerUrl(url);
        lowerUrl.MakeLower();

        if(url.IsEmpty()){
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_EMPTY_URL);
            TimedMessageBox(NULL, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                             MB_OK | MB_ICONERROR | MB_SETFOREGROUND, INFINITE);
            //AfxMessageBox(s1);
            return false;
        };
    // check if url is valid
    if( ((lowerUrl.Left(7).Compare(_T("http://"))  != 0) &&
         (lowerUrl.Left(8).Compare(_T("https://")) != 0)   )  ||

         (lowerUrl.Compare(_T("http://")) == 0)               ||
         (lowerUrl.Compare(_T("https://")) == 0)              ||
         (lowerUrl.Left(8).Compare(_T("http:///")) == 0)      ||
         (lowerUrl.Left(9).Compare(_T("https:///"))== 0)      ||
         (lowerUrl.Mid(7).Find(_T("http:/")) != -1)        ||
         (lowerUrl.Mid(7).Find(_T("https:/")) != -1)        ||
         (lowerUrl.Find(' ') != -1)

      ) {

       // url is not valid
       s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_INVALID_URL);
       s1+=":\n\""; s1+=url; s1+="\"";
        TimedMessageBox(NULL, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                     MB_OK | MB_ICONERROR | MB_SETFOREGROUND, INFINITE);
        //AfxMessageBox(s1);
       return false;
        }
    }

    return true;
}


HBRUSH CCommunicationSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
        if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
            pDC->SetBkMode(TRANSPARENT);
            pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
        }

    #endif
        if(pWnd->GetDlgCtrlID() == IDC_ALERT) {
            pDC->SetBkMode(TRANSPARENT);
            pDC->SetTextColor(ALERT_COLOR);
        }
    return hbr;
}

void CCommunicationSettings::changeMailSettingsInfo(CString username) {

    SetCursor(LoadCursor(NULL, IDC_WAIT));
    HRESULT ret = -1;
    ret = modifyAccount(TEXT(""), DEFAULT_USER_W);
    SetCursor(LoadCursor(NULL, IDC_ARROW));
}