/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// CustomButton.cpp : implementation file
//

#include "stdafx.h"
#include "CustomButton.h"
#include "ui.h"


// CCustomButton dialog

IMPLEMENT_DYNAMIC(CCustomButton, CButton)

CCustomButton::CCustomButton()
	: CButton()
{
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD; lf.lfUnderline = TRUE;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontLink.CreateFontIndirect(&lf));
}

CCustomButton::~CCustomButton()
{
}

void CCustomButton::DoDataExchange(CDataExchange* pDX)
{
	CButton::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CCustomButton, CButton)
    ON_WM_DRAWITEM()
    ON_WM_PAINT()
    ON_WM_SETFOCUS()
    ON_WM_CTLCOLOR()
    ON_WM_ENABLE()
    ON_WM_LBUTTONDOWN()
    ON_WM_LBUTTONUP()
    ON_WM_SETTINGCHANGE()
END_MESSAGE_MAP()


// CCustomButton message handlers

void CCustomButton::OnPaint(){
    CPaintDC dc(this);

    int savedDC = dc.SaveDC();
    DrawButton(dc);
    dc.RestoreDC(savedDC);
}


void CCustomButton::OnSetFocus(CWnd* pOldWnd ){
    this->SetFocus();
    Invalidate();
}

HBRUSH CCustomButton::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor){
    HBRUSH hbr = (HBRUSH) CButton::OnCtlColor(pDC, pWnd, nCtlColor);

    pDC->SetBkMode(TRANSPARENT);
    Invalidate();
    return (HBRUSH) GetStockObject(WHITE_BRUSH);
}

void CCustomButton::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct){
    CDC dc;
    dc.Attach(lpDrawItemStruct->hDC);
    int savedDC = dc.SaveDC();

    DrawButton(dc);

    dc.RestoreDC(savedDC);
    dc.Detach();
}

void CCustomButton::DrawButton( CDC &dc ){
    CRect rect;
    GetClientRect(&rect);

    dc.FillSolidRect(rect, RGB(255,255,255));

    CRect rectFocus(rect);
    if(this->GetFocus() == this){
        dc.DrawFocusRect(rectFocus);
    };

    rect.DeflateRect(3,1);

    CFont* pOldFont = dc.SelectObject(&fontLink);

    if(IsWindowEnabled()){
        dc.SetTextColor(LINK_COLOR_NORMAL);
    }
    else{
        dc.SetTextColor(GetSysColor(COLOR_GRAYTEXT));
    }
    dc.DrawText(text, rect, DT_LEFT | DT_SINGLELINE);

    dc.SelectObject(pOldFont);
}

void CCustomButton::OnEnable( BOOL bEnable ){
    CButton::OnEnable(bEnable);
    Invalidate();
}

void CCustomButton::OnLButtonDown( UINT nFlags, CPoint point ){
    CButton::OnLButtonDown(nFlags, point);
    Invalidate();
}

void CCustomButton::OnLButtonUp( UINT nFlags, CPoint point ){
    CButton::OnLButtonUp(nFlags, point);
    Invalidate();
}

void CCustomButton::OnSettingChange( UINT uFlags, LPCTSTR lpszSection ){
    Default();
    Invalidate();
}