/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// RecoverSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "RecoverSettings.h"
#include "pim/utils.h"
#include "pim/Maincpp.h"
#include "base/startcmd.h"
#include "pim/SettingFunctions.h"
#include "pim/ClientSettings.h"
#include "pim/account.h"
#include "ClientUtil.h"
#include "notify/timed_msgbox.h"
#include "HwndFunctions.h"
#include "winreg.h"
#include <vector>
#include "localizationUtils.h"
#include "processUtils.h"

 static BOOL cleanupStarted = false;

static int checkRefreshFromServer(HWND wndUI, vector<const wchar_t*> &sources, BOOL cleanup);
static wstring getCommandLine(vector<const wchar_t*> &sources);
int endCleanUpProcess(HWND wndUI, vector<const wchar_t*> &sources);
static LOGFONT lf;
static HANDLE hMutex;

// CRecoverSettings dialog
void writeSourceForSlow(const wchar_t*  value);
IMPLEMENT_DYNAMIC(CRecoverSettings, CDialog)

/**
 * Parameters for the cleanup task
 */
class CleanUpParams {
public:
    CDialog &wndRef;
    vector<const wchar_t*> srcRef;
    CuiDlg* mainDlg;

    CleanUpParams(CDialog &wnd, vector<const wchar_t*> sources, CuiDlg* dlg) :
            wndRef(wnd),srcRef(sources), mainDlg(dlg) {};
    ~CleanUpParams() {};
} ;

// Cleanup thread
static DWORD WINAPI cleanUpTask(LPVOID lpv);


CRecoverSettings::CRecoverSettings(CWnd* pParent /*=NULL*/)
    : CDialog(CRecoverSettings::IDD, pParent)
{
    oldResId = -1;
}

CRecoverSettings::~CRecoverSettings()
{
}

void CRecoverSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_RECOVER_CONTACTS, checkContacts);
    DDX_Control(pDX, IDC_RECOVER_CALENDAR, checkCalendar);
    DDX_Control(pDX, IDC_RECOVER_TASKS, checkTasks);
    DDX_Control(pDX, IDC_RECOVER_MAIL, checkMail);
    DDX_Control(pDX, IDC_RECOVER_BRIEFCASE, checkBriefcase);
    #if defined(WIN32_PLATFORM_WFSP)
        DDX_Control(pDX, IDC_RECOVER_SPIN1, spinSyncType );
    #endif
    DDX_Control(pDX, IDC_RECOVER_LIST_SYNCTYPE, lstSyncType );

    #if defined(WIN32_PLATFORM_PSPC)
        DDX_Control(pDX, IDC_RECOVER_NOTES, checkNotes);
    #endif
}


BEGIN_MESSAGE_MAP(CRecoverSettings, CDialog)
    ON_BN_CLICKED(IDC_RECOVER_MAIL, &CRecoverSettings::OnBnClickedRecoverMail)
    ON_BN_CLICKED(IDC_RECOVER_CALENDAR, &CRecoverSettings::OnBnClickedRecoverCalendar)
    ON_BN_CLICKED(IDC_RECOVER_CONTACTS, &CRecoverSettings::OnBnClickedRecoverContacts)
    ON_BN_CLICKED(IDC_RECOVER_TASKS, &CRecoverSettings::OnBnClickedRecoverTasks)
    ON_BN_CLICKED(IDC_RECOVER_BRIEFCASE, &CRecoverSettings::OnBnClickedRecoverBriefcase)
    #if defined(WIN32_PLATFORM_PSPC)
        ON_BN_CLICKED(IDC_RECOVER_NOTES, &CRecoverSettings::OnBnClickedRecoverNotes)
    #endif
    ON_WM_SIZE()
    //ON_WM_ACTIVATE()
    ON_LBN_SELCHANGE(IDC_RECOVER_LIST_SYNCTYPE, &CRecoverSettings::OnLbnSelchangeRecoverListSynctype)
    ON_WM_CTLCOLOR()
    ON_WM_PAINT()

END_MESSAGE_MAP()

/*void CRecoverSettings::OnActivate(UINT nState, CWnd* pWndOther,BOOL bMinimized ){

    if(cleanupStarted){
        pWndOther->GetDlgItem(IDC_CLEANUP)->ShowWindow(SW_SHOW);
    }else{
        pWndOther->GetDlgItem(IDC_CLEANUP)->ShowWindow(SW_HIDE);
    }

}*/



// CRecoverSettings message handlers

BOOL CRecoverSettings::OnInitDialog()
{
    CString s1, s2;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER); SetWindowText(s1);
    CDialog::OnInitDialog();

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_RECOVER;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_RECOVER;
    #endif
    #endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    #ifdef WIN32_PLATFORM_WFSP
        spinSyncType.SetRange(2,0);	spinSyncType.SetBuddy(&lstSyncType);
    #endif

    // Hide the drop down list
    lstSyncType.ShowWindow(SW_HIDE);
    GetDlgItem(IDC_RECOVER_STATIC_DIRECTION)->ShowWindow(SW_HIDE);

#if 0
    // load string resources
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_SYNCTYPE2); lstSyncType.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_SYNCTYPE3); lstSyncType.AddString(s1);
    // add string for the cleanup only and set the cursor
    lstSyncType.AddString(L"cleanup");
    lstSyncType.SetCurSel(2);
#endif

    s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_ITEMS);   SetDlgItemText(IDC_RECOVER_STATIC, s2);

    #if defined(WIN32_PLATFORM_PSPC)
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_TITLE);   SetDlgItemText(IDC_RECOVER_STATIC_TITLE, s2);
    #endif


    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_CONTACTS);	SetDlgItemText(IDC_RECOVER_CONTACTS, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_CALENDAR);	SetDlgItemText(IDC_RECOVER_CALENDAR, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_TASKS);   	SetDlgItemText(IDC_RECOVER_TASKS, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_BRIEFCASE);   SetDlgItemText(IDC_RECOVER_BRIEFCASE, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DIRECTION);        SetDlgItemText(IDC_RECOVER_STATIC_DIRECTION, s1);

    // for cleanup
    SetDlgItemText(IDC_CLEANUP, TEXT(""));
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_MAIL);  SetDlgItemText(IDC_RECOVER_MAIL, s1);
    //*******
#if 0
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_MAIL);  s1+=" "; s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_EMAIL_SYNCTYPE);
    s1+=s2;
    SetDlgItemText(IDC_RECOVER_MAIL, s1);
#endif


    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_RECOVER_STATIC)->SetFont(&fontBold);

    #if defined(WIN32_PLATFORM_PSPC)
        GetDlgItem(IDC_RECOVER_STATIC_TITLE)->SetFont(&fontBold);
    #endif

    // check what sources are set to synctype none, and disable the checkboxes
    ClientSettings* cs = getRegConfig();

    if(!strcmp(cs->getConfigSourcesParameter("contact", "sync"),"none")){
        checkContacts.EnableWindow(FALSE);
    }

    if(!strcmp(cs->getConfigSourcesParameter("calendar", "sync"),"none")){
        checkCalendar.EnableWindow(FALSE);
    }

    if(!strcmp(cs->getConfigSourcesParameter("task", "sync"),"none")){
        checkTasks.EnableWindow(FALSE);
    }

    if(!strcmp(cs->getConfigSourcesParameter("briefcase", "sync"),"none")){
        checkBriefcase.EnableWindow(FALSE);
    }

    #if defined(WIN32_PLATFORM_PSPC)
    if(!strcmp(cs->getConfigSourcesParameter("note", "sync"),"none")){
        checkNotes.EnableWindow(FALSE);
    }
    #endif

    if(!strcmp(cs->getConfigSourcesParameter("mail", "sync"),"none") ||
        !doesFunambolAccountExist()){
       checkMail.EnableWindow(FALSE);
    }

    if(cs->getIsPortal()){
        checkBriefcase.EnableWindow(FALSE);
        checkBriefcase.ShowWindow(SW_HIDE);
        #if defined(WIN32_PLATFORM_PSPC)
            checkNotes.EnableWindow(FALSE);
            checkNotes.ShowWindow(SW_HIDE);
        #endif

#if 0
        GetDlgItem(IDC_RECOVER_STATIC_DIRECTION)->SetFont(&fontBold);
        GetDlgItem(IDC_RECOVER_STATIC_DIRECTION)->ShowWindow(SW_HIDE);
        lstSyncType.ShowWindow(SW_HIDE);
#endif

    }

    EnableRecoverButton(false);

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    checkMail.SetFocus();

    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_ITEMS); SetDlgItemText(IDC_RECOVER_STATIC, s1);
        GetDlgItem(IDC_RECOVER_STATIC)->SetFont(&fontBold);
        GetDlgItem(IDC_RECOVER_STATIC_TITLE)->SetFont(&fontBold);
    #endif


    return FALSE;
}

/**
* All the choiche are removed and remain only the one for the cleanup
* Actually should be the only used.
*/
void CRecoverSettings::OnOK(){

    wstring command;
    int nCheckedSources = 0;
    int firstSourceToSyncId = -1;
    vector<const wchar_t*> sourceToSync;

#if 0
    BOOL mailToRecover = FALSE;
    if(lstSyncType.GetCurSel() == 0) {
        command += L"refresh-from-server ";
        mailToRecover = TRUE;
    } else if(lstSyncType.GetCurSel() == 1) {
        command += L"refresh-from-client ";
        mailToRecover = FALSE;
    } else if(lstSyncType.GetCurSel() == 2) {
        command += L"cleanup ";
        mailToRecover = TRUE;
    }
#endif

    // put here only for the cleanup task
    BOOL mailToRecover = TRUE;

    if(checkMail.GetCheck() == BST_CHECKED && doesFunambolAccountExist()) {
        if (mailToRecover) {
            firstSourceToSyncId = SOURCE_MAIL;
            sourceToSync.push_back(MAILS_NAME);
            nCheckedSources++;
            // we need to know if this source is to be synced,
            // in the case when the sync is canceled, to display the appropriate text
            ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_MAIL, ITEM_STATE_TO_SYNC);
        }
    }

    if (checkContacts.GetCheck() == BST_CHECKED) {
        if(firstSourceToSyncId == -1) {             //not set
            firstSourceToSyncId = SOURCE_CONTACTS;
        }
        sourceToSync.push_back(CONTACTS_NAME);
        nCheckedSources ++;
        ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_CONTACTS, ITEM_STATE_TO_SYNC);
    }

    if(checkCalendar.GetCheck() == BST_CHECKED) {
        if(firstSourceToSyncId == -1) {             //not set
            firstSourceToSyncId = SOURCE_CALENDAR;
        }
        sourceToSync.push_back(APPOINTMENTS_NAME);
        nCheckedSources++;
        ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_CALENDAR, ITEM_STATE_TO_SYNC);
    }

    if(checkTasks.GetCheck() == BST_CHECKED) {
        if(firstSourceToSyncId == -1) {             //not set
            firstSourceToSyncId = SOURCE_TASKS;
        }
        sourceToSync.push_back(TASKS_NAME);
        nCheckedSources++;
        ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_TASKS, ITEM_STATE_TO_SYNC);
    }

#if defined(WIN32_PLATFORM_PSPC)
    if(checkNotes.GetCheck() == BST_CHECKED) {
        if(firstSourceToSyncId == -1) {            //not set
            firstSourceToSyncId = SOURCE_NOTES;
        }
        sourceToSync.push_back(NOTES_NAME);
        nCheckedSources++;
        ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_NOTES, ITEM_STATE_TO_SYNC);
    }
#endif

    if(checkBriefcase.GetCheck() == BST_CHECKED) {
        if(firstSourceToSyncId == -1){ //not set
            firstSourceToSyncId = SOURCE_BRIEFCASE;
        }
        sourceToSync.push_back(FILES_NAME);
        nCheckedSources++;
        ((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl().setState(SOURCE_BRIEFCASE, ITEM_STATE_TO_SYNC);
    }

    if(nCheckedSources != 0){
#if 0
        wstring sourceList = getCommandLine(sourceToSync);

        command += sourceList;

        // start recover sync
        int check  = checkStartSync();
        if(check == 0) {
            BOOL toCleanup = TRUE;
            BOOL toSync = TRUE;
            if (command.find(L"refresh-from-server") != wstring::npos) {
                toCleanup = FALSE;
                // hide recover window
                ShowWindow(SW_HIDE);
                if (checkRefreshFromServer(GetSafeHwnd(), sourceToSync, FALSE) != 0) {
                    toSync = FALSE;
                }
            } else if (command.find(L"refresh-from-client") != wstring::npos) {
                toCleanup = FALSE;
                // hide recover window
                ShowWindow(SW_HIDE);
            } else if (command.find(L"cleanup") != wstring::npos) {
                toSync = FALSE;
#endif

                /*
                * Create the Mutex of the sync to avoid to start the sync when the
                * clean up process is in progress. The mutex is closed when the
                * cleanup procedure is finished.
                */

                hMutex = NULL;  //Handle to mutex...
                hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );
                CString sss;
                switch(GetLastError()) {
                    case ERROR_SUCCESS:
                        LOG.debug("Mutex created.");
                        cleanupStarted = true;
                        break;
                    case ERROR_ALREADY_EXISTS:
                        if(!cleanupStarted){
                            LOG.info("Cleanup process: a sync is in progress. No cleanup possible...");
                            sss.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_SYNC_IN_PROG);
                            TimedMessageBox(GetSafeHwnd(), sss,
                                getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                                MB_OK | MB_ICONEXCLAMATION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10
                        }else{
                        }
                        // close the handle of the cleanup
                        if (hMutex) {
                            CloseHandle(hMutex);
                            //cleanupStarted = false;
                        }
                        return;
                    default:
                        LOG.error("Failed to create mutex");
                        // close the handle of the cleanup
                        if (hMutex) {
                            CloseHandle(hMutex);
                            cleanupStarted = false;
                        }
                        return;
                }

                EnableRecoverButton(false);
                EnableCancelButton(false);

                if (checkRefreshFromServer(GetSafeHwnd(), sourceToSync, TRUE) == 0) {

                    CString s1;
                    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_IN_PROG);
                    SetDlgItemText(IDC_CLEANUP, s1);

                    try {
                        GetDlgItem(IDC_CLEANUP)->SetFont(&fontBold);
                    } catch (...) {
                        LOG.debug("RecoverSettings: font not loaded");
                    }
                    disableCheckbox();
                    startCleanUp(sourceToSync);
                }
                else {
                    if (hMutex) {
                        CloseHandle(hMutex);
                        cleanupStarted = false;
                    }
                    enableCheckbox();
                    EnableRecoverButton(true);
                    EnableCancelButton(true);
                }
                GetActiveWindow()->Invalidate();
                GetActiveWindow()->UpdateWindow();
#if 0
            }

            if (toSync) {
                // set first source to sync
                ((CuiDlg*)AfxGetMainWnd())->setFirstSourceToSync(firstSourceToSyncId);
                // start sync
                ((CuiDlg*)AfxGetMainWnd())->refreshSourcePane(SOURCE_MAIL);
                ((CuiDlg*)AfxGetMainWnd())->StartSync(command.c_str());

            }
#endif
        }
    }


int CRecoverSettings::startCleanUp(vector<const wchar_t*> &sources) {
    CleanUpParams *par = new CleanUpParams(*this, sources, ((CuiDlg*)AfxGetMainWnd()));

    if ( !CreateThread(NULL, 0, cleanUpTask, (LPVOID)par, 0, NULL) ) {
        LOG.error("startCleanUp: error running thread.");
        return -1;
    }
    else {
        return 0;
}
}

/**
 * Clean up thread.
 */
static DWORD WINAPI cleanUpTask(LPVOID lpv) {

    CleanUpParams *par = (CleanUpParams *)lpv;

    if( !par ) {
        LOG.error("cleanUpTask: invalid parameters.");
        return 0;
    }

    SetCursor(LoadCursor(NULL, IDC_WAIT));

    //
    // Call the startsync process: "startsync.exe cleanup <source1>,<source2>..."
    //
    wstring cmdLine(TEXT("cleanup"));
    vector<const wchar_t*> sourceToSync;
    sourceToSync = par->srcRef;
    vector<const wchar_t*>::iterator it;
    int count = 0;
    for (it = sourceToSync.begin(); it != sourceToSync.end(); it++) {
        if (count == 0) { cmdLine += L" "; }
        else            { cmdLine += L","; }
        cmdLine += (*it);
        count++;
    }
    int pid = startProgram(SYNCAPP, cmdLine.c_str());

    // Wait till the end of the cleanup...
    int ret = waitProcess(pid, INFINITE);


    SetCursor(LoadCursor(NULL, IDC_ARROW));
    par->wndRef.SetDlgItemText(IDC_CLEANUP, TEXT(""));

    // read the queue list and remove the sources that are already clenedup.
    // set in the registry only the ones that are not cleaned up.
    wchar_t** queue = readQueueList();
    if (queue) {
        vector<const wchar_t*> sourceToSync;
        sourceToSync = par->srcRef;
        vector<const wchar_t*>::iterator it;
        bool found = false;
        wchar_t** newQueue = new wchar_t*[10];
        int i = 0;
        for (int count = 0; queue[count] != NULL; count++) {
            for (it = sourceToSync.begin(); it != sourceToSync.end(); it++) {
                if (wcscmp(queue[count], (*it)) == 0) {
                    found = true;
                    break;
                }
            }
            if (found) {
                if (sourceToSync.size() > 0) {
                    sourceToSync.erase(it);
                }
                found = false;
            } else {
                newQueue[i] = wstrdup(queue[count]);
                i++;
            }
        }
        newQueue[i] = NULL;
        delete [] queue;
        updateQueueList((const wchar_t**)newQueue);
        delete [] newQueue;
    }

    endCleanUpProcess(par->wndRef.GetSafeHwnd(), par->srcRef);

    // Release the mutex if the cleanup started
    if (hMutex) {
        CloseHandle(hMutex);
        cleanupStarted = false;
    }

    //par->mainDlg->refreshSourcePanes();
    par->wndRef.EndDialog(RECOVER_SYNC_STARTED);

    delete par;

    return 0;
}


#if defined(WIN32_PLATFORM_PSPC)
void CRecoverSettings::OnBnClickedRecoverNotes()
{
    EnableRecoverButton(isCheckboxChecked());
}
#endif


void CRecoverSettings::EnableRecoverButton(bool bEnable){
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB)
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, IDOK, (LPARAM)&tbi);

    if(bEnable)
        tbi.fsState = TBSTATE_ENABLED;
    else
        tbi.fsState &= ~TBSTATE_ENABLED;

    ::SendMessage (hwndMB, TB_SETBUTTONINFO, IDOK, (LPARAM)&tbi);
}

void CRecoverSettings::EnableCancelButton(bool bEnable){
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB)
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, IDCANCEL, (LPARAM)&tbi);

    if(bEnable)
        tbi.fsState = TBSTATE_ENABLED;
    else
        tbi.fsState &= ~TBSTATE_ENABLED;

    ::SendMessage (hwndMB, TB_SETBUTTONINFO, IDCANCEL, (LPARAM)&tbi);
}

void CRecoverSettings::OnBnClickedRecoverMail()
{
    EnableRecoverButton(isCheckboxChecked());
    //refreshSyncType();
}

void CRecoverSettings::OnBnClickedRecoverCalendar()
{
    EnableRecoverButton(isCheckboxChecked());
    //refreshSyncType();
}

void CRecoverSettings::OnBnClickedRecoverContacts()
{
    EnableRecoverButton(isCheckboxChecked());
    //refreshSyncType();
}

void CRecoverSettings::OnBnClickedRecoverTasks()
{
    EnableRecoverButton(isCheckboxChecked());
    //refreshSyncType();
}

void CRecoverSettings::OnBnClickedRecoverBriefcase()
{
    EnableRecoverButton(isCheckboxChecked());
    //refreshSyncType();
}

bool CRecoverSettings::isCheckboxChecked()
{
    bool value = false;
    value = (checkContacts.GetCheck() == BST_CHECKED) ||
            (checkCalendar.GetCheck() == BST_CHECKED) ||
            (checkTasks.GetCheck() == BST_CHECKED)    ||
            (checkBriefcase.GetCheck() == BST_CHECKED) ||
            (checkMail.GetCheck() == BST_CHECKED);

    #if defined(WIN32_PLATFORM_PSPC)
        value |= (checkNotes.GetCheck() == BST_CHECKED);
    #endif

    return value;
}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CRecoverSettings::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    int resId;
    CString s1,s2;

    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_RECOVER_WIDE; break;
    case DRA::Portrait:
        resId = IDD_UI_RECOVER; break;
    case DRA::Square:
        resId = IDD_UI_RECOVER_SQUARE; break;
    default:
        resId = IDD_UI_BRIEFCASE; break;
    };

    // If the resource ID is not changed, it's useless to re-layout the dialog.
    // This helps also to keep the correct strings if cleanup is running or not.
    if (oldResId == resId) {
        return;
    }

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
    oldResId = resId;

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_CONTACTS);	SetDlgItemText(IDC_RECOVER_CONTACTS, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_CALENDAR);	SetDlgItemText(IDC_RECOVER_CALENDAR, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_TASKS);   	SetDlgItemText(IDC_RECOVER_TASKS, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_ITEMS);   	SetDlgItemText(IDC_RECOVER_STATIC, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_BRIEFCASE);   SetDlgItemText(IDC_RECOVER_BRIEFCASE, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DIRECTION);           SetDlgItemText(IDC_RECOVER_STATIC_DIRECTION, s1);

    // for cleanup
    if (cleanupStarted)   s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_IN_PROG);
    else                  s1 = L"";
    SetDlgItemText(IDC_CLEANUP, s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_MAIL);  SetDlgItemText(IDC_RECOVER_MAIL, s1);
    //********
#if 0
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_MAIL);  s1+=" "; s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_EMAIL_SYNCTYPE);
    s1+=s2;
    SetDlgItemText(IDC_RECOVER_MAIL, s1);
#endif

    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_ITEMS); SetDlgItemText(IDC_RECOVER_STATIC, s1);
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_TITLE);      SetDlgItemText(IDC_RECOVER_STATIC_TITLE, s1);
        GetDlgItem(IDC_RECOVER_STATIC)->SetFont(&fontBold);
        GetDlgItem(IDC_RECOVER_STATIC_TITLE)->SetFont(&fontBold);
    #endif
}
#endif

wstring getCommandLine(vector<const wchar_t*> &sources) {

    wstring sourceList;
    vector<const wchar_t*>::iterator sourcesIterator;
    int i = 1;
    for (sourcesIterator = sources.begin();
            sourcesIterator != sources.end();
            sourcesIterator++) {

        sourceList.append(*sourcesIterator);
        if (sources.size() - i != 0) {
            sourceList.append(TEXT(","));
        }
        i++;
    }

    return sourceList;
}

int checkRefreshFromServer(HWND wndUI, vector<const wchar_t*> &sources,  BOOL cleanup) {

    bool isEmail = false;
    wstring sourceList;
    int i = 1, ret = 0;
    wchar_t t[512];
    vector<const wchar_t*>::iterator sourcesIterator;

    for (sourcesIterator = sources.begin();
            sourcesIterator != sources.end();
            sourcesIterator++) {

        sourceList.append(getFriendlyName(*sourcesIterator));
        if(wcscmp( TEXT("mail") , *sourcesIterator ) == 0 ){
            isEmail = true;
        }

        if (sources.size() - i == 1) {
            sourceList.append(TEXT(" "));
            sourceList.append(getLocalizationUtils()->getLocalizationString(IDS_AND));
            sourceList.append(TEXT(" "));

        } else if (sources.size() - i != 0) {
            sourceList.append(TEXT(", "));
        }
        i++;
    }

    if (cleanup) {
        wsprintf(t, getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_MSG),
             sourceList.c_str());
    } else {
    wsprintf(t, getLocalizationUtils()->getLocalizationString(IDS_RS_ARMSG1),
             sourceList.c_str());
    }

    wstring message = t;
    //if(cleanup && isEmail){
    //    message.append(getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_MSG2));
    //}

    if(cleanup){
        message.append(getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_MSG3));
    }

    HwndFunctions::closePreviousMsgBox();
    int val = TimedMessageBox(wndUI, message.c_str(), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                         MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    if (val == IDYES) {
        LOG.debug("User said yes to delete local %S and refresh.", sourceList.c_str());
    }
    else {
        LOG.debug("User said no to delete local %S and refresh." , sourceList.c_str());
        ret = -1;
    }

    return ret;
}

/**
* Message used after the clean up process
*/
int endCleanUpProcess(HWND wndUI, vector<const wchar_t*> &sources) {

    wchar_t t[512];
    int i = 1;
    wstring sourceList;
    vector<const wchar_t*>::iterator sourcesIterator;

    for (sourcesIterator = sources.begin();
            sourcesIterator != sources.end();
            sourcesIterator++) {

        sourceList.append(getFriendlyName(*sourcesIterator));

        if (sources.size() - i == 1) {
            sourceList.append(TEXT(" "));
            sourceList.append(getLocalizationUtils()->getLocalizationString(IDS_AND));
            sourceList.append(TEXT(" "));

        } else if (sources.size() - i != 0) {
            sourceList.append(TEXT(", "));
        }
        i++;
    }

    wsprintf(t, getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_COMPLETE_MSG),
         sourceList.c_str());

    HwndFunctions::closePreviousMsgBox();
    int val = TimedMessageBox(wndUI, t, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                         MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    return 0;
}

void CRecoverSettings::OnLbnSelchangeRecoverListSynctype(){
    ClientSettings* cs = getRegConfig();

    // if replace server data is selected
    if(lstSyncType.GetCurSel() == 1){
        checkMail.SetCheck(BST_UNCHECKED);
        checkMail.EnableWindow(FALSE);
    }
    else{
        if( strcmp(cs->getConfigSourcesParameter("mail", "sync"),"none") &&
            doesFunambolAccountExist())
           checkMail.EnableWindow(TRUE);
    }
}

HBRUSH CRecoverSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_RECOVER_STATIC_TITLE) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}

/**
* It disables all the checkbox to avoid they are pressed when the
* cleanup is in progress
*/
void CRecoverSettings::disableCheckbox() {

    checkContacts.EnableWindow(FALSE);
    checkCalendar.EnableWindow(FALSE);
    checkTasks.EnableWindow(FALSE);
    checkBriefcase.EnableWindow(FALSE);
    checkMail.EnableWindow(FALSE);

    #if defined(WIN32_PLATFORM_PSPC)
        checkNotes.EnableWindow(FALSE);
    #endif
}

void CRecoverSettings::enableCheckbox() {

    ClientSettings* cs = getRegConfig();

    if(!strcmp(cs->getConfigSourcesParameter("contact", "sync"),"none")){
        checkContacts.EnableWindow(FALSE);
    }

    if(!strcmp(cs->getConfigSourcesParameter("calendar", "sync"),"none")){
        checkCalendar.EnableWindow(FALSE);
    }

    if(!strcmp(cs->getConfigSourcesParameter("task", "sync"),"none")){
        checkTasks.EnableWindow(FALSE);
    }

    if(!strcmp(cs->getConfigSourcesParameter("briefcase", "sync"),"none")){
        checkBriefcase.EnableWindow(FALSE);
    }

    #if defined(WIN32_PLATFORM_PSPC)
    if(!strcmp(cs->getConfigSourcesParameter("note", "sync"),"none")){
        checkNotes.EnableWindow(FALSE);
    }
    #endif

    if(!strcmp(cs->getConfigSourcesParameter("mail", "sync"),"none") ||
        !doesFunambolAccountExist()){
       checkMail.EnableWindow(FALSE);
    }

    if(cs->getIsPortal()){
        checkBriefcase.EnableWindow(FALSE);
        checkBriefcase.ShowWindow(SW_HIDE);
        #if defined(WIN32_PLATFORM_PSPC)
            checkNotes.EnableWindow(FALSE);
            checkNotes.ShowWindow(SW_HIDE);
        #endif
    }
}


void CRecoverSettings::OnPaint() {

    // Set the cursor if cleanup is running or not: we could have switched to
    // another window, so we set again the cursor.
    if (cleanupStarted)  SetCursor(LoadCursor(NULL, IDC_WAIT ));
    else                 SetCursor(LoadCursor(NULL, IDC_ARROW));

    CDialog::OnPaint();
}