/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// SyncSettings.cpp : implementation file
//

#include "stdafx.h"
#include "SyncSettings.h"

#include "PIMSettings.h"
#include "ContactSettings.h"
#include "CalendarSettings.h"
#include "TaskSettings.h"
#include "MailSettings.h"
#include "BriefcaseSettings.h"
#include "pim/ClientSettings.h"
#include "SyncSettingsAdvanced.h"
#include "SyncMethod.h"

#if defined(WIN32_PLATFORM_PSPC)
#include "NotesSettings.h"
#endif

#include "aygshell.h"
#include "pim/maincpp.h"
#include "pim/account.h"

// CSyncSettings dialog

#define ADVANCED_CONTACTS 1
#define ADVANCED_CALENDAR 2
#define ADVANCED_TASKS 4
#define ADVANCED_MAIL 8
#define ADVANCED_BRIEFCASE 16
#define ADVANCED_NOTES 32
#define ADVANCED_PIM        7       // 1 + 2 + 4

IMPLEMENT_DYNAMIC(CSyncSettings, CDialog)

CSyncSettings::CSyncSettings(CWnd* pParent /*=NULL*/)
: CDialog(CSyncSettings::IDD, pParent)
{
    paddingVal = 100;
}

CSyncSettings::~CSyncSettings()
{
}

void CSyncSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_SYNC_CONTACTS, checkContacts);
    DDX_Control(pDX, IDC_SYNC_CALENDAR, checkCalendar);
    DDX_Control(pDX, IDC_SYNC_TASKS, checkTasks);
    DDX_Control(pDX, IDC_SYNC_MAIL, checkMail);
    DDX_Control(pDX, IDC_SYNC_BRIEFCASE, checkBriefcase);
#if defined(WIN32_PLATFORM_PSPC)
    DDX_Control(pDX, IDC_SYNC_NOTES, checkNotes);
  //  DDX_Control(pDX, IDC_SYNC_BUTTON_NOTES, butNotes);
#endif
    DDX_Control(pDX, IDC_SYNC_BUTTON_MAIL, butMail);
//    DDX_Control(pDX, IDC_SYNC_BUTTON_BRIEFCASE, butBriefcase);
    DDX_Control(pDX, IDC_SYNC_BUTTON_PIM_OPTIONS, butPimOptions);
DDX_Control(pDX, IDC_SYNC_LINK_ADVANCED, linkAdvanced);
}


BEGIN_MESSAGE_MAP(CSyncSettings, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE)
    ON_WM_SIZE()
#endif
    ON_WM_CTLCOLOR()
    ON_WM_PAINT()
    ON_BN_SETFOCUS(IDC_SYNC_CONTACTS, &CSyncSettings::OnSetfocusSyncContacts)
    ON_BN_SETFOCUS(IDC_SYNC_CALENDAR, &CSyncSettings::OnBnSetfocusSyncCalendar)
    ON_BN_SETFOCUS(IDC_SYNC_TASKS, &CSyncSettings::OnBnSetfocusSyncTasks)
    ON_BN_SETFOCUS(IDC_SYNC_MAIL, &CSyncSettings::OnBnSetfocusSyncMail)
    ON_BN_SETFOCUS(IDC_SYNC_BRIEFCASE, &CSyncSettings::OnBnSetfocusSyncBriefcase)
    ON_BN_SETFOCUS(IDC_SYNC_LINK_ADVANCED, &CSyncSettings::OnBnSetfocusSyncLinkAdvanced)
    ON_WM_INITMENUPOPUP()

#if defined(WIN32_PLATFORM_PSPC)
    ON_BN_SETFOCUS(IDC_SYNC_NOTES, &CSyncSettings::OnBnSetfocusSyncNotes)
#endif

    ON_BN_CLICKED(IDC_SYNC_CONTACTS, &CSyncSettings::OnBnClickedSyncContacts)
    ON_BN_CLICKED(IDC_SYNC_CALENDAR, &CSyncSettings::OnBnClickedSyncCalendar)
    ON_BN_CLICKED(IDC_SYNC_TASKS, &CSyncSettings::OnBnClickedSyncTasks)
    ON_BN_CLICKED(IDC_SYNC_MAIL, &CSyncSettings::OnBnClickedSyncMail)
    ON_BN_CLICKED(IDC_SYNC_BRIEFCASE, &CSyncSettings::OnBnClickedSyncBriefcase)
    //ON_COMMAND(IDADVANCED, &CSyncSettings::showAdvanced)
    ON_BN_CLICKED(IDC_SYNC_BUTTON_MAIL, &CSyncSettings::OnClickedSyncButtonMail)
    ON_BN_CLICKED(IDC_SYNC_BUTTON_BRIEFCASE, &CSyncSettings::OnClickedSyncButtonBriefcase)
    ON_BN_CLICKED(IDC_SYNC_BUTTON_PIM_OPTIONS, &CSyncSettings::OnClickedSyncButtonPimOptions)
#if defined(WIN32_PLATFORM_PSPC)
    ON_BN_CLICKED(IDC_SYNC_NOTES, &CSyncSettings::OnBnClickedSyncNotes)
    ON_BN_CLICKED(IDC_SYNC_BUTTON_NOTES, &CSyncSettings::OnClickedSyncButtonNotes)
#endif
    ON_BN_CLICKED(IDC_SYNC_LINK_ADVANCED, &CSyncSettings::OnBnClickedSyncLinkAdvanced)

    ON_COMMAND(ID_EMAIL_OPTIONS, &CSyncSettings::OnEmailOptionsClicked)
    ON_COMMAND(ID_PIM_OPTIONS,   &CSyncSettings::OnPIMOptionsClicked)
    ON_COMMAND(ID_SYNC_METHOD,   &CSyncSettings::OnSyncMethodClicked)
    ON_COMMAND(ID_MENU_ADVANCED,   &CSyncSettings::OnAdvanced)
END_MESSAGE_MAP()

void CSyncSettings::OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex,BOOL bSysMenu)
{

    if(checkMail.GetCheck()==BST_CHECKED){
        pPopupMenu->EnableMenuItem(ID_EMAIL_OPTIONS,MF_ENABLED);
    }else{
        pPopupMenu->EnableMenuItem(ID_EMAIL_OPTIONS,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
    }

    bool flag = false;
    if(checkContacts.GetCheck() == BST_CHECKED && !flag){
        flag = true;
    }
    if(checkCalendar.GetCheck() == BST_CHECKED && !flag){
        flag = true;
    }
    if(checkTasks.GetCheck() == BST_CHECKED && !flag){
        flag = true;
    }
    if(!(getRegConfig()->getIsPortal())){
        if(checkBriefcase.GetCheck() == BST_CHECKED && !flag){
            flag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED && !flag){
            flag = true;
        }
#endif
    }

    if(flag){
        pPopupMenu->EnableMenuItem(ID_PIM_OPTIONS,MF_ENABLED);
    }else{
        pPopupMenu->EnableMenuItem(ID_PIM_OPTIONS,MF_BYCOMMAND|MF_DISABLED|MF_GRAYED);
    }
}

// CSyncSettings message handlers
BOOL CSyncSettings::OnInitDialog()
{
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();

    butMail.ShowWindow(TRUE);
    //butBriefcase.ShowWindow(TRUE);

    butPimOptions.ShowWindow(TRUE);

#if defined(WIN32_PLATFORM_PSPC)
  //  butNotes.ShowWindow(TRUE);
#endif

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    if(getRegConfig()->getIsPortal()){
        dlgCommandBar.nToolBarId = IDM_SYNC_SETTINGS_NO_ADV;
    }else{
        dlgCommandBar.nToolBarId = IDM_SYNC_SETTINGS;
    }

#if defined WIN32_PLATFORM_WFSP
#if (_WIN32_WCE < 0x500)
    dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
#endif
#endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    // load strings
    LoadStringResources();

    GetDlgItem(IDC_SYNC_STATIC_ITEMS)->SetFont(&fontBold);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ADVANCED_SETTINGS);
    linkAdvanced.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL_OPTIONS);
    butMail.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_OPTIONS);
    butPimOptions.setText(s1);

//    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE_OPTIONS);
//    butBriefcase.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_METHOD_ABB);
    linkAdvanced.setText(s1);

    int dim = s1.GetLength();

    //CRect advancedRect;
    //CRect syncMethodRect;

    //GetDlgItem(IDC_SYNC_LINK_ADVANCED)->GetWindowRect(&advancedRect);
    //GetDlgItem(IDC_PIM_GROUPBOX)->GetWindowRect(&syncMethodRect);


    //linkAdvanced.SetWindowPos(&CWnd::wndTop, advancedRect.TopLeft().x,
    //    advancedRect.BottomRight().y, dim*15, advancedRect.Height(),SWP_NOMOVE);
    //linkAdvanced.SetWindowPos(&CWnd::wndTop, 5,syncMethodRect.bottom,
    //    dim*7, advancedRect.Height(),SWP_NOZORDER);


    //s1 = getActiveSyncMethod();
    //int dimSyncMethod = s1.GetLength();

    //GetDlgItem(IDC_SYNC_METHOD_LABEL)->SetWindowPos(&CWnd::wndTop,
    //    (dim*7)+2, syncMethodRect.bottom+2,
    //    dimSyncMethod*7,advancedRect.Height(),SWP_NOZORDER);


#if defined(WIN32_PLATFORM_PSPC)
  //  s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES_OPTIONS);
  //  butNotes.setText(s1);
#endif


    // Hide groupbox, it's used only to know the position for the bottom line
    GetDlgItem(IDC_PIM_GROUPBOX)->ShowWindow(SW_HIDE);


    ClientSettings* cs = getRegConfig();
    if( strcmp(cs->getConfigSourcesParameter("contact","sync"), "none")){
        checkContacts.SetCheck(BST_CHECKED);
    }
    else {
        checkContacts.SetCheck(BST_UNCHECKED);
    }

    if(strcmp(cs->getConfigSourcesParameter("calendar","sync"), "none")) {
        checkCalendar.SetCheck(BST_CHECKED);
    }
    else {
        checkCalendar.SetCheck(BST_UNCHECKED);
    }


    checkTasks.EnableWindow(TRUE);
    if( strcmp(cs->getConfigSourcesParameter("task","sync"), "none")){
        checkTasks.SetCheck(BST_CHECKED);
    }
    else{
        checkTasks.SetCheck(BST_UNCHECKED);
    }

    if (doesFunambolAccountExist()) {
        // Funambol mail account exists
        checkMail.EnableWindow(TRUE);
        if( strcmp(cs->getConfigSourcesParameter("mail","sync"), "none")){
            checkMail.SetCheck(BST_CHECKED);
        }
        else{
            checkMail.SetCheck(BST_UNCHECKED);
            butMail.EnableWindow(FALSE);
        };
    } else {
        // Funambol mail account DOES NOT exist
        checkMail.EnableWindow(FALSE);
        butMail.EnableWindow(FALSE);
        checkMail.SetCheck(BST_UNCHECKED);
    }

    if (!cs->getIsPortal()) {
        checkBriefcase.EnableWindow(TRUE);
        checkBriefcase.ShowWindow(SW_SHOW);
       // butBriefcase.ShowWindow(SW_SHOW);
        if( strcmp(cs->getConfigSourcesParameter("briefcase","sync"), "none"))
            checkBriefcase.SetCheck(BST_CHECKED);
        else
        {
            checkBriefcase.SetCheck(BST_UNCHECKED);
            //butBriefcase.EnableWindow(FALSE);
        }
    } else {
        checkBriefcase.EnableWindow(FALSE);
        checkBriefcase.SetCheck(BST_UNCHECKED);
        checkBriefcase.ShowWindow(SW_HIDE);
        //butBriefcase.EnableWindow(FALSE);
        //butBriefcase.ShowWindow(SW_HIDE);
    }

#if defined(WIN32_PLATFORM_PSPC)
    if (!cs->getIsPortal()) {
        checkNotes.EnableWindow(TRUE);
        checkNotes.ShowWindow(SW_SHOW);
      //  butNotes.ShowWindow(SW_SHOW);
        if( strcmp(cs->getConfigSourcesParameter("note","sync"), "none"))
            checkNotes.SetCheck(BST_CHECKED);
        else
        {
            checkNotes.SetCheck(BST_UNCHECKED);
          //  butNotes.EnableWindow(FALSE);
        }
    }  else {
        checkNotes.EnableWindow(FALSE);
        checkNotes.SetCheck(BST_UNCHECKED);
        checkNotes.ShowWindow(SW_HIDE);
      //  butNotes.EnableWindow(FALSE);
      //  butNotes.ShowWindow(SW_HIDE);
    }
#endif
/*
    if (checkContacts.GetCheck() == BST_UNCHECKED &&
        checkCalendar.GetCheck() == BST_UNCHECKED &&
        checkTasks.GetCheck()    == BST_UNCHECKED ) {
        butPimOptions.EnableWindow(FALSE);
    }*/

        bool pimButFlag = false;
    if(checkContacts.GetCheck() == BST_CHECKED && !pimButFlag){
        pimButFlag = true;
    }
    if(checkCalendar.GetCheck() == BST_CHECKED && !pimButFlag){
        pimButFlag = true;
    }
    if(checkTasks.GetCheck() == BST_CHECKED && !pimButFlag){
        pimButFlag = true;
    }
    if(!(getRegConfig()->getIsPortal())){
        if(checkBriefcase.GetCheck() == BST_CHECKED && !pimButFlag){
            pimButFlag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED && !pimButFlag){
            pimButFlag = true;
        }
#endif
    }


    if (!pimButFlag){
        butPimOptions.EnableWindow(FALSE);
    }



    source = TEXT("");
    advanced = 0;
    checkedSources = 0;

    Invalidate();
    UpdateWindow();

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    checkMail.SetFocus();

    return FALSE;
}



HBRUSH CSyncSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}





void CSyncSettings::OnSetfocusSyncContacts()
{
    if(checkContacts.GetCheck() == BST_CHECKED) {
        source=TEXT("contacts");
    }
}

void CSyncSettings::OnBnClickedSyncContacts()
{
    if(checkContacts.GetCheck() == BST_CHECKED) {
        source=TEXT("contacts");
    }

    bool flag = true;
    if (checkContacts.GetCheck() == BST_UNCHECKED &&
        checkCalendar.GetCheck() == BST_UNCHECKED &&
        checkTasks.GetCheck()    == BST_UNCHECKED ) {
            flag = false;
    }
    if (!getRegConfig()->getIsPortal()){
        if(checkBriefcase.GetCheck() == BST_CHECKED){
            flag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED){
            flag = true;
        }
#endif
    }
    butPimOptions.EnableWindow(flag);


}

void CSyncSettings::OnBnSetfocusSyncCalendar()
{
    if(checkCalendar.GetCheck() == BST_CHECKED) {
        source=TEXT("calendar");
    }
}

void CSyncSettings::OnBnSetfocusSyncTasks()
{
    if(checkTasks.GetCheck() == BST_CHECKED) {
        source=TEXT("tasks");
    }
}

void CSyncSettings::OnBnSetfocusSyncMail()
{
    if(checkMail.GetCheck() == BST_CHECKED) {
        source=TEXT("mail");
    }
}

void CSyncSettings::OnBnSetfocusSyncBriefcase()
{
    if(checkBriefcase.GetCheck() == BST_CHECKED) {
        source=TEXT("briefcase");
    }
}

void CSyncSettings::OnBnClickedSyncCalendar()
{
    if(checkCalendar.GetCheck() == BST_CHECKED) {
        source=TEXT("calendar");
    }

    bool flag = true;
    if (checkContacts.GetCheck() == BST_UNCHECKED &&
        checkCalendar.GetCheck() == BST_UNCHECKED &&
        checkTasks.GetCheck()    == BST_UNCHECKED ) {
            flag = false;
    }
    if (!getRegConfig()->getIsPortal()){
        if(checkBriefcase.GetCheck() == BST_CHECKED){
            flag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED){
            flag = true;
        }
#endif
    }
    butPimOptions.EnableWindow(flag);

}

void CSyncSettings::OnBnClickedSyncTasks()
{
    if(checkTasks.GetCheck() == BST_CHECKED) {
        source=TEXT("tasks");
    }

    bool flag = true;
    if (checkContacts.GetCheck() == BST_UNCHECKED &&
        checkCalendar.GetCheck() == BST_UNCHECKED &&
        checkTasks.GetCheck()    == BST_UNCHECKED ) {
            flag = false;
    }
    if (!getRegConfig()->getIsPortal()){
        if(checkBriefcase.GetCheck() == BST_CHECKED){
            flag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED){
            flag = true;
        }
#endif
    }
    butPimOptions.EnableWindow(flag);

}

void CSyncSettings::OnBnClickedSyncMail()
{
    if(checkMail.GetCheck() == BST_UNCHECKED)
        butMail.EnableWindow(FALSE);
    else
        butMail.EnableWindow(TRUE);

    if(checkMail.GetCheck() == BST_CHECKED) {
        source=TEXT("mail");
    }
}

void CSyncSettings::OnBnClickedSyncBriefcase()
{
   // if(checkBriefcase.GetCheck() == BST_UNCHECKED)
       // butBriefcase.EnableWindow(FALSE);
  //  else
       // butBriefcase.EnableWindow(TRUE);

    if(checkBriefcase.GetCheck() == BST_CHECKED) {
        source=TEXT("briefcase");
    }

        bool flag = true;
    if (checkContacts.GetCheck() == BST_UNCHECKED &&
        checkCalendar.GetCheck() == BST_UNCHECKED &&
        checkTasks.GetCheck()    == BST_UNCHECKED ) {
            flag = false;
    }
    if (!getRegConfig()->getIsPortal()){
        if(checkBriefcase.GetCheck() == BST_CHECKED){
            flag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED){
            flag = true;
        }
#endif
    }
    butPimOptions.EnableWindow(flag);

}

void CSyncSettings::OnAdvanced(){

    CSyncSettingsAdvanced wnd;

    INT_PTR result = wnd.DoModal();

}
void CSyncSettings::showAdvanced()
{
    if(source == "pim"){
        CPIMSettings wndPimSettings;
        INT_PTR result = wndPimSettings.DoModal();
        if(result == IDOK) {
            advanced |= ADVANCED_PIM;
    }
        butPimOptions.SetFocus();
    }

    if(source == "mail"){
        CMailSettings wndSett;
        INT_PTR result = wndSett.DoModal();
        if(result == IDOK)
            advanced |= ADVANCED_MAIL;

        butMail.SetFocus();
    }

    if(source == "briefcase"){
        CBriefcaseSettings wndSett;
        INT_PTR result = wndSett.DoModal();
        if(result == IDOK)
            advanced |= ADVANCED_BRIEFCASE;

        //butBriefcase.SetFocus();
    }

#if defined(WIN32_PLATFORM_PSPC)
    if(source == "note"){
        CNotesSettings wndSett;
        INT_PTR result = wndSett.DoModal();
        if(result == IDOK)
            advanced |= ADVANCED_NOTES;

      //  butNotes.SetFocus();
    }
#endif
}


void CSyncSettings::OnOK(){
    ClientSettings* cs = getRegConfig();
    BOOL dirtyFlag=FALSE;

    if(checkStartSync()){
        CDialog::OnOK();
    }

    if( (checkContacts.GetCheck() == BST_UNCHECKED)){
        // save 'none'
        if(strcmp(cs->getConfigSourcesParameter("contact","sync"),"none")!=0){
            cs->setConfigSourcesParameter("contact", "sync","none");
            dirtyFlag=TRUE;
        }

    }
    else{
        if( ( ~advanced | ADVANCED_CONTACTS) &&
            (!strcmp(cs->getConfigSourcesParameter("contact", "sync"),"none")) )
        {
            // save 'two-way'
            if(strcmp(cs->getConfigSourcesParameter("contact","sync"),"two-way")!=0){
                cs->setConfigSourcesParameter("contact", "sync","two-way");
                dirtyFlag=TRUE;
            }
        }
    }

    if( (checkCalendar.GetCheck() == BST_UNCHECKED))
    {
        // save 'none'
        if(strcmp(cs->getConfigSourcesParameter("calendar","sync"),"none")!=0){
            cs->setConfigSourcesParameter("calendar", "sync","none");
            dirtyFlag=TRUE;
        }
    }
    else
    {
        if( (! (advanced & ADVANCED_CALENDAR)) &&
            (!strcmp(cs->getConfigSourcesParameter("calendar", "sync"),"none")) )
        {
            // save 'two-way'
            if(strcmp(cs->getConfigSourcesParameter("calendar","sync"),"two-way")!=0){
                cs->setConfigSourcesParameter("calendar", "sync","two-way");
                dirtyFlag=TRUE;
            }
        }
    }

    if((checkTasks.GetCheck() == BST_UNCHECKED))
    {
        // save 'none'
        if(strcmp(cs->getConfigSourcesParameter("task","sync"),"none")!=0){
            cs->setConfigSourcesParameter("task", "sync","none");
            dirtyFlag=TRUE;
        }
    }
    else
    {
        if( (! (advanced & ADVANCED_TASKS)) &&
            (!strcmp(cs->getConfigSourcesParameter("task", "sync"),"none")) )
        {
            // save 'two-way'
            if(strcmp(cs->getConfigSourcesParameter("task","sync"),"two-way")!=0){
                cs->setConfigSourcesParameter("task", "sync","two-way");
                dirtyFlag=TRUE;
            }
        }
    }

    if( (checkMail.GetCheck() == BST_UNCHECKED || !doesFunambolAccountExist()))
    {
        // save 'none'
        if(strcmp(cs->getConfigSourcesParameter("mail","sync"),"none")!=0){
            cs->setConfigSourcesParameter("mail", "sync","none");
            dirtyFlag=TRUE;
        }
    }
    else
    {
        // There's no (! (advanced & ADVANCED_MAIL)) condition in this if because
        // in the mail advance settings there's no a sync type setting to set
        if( (!strcmp(cs->getConfigSourcesParameter("mail", "sync"),"none")) )
        {
            // save 'two-way'
            if(strcmp(cs->getConfigSourcesParameter("mail","sync"),"two-way")!=0){
                cs->setConfigSourcesParameter("mail", "sync","two-way");
                dirtyFlag=TRUE;
            }
        }
    }

    if( (checkBriefcase.GetCheck() == BST_UNCHECKED))
    {
        // save 'none'
        if(strcmp(cs->getConfigSourcesParameter("briefcase","sync"),"none")!=0){
            cs->setConfigSourcesParameter("briefcase", "sync","none");
            dirtyFlag=TRUE;
        }
    }
    else
    {
        if( (! (advanced & ADVANCED_BRIEFCASE)) &&
            (!strcmp(cs->getConfigSourcesParameter("briefcase", "sync"),"none")) )
        {
            // save 'two-way'
            if(strcmp(cs->getConfigSourcesParameter("briefcase","sync"),"two-way")!=0){
                cs->setConfigSourcesParameter("briefcase", "sync","two-way");
                dirtyFlag=TRUE;
            }
        }
    }
#if defined(WIN32_PLATFORM_PSPC)
    if( (checkNotes.GetCheck() == BST_UNCHECKED))
    {
        // save 'none'
        if(strcmp(cs->getConfigSourcesParameter("note","sync"),"none")!=0){
            cs->setConfigSourcesParameter("note", "sync","none");
            dirtyFlag=TRUE;
        }
    }
    else
    {
        if( (! (advanced & ADVANCED_NOTES)) &&
            (!strcmp(cs->getConfigSourcesParameter("note", "sync"),"none")) )
        {
            // save 'two-way'
            if(strcmp(cs->getConfigSourcesParameter("note","sync"),"two-way")!=0){
                cs->setConfigSourcesParameter("note", "sync","two-way");
                dirtyFlag=TRUE;
            }
        }
    }
#endif

    //getRegConfig()->setDirty(CS_DIRTY_SOURCE_ALL);
    if(dirtyFlag){
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_ENABLED);
        getRegConfig()->saveDirty();
    }
    CDialog::OnOK();
}

#if defined(_DEVICE_RESOLUTION_AWARE)
void CSyncSettings::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/){
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_METHOD_ABB);
#if defined(WIN32_PLATFORM_PSPC)
    int resId;
    BOOL showScroll = FALSE;
    if(s1.GetLength()>13){
        paddingVal = 130;
    }else{
        paddingVal = 100;
    }
    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_SYNC_WIDE;
        showScroll = TRUE;
        break;
    case DRA::Portrait:
        resId = IDD_UI_SYNC; break;
        showScroll = FALSE;
        break;
    case DRA::Square:
        resId = IDD_UI_SYNC_SQUARE; break;
        showScroll = TRUE;
        break;
    default:
        resId = IDD_UI_SYNC; break;
        showScroll = FALSE;
        break;

    };
        DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
#else
    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
    if(s1.GetLength()>13){
            paddingVal = 140;
        }else{
            paddingVal = 100;
        }
        break;
    case DRA::Portrait:
        if(s1.GetLength()>13){
            paddingVal = 130;
        }else{
            paddingVal = 90;
        }
        break;
    default:
        paddingVal = 6;
        break;

    };
#endif



    LoadStringResources();
}
#endif

void CSyncSettings::OnClickedSyncButtonMail(){
    source = "mail";
    showAdvanced();
}


void CSyncSettings::OnClickedSyncButtonBriefcase(){
    source = "briefcase";
    showAdvanced();
}


void CSyncSettings::OnClickedSyncButtonPimOptions(){
    source = "pim";
    showAdvanced();
}


#if defined(WIN32_PLATFORM_PSPC)
void CSyncSettings::OnBnClickedSyncNotes(){
//    if(checkNotes.GetCheck() == BST_UNCHECKED)
    //    butNotes.EnableWindow(FALSE);
//    else
    //    butNotes.EnableWindow(TRUE);

    if(checkNotes.GetCheck() == BST_CHECKED) {
        source=TEXT("note");
    }
        bool flag = true;
    if (checkContacts.GetCheck() == BST_UNCHECKED &&
        checkCalendar.GetCheck() == BST_UNCHECKED &&
        checkTasks.GetCheck()    == BST_UNCHECKED ) {
            flag = false;
    }
    if (!getRegConfig()->getIsPortal()){
        if(checkBriefcase.GetCheck() == BST_CHECKED){
            flag = true;
        }
#if defined(WIN32_PLATFORM_PSPC)
        if(checkNotes.GetCheck() == BST_CHECKED){
            flag = true;
        }
#endif
    }
    butPimOptions.EnableWindow(flag);

}


void CSyncSettings::OnClickedSyncButtonNotes(){
    source = "note";
    showAdvanced();
}
#endif


void CSyncSettings::LoadStringResources(){
    CString s1;

    // load strings
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL); SetDlgItemText(IDC_SYNC_MAIL, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CONTACTS); SetDlgItemText(IDC_SYNC_CONTACTS, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CALENDAR); SetDlgItemText(IDC_SYNC_CALENDAR,s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TASKS); SetDlgItemText(IDC_SYNC_TASKS, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE); SetDlgItemText(IDC_SYNC_BRIEFCASE, s1);

#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES);
    SetDlgItemText(IDC_SYNC_NOTES, s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_SETTINGS);
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_ITEMS); SetDlgItemText(IDC_SYNC_STATIC_ITEMS, s1);

    // Read sync method from config
    SetDlgItemText(IDC_SYNC_METHOD_LABEL, getActiveSyncMethod());
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_METHOD_ABB);

    CRect advancedRect;
    CRect syncMethodRect;

    GetDlgItem(IDC_SYNC_LINK_ADVANCED)->GetWindowRect(&advancedRect);
    GetDlgItem(IDC_PIM_GROUPBOX)->GetWindowRect(&syncMethodRect);

   linkAdvanced.SetWindowPos(&CWnd::wndTop, 3,syncMethodRect.bottom - 20,
        paddingVal, advancedRect.Height(),SWP_NOZORDER);


    s1 = getActiveSyncMethod();
    int dimSyncMethod = s1.GetLength();

    GetDlgItem(IDC_SYNC_METHOD_LABEL)->SetWindowPos(&CWnd::wndTop,
        (paddingVal)+4, syncMethodRect.bottom - 20,
        dimSyncMethod*8,advancedRect.Height(),SWP_NOZORDER);


}


#if defined(WIN32_PLATFORM_PSPC)
void CSyncSettings::OnBnSetfocusSyncNotes(){
    if(checkNotes.GetCheck() == BST_CHECKED) {
        source=TEXT("note");
    }
}
#endif

// user clicked 'Sync Method' link
void CSyncSettings::OnBnClickedSyncLinkAdvanced(){

    CSyncMethod wndSyncAdvanced;
    INT_PTR result = wndSyncAdvanced.DoModal();

    // Reload resources, this is necessary to correctly
    // display the Sync method used.
    LoadStringResources();
}

// capture pressing of 'Advanced' link
BOOL CSyncSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            if(processKeyEvents(pMsg->wParam)){
                bProcessed = true;
            }
        };

        // left/right/down arrow keys
        if( (pMsg->wParam == VK_LEFT) || (pMsg->wParam == VK_RIGHT) ||
            (pMsg->wParam == VK_DOWN) || (pMsg->wParam == VK_UP)){
                if(processKeyEvents(pMsg->wParam)){
                    bProcessed = true;
                }
        };

    };

    if(bProcessed){
        return TRUE;
    }
    else{
        return CDialog::PreTranslateMessage(pMsg);
    }
}

void CSyncSettings::OnBnSetfocusSyncLinkAdvanced(){
}

/*bool CSyncSettings::isFocusOnSourceCheckbox(){
HWND wndFocus = GetFocus()->GetSafeHwnd();
bool hasFocus = false;

if( (wndFocus == checkMail.GetSafeHwnd()) ||  (wndFocus == checkContacts.GetSafeHwnd()) ||
(wndFocus == checkCalendar.GetSafeHwnd()) || (wndFocus == checkTasks.GetSafeHwnd()) ||
(wndFocus == checkBriefcase.GetSafeHwnd()) ){
hasFocus = true;
}

#if defined(WIN32_PLATFORM_PSPC)
if(wndFocus == checkNotes.GetSafeHwnd()){
hasFocus = true;
}
#endif

return hasFocus;
}*/

bool CSyncSettings::processKeyEvents(WPARAM key){
    bool bProcessed = false;
    HWND wndFocus = GetFocus()->GetSafeHwnd();

    if(key == VK_RETURN){
        if(wndFocus == butMail.GetSafeHwnd()){
            OnClickedSyncButtonMail();
            return true;
        };
       // if(wndFocus == butBriefcase.GetSafeHwnd()){
       //     OnClickedSyncButtonBriefcase();
       //     return true;
       // };
        if(wndFocus == butPimOptions.GetSafeHwnd()){
            OnClickedSyncButtonPimOptions();
            return true;
        }

        if(wndFocus == linkAdvanced.GetSafeHwnd()){
            OnBnClickedSyncLinkAdvanced();
            return true;
        };

#if defined(WIN32_PLATFORM_PSPC)
      //  if(wndFocus == butNotes.GetSafeHwnd()){
        //    OnClickedSyncButtonNotes();
        //    return true;
       // };
#endif
    }

    if( (key == VK_LEFT) || (key == VK_RIGHT) ){
        if(wndFocus == checkMail.GetSafeHwnd()){
            butMail.SetFocus();
            return true;
        }
        if ( wndFocus == checkContacts.GetSafeHwnd() ||
             wndFocus == checkCalendar.GetSafeHwnd() ||
             wndFocus == checkTasks.GetSafeHwnd() ){
            butPimOptions.SetFocus();
            return true;
        }

#if defined(WIN32_PLATFORM_PSPC)
        if(wndFocus == checkNotes.GetSafeHwnd()){
           // butNotes.SetFocus();
            return true;
        };
#endif

        if(wndFocus == checkBriefcase.GetSafeHwnd()){
           // butBriefcase.SetFocus();
            return true;
        }
        if(wndFocus == butMail.GetSafeHwnd()){
            checkMail.SetFocus();
            return true;
        }
        if (wndFocus == butPimOptions.GetSafeHwnd()){
            checkContacts.SetFocus();
            return true;
        }

#if defined(WIN32_PLATFORM_PSPC)
       // if(wndFocus == butNotes.GetSafeHwnd()){
       //     checkNotes.SetFocus();
       //     return true;
       // };
#endif

      //  if(wndFocus == butBriefcase.GetSafeHwnd()){
      //      checkBriefcase.SetFocus();
      //      return true;
      //  };
    }

    if(key == VK_DOWN){
        if(checkBriefcase.IsWindowEnabled()){
            if(wndFocus ==  checkBriefcase.GetSafeHwnd()){
                linkAdvanced.SetFocus();
                return true;
            }
        }
        else{
            // for portal last source is Tasks
            if(wndFocus ==  checkTasks.GetSafeHwnd()){
                linkAdvanced.SetFocus();
                return true;
            }
        }

    }

    if(key == VK_UP){
        if(wndFocus == linkAdvanced.GetSafeHwnd()){
            if(checkBriefcase.IsWindowEnabled()){
                checkBriefcase.SetFocus();
            }
            else{
                checkTasks.SetFocus();
            }

            return true;
        }
    }

    return false;
}


//
// These are called from menu
//
void CSyncSettings::OnEmailOptionsClicked() {
    OnClickedSyncButtonMail();
}

void CSyncSettings::OnPIMOptionsClicked() {
    OnClickedSyncButtonPimOptions();
}

void CSyncSettings::OnSyncMethodClicked() {
    OnBnClickedSyncLinkAdvanced();
}


/**
 * Repaint objects. Draw 2 lines as separators between email and PIM,
 * and between PIM and SyncMode.
 */
void CSyncSettings::OnPaint() {

    // Call default handler to redraw items if needed
    Default();


    // CDC::LineTo() uses x,y coordinates related to the display window area,
    // we need the offset between the window rect and the client rect.
    CRect clientRect, windowrect;
    GetClientRect(&clientRect);
    GetWindowRect(&windowrect);
    int offset = windowrect.top - clientRect.top;


    //
    // Set the SyncMode link & label...
    //
    int border = 2;
    int linkLength = (int)(clientRect.Width() * 0.45);     // too much?
    int labelLength = clientRect.Width() - linkLength;     // all the remaining

    CRect syncModeRect, grboxRect;
    linkAdvanced.GetWindowRect(&syncModeRect);
    GetDlgItem(IDC_PIM_GROUPBOX)->GetWindowRect(&grboxRect);
    int linkTop = grboxRect.bottom - offset + 5;           // +5 = some space under the separator line

    linkAdvanced.SetWindowPos(&CWnd::wndTop,
                              border,                   // some space from border
                              linkTop,
                              linkLength-border,
                              syncModeRect.Height(),
                              SWP_NOZORDER);

    GetDlgItem(IDC_SYNC_METHOD_LABEL)->SetWindowPos(&CWnd::wndTop,
                             linkLength,
                             linkTop+2,                 // +2 because the other one is underlined, so it's higher
                             labelLength,
                             syncModeRect.Height(),
                             SWP_NOZORDER);

    linkAdvanced.GetWindowRect(&syncModeRect);


    //
    // 1st horizontal line: black line above the SyncMode.
    // Starts left aligned with "SyncMode" and ends at 95% of the screen width
    // Top aligned with the groupbox (hidden)
    //
    CPoint start, end;
    start.x = syncModeRect.left +2;               start.y = grboxRect.bottom - offset;
    end.x   = (int)(clientRect.Width() * 0.95);   end.y   = grboxRect.bottom - offset;

    CClientDC dc(this);
    drawLine(&dc, start, end);


    //
    // 2nd horizontal line: gray dashed line [5% - 95%] between checkMail and checkContacts
    //
    CRect mailCheckboxRect, contactsCheckboxRect, mailOptionsRect;
    checkMail.GetWindowRect(&mailCheckboxRect);
    checkContacts.GetWindowRect(&contactsCheckboxRect);
    butMail.GetWindowRect(&mailOptionsRect);

    // Get the middle Y position between checkMail and checkContacts
    int posY = mailCheckboxRect.bottom + (contactsCheckboxRect.top - mailCheckboxRect.bottom)/2;
    start.x = (int)(clientRect.Width() * 0.05);   start.y = posY - offset;
    end.x   = (int)(clientRect.Width() * 0.95);   end.y   = posY - offset;

    COLORREF grey = RGB(180,180,180);
    drawLine(&dc, start, end, grey, 1, PS_DASH);
}

