/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "pim/AppointmentBuilder.h"
#include "base/Log.h"
#include "pim/difftime.h"
#include "pim/UtilsRecurrence.h"

/*
* Method to populate the appointment XML structure.
* It get every property name and value and build the XML structure to exchange with server.
*
* @param appointmentStringItem : the string that will contain the xml structure
* @param pAppointment          : the appointment object that contain every value
*
*/
void populateAppointmentStringItem(std::wstring &appointmentStringItem, IAppointment *pAppointment, BOOL isCRC) {

    BSTR element        = NULL      ;
    long elementLong    = 0         ;
    DATE startdate      = NULL      ;
    DATE enddate        = NULL      ;
    DATE date           = NULL      ;
    wstring element_s;

    //
    // FOR RECURRENCE
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int x = 0, y = 0, z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;


    VARIANT_BOOL variantBool = NULL; // 0==false -1==true
    wchar_t tempDate [DIM_LOCAL_TEMP];
    wchar_t* charBlob = NULL;

    appointmentStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    appointmentStringItem +=  TEXT("<appointment>");

        // BSTR
        pAppointment->get_Subject(&element);
        element_s = wstring(element);
        encodeSpecialChar(element_s);
        appointmentStringItem  += TEXT("<Subject>") + element_s + TEXT("</Subject>");
        SysFreeString(element);

        pAppointment->get_Location(&element);
        element_s = wstring(element); encodeSpecialChar(element_s);
        appointmentStringItem  += TEXT("<Location>") + element_s + TEXT("</Location>");
        SysFreeString(element);

        pAppointment->get_Categories(&element);
        element_s = wstring(element); encodeSpecialChar(element_s);
        appointmentStringItem  +=  TEXT("<Categories>") + element_s + TEXT("</Categories>");
        SysFreeString(element);

        pAppointment->get_ReminderSoundFile(&element);
        element_s = wstring(element); encodeSpecialChar(element_s);
        appointmentStringItem  +=  TEXT("<ReminderSoundFile>") + element_s + TEXT("</ReminderSoundFile>");
        SysFreeString(element);

        pAppointment->get_Body(&element);
        removeEndCarriage(&element);
        element_s = wstring(element);  encodeSpecialChar(element_s);
        appointmentStringItem  +=  TEXT("<Body>") + element_s + TEXT("</Body>");
        SysFreeString(element);

        // DATE
        pAppointment->get_Start(&startdate);
        pAppointment->get_End(&enddate);


        /*
        * see the start and end date. If the start is 00:00 and end is 23:59 the appointment is decided to be
        * an all day event. So the AllDayEvent tag is put to 'true' only in the SIF structure. The format of the date
        * is like the birthday: yyyy-MM-dd
        *
        */

        BOOL isAllDay = FALSE;

        if (isWindowsMobile5()) {
            pAppointment->get_AllDayEvent(&variantBool);
            variantBool == -1 ? isAllDay = TRUE : isAllDay = FALSE;

            if (isAllDay)
                enddate -= 1; // to be decremented of a day if it is all day in windows mobile 5.
                // This because the end date is 00:00:00 of the next day. we want the all day
                // is represented as <startDate>2007-01-15</StartDate> and and date as the same

        } else {
            isAllDayEvent(startdate, enddate);
        }


        wsprintf(tempDate, TEXT(""));
        double toCompare = MAX_DATE_DOUBLE;
        double toCompare4000 = MAX_DATE_DOUBLE_4000;

        SYSTEMTIME st;

        if (startdate != toCompare) {
            VariantTimeToSystemTime(startdate, &st);
            x = st.wHour;
            localTimeToUTC(st);
            y = st.wHour;
            if (isAllDay) {
                doubleToSystemTimeBirthday(tempDate, startdate);
            } else {
                doubleToSystemTime(tempDate, startdate);
            }
        }
        appointmentStringItem  = appointmentStringItem + TEXT("<Start>") + tempDate + TEXT("</Start>");

        wsprintf(tempDate, TEXT(""));
        if (enddate != toCompare) {
            if (isAllDay) {
                doubleToSystemTimeBirthday(tempDate, enddate);
            } else {
                doubleToSystemTime(tempDate, enddate);
            }
        }
        appointmentStringItem  = appointmentStringItem + TEXT("<End>") + tempDate + TEXT("</End>");

        if (isAllDay) {
            wsprintf(tempDate, TEXT("1")); // TRUE
        } else {
            wsprintf(tempDate, TEXT("0")); // TRUE
        }
        appointmentStringItem  = appointmentStringItem + TEXT("<AllDayEvent>") + tempDate + TEXT("</AllDayEvent>");

        pAppointment->get_ReminderSet(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        appointmentStringItem  = appointmentStringItem + TEXT("<ReminderSet>") + tempDate + TEXT("</ReminderSet>");

        // long
        //pAppointment->get_Duration(&elementLong);
        //wsprintf(tempDate, TEXT("%i"), elementLong);
        //appointmentStringItem  = appointmentStringItem + TEXT("<Duration>") + tempDate + TEXT("</Duration>");

        pAppointment->get_BusyStatus(&elementLong);
        wsprintf(tempDate, TEXT("%i"), elementLong);
        appointmentStringItem  = appointmentStringItem + TEXT("<BusyStatus>") + tempDate + TEXT("</BusyStatus>");

        pAppointment->get_Sensitivity(&elementLong);
        wsprintf(tempDate, TEXT("%i"), elementLong);
        appointmentStringItem  = appointmentStringItem + TEXT("<Sensitivity>") + tempDate + TEXT("</Sensitivity>");

        pAppointment->get_ReminderOptions(&elementLong);
        wsprintf(tempDate, TEXT("%i"), elementLong);
        appointmentStringItem  = appointmentStringItem + TEXT("<ReminderOptions>") + tempDate + TEXT("</ReminderOptions>");

        pAppointment->get_ReminderMinutesBeforeStart(&elementLong);
        wsprintf(tempDate, TEXT("%i"), elementLong);
        appointmentStringItem  = appointmentStringItem + TEXT("<ReminderMinutesBeforeStart>") + tempDate + TEXT("</ReminderMinutesBeforeStart>");


        /******************************************************/

        //Recurrence
        VARIANT_BOOL isRecurring;
        pAppointment->get_IsRecurring(&isRecurring);
        wsprintf(tempDate, TEXT("%i"), isRecurring);
        normalizeBoolean(tempDate);
        appointmentStringItem  = appointmentStringItem + TEXT("<IsRecurring>") + tempDate + TEXT("</IsRecurring>");

        IRecurrencePattern *pRecurrence;
        pAppointment->GetRecurrencePattern (&pRecurrence);

        wstring recurrence = L"";
        if (!isCRC || isRecurring == -1) { // variant bool to say TRUE
            recurrence = getRecurrenceTags(pRecurrence, isRecurring, x, y, olFolderCalendar, startdate, isAllDay);
        }
        // handle recurrence and exceptions events
        // wstring recurrence = getRecurrenceTags(pRecurrence, isRecurring, x, y, olFolderCalendar, startdate, isAllDay);

        pRecurrence->Release();

    appointmentStringItem += recurrence;
    appointmentStringItem = appointmentStringItem +  TEXT("</appointment>");

}

/*
wstring createExceptions(IRecurrencePattern* pRecurrence) {

    IExceptions* pExceptions;
    int numExceptions;

    pRecurrence->get_Exceptions(&pExceptions);
    pExceptions->get_Count(&numExceptions);
    wchar_t temp [DIM_LOCAL_TEMP];

    wstring exceptions = TEXT("<Exceptions>");
        wsprintf(temp, TEXT("<ExceptionNumber>%i</ExceptionNumber>"), numExceptions);
        exceptions += wstring(temp);

    for (int i = 0; i < numExceptions; i++) {
        IException* pException;
        pExceptions->Item(i + 1, &pException);
        wstring exception = createException(pException);
        exceptions += exception;
        pException->Release();
    }

    pExceptions->Release();

    exceptions +=  TEXT("</Exceptions>");

    return exceptions;
}

wstring createException(IException* pException) {

    wstring exceptionItem = TEXT("<Exception>");
    DATE originalDate = 0;
    VARIANT_BOOL varBool; // -1 = TRUE, 0 = FALSE
    SYSTEMTIME st;
    memset(&st, 0, sizeof(SYSTEMTIME));
    BSTR element        = NULL      ;
    wchar_t tempDate [DIM_LOCAL_TEMP];
    wstring element_s;

    pException->get_OriginalDate(&originalDate);
    doubleToSystemTimeBirthday(tempDate, originalDate);    // YYYY-MM-DD

    exceptionItem += TEXT("<OriginalDate>") + wstring(tempDate) + TEXT("</OriginalDate>");

    // if the recurrence is deleted get the delation and go exit
    pException->get_Deleted(&varBool);
    if (varBool == -1)
        goto finally;

    exceptionItem += TEXT("<ExAppointment>");
    IAppointment* pAppointment;
    pException->get_AppointmentItem(&pAppointment);

        pAppointment->get_Subject(&element);
        element_s = wstring(element);
        encodeSpecialChar(element_s);
        exceptionItem  += TEXT("<ExSubject>") + element_s + TEXT("</ExSubject>");
        SysFreeString(element); element = NULL;

        pAppointment->get_Location(&element);
        element_s = wstring(element);
        encodeSpecialChar(element_s);
        exceptionItem  += TEXT("<ExLocation>") + element_s + TEXT("</ExLocation>");
        SysFreeString(element); element = NULL;

        pAppointment->get_Body(&element);
        removeEndCarriage(&element);
        element_s = wstring(element);  encodeSpecialChar(element_s);
        exceptionItem  +=  TEXT("<ExBody>") + element_s + TEXT("</ExBody>");
        SysFreeString(element);

         // VARIANT

        pAppointment->get_AllDayEvent(&varBool);
        wsprintf(tempDate, TEXT("%i"), varBool);
        if (varBool == -1) {
            wsprintf(tempDate, TEXT("1")); // TRUE
        }
        exceptionItem  += TEXT("<ExAllDayEvent>") + wstring(tempDate) + TEXT("</ExAllDayEvent>");

        // DATE
        DATE startdate;
        DATE enddate;
        pAppointment->get_Start(&startdate);
        pAppointment->get_End(&enddate);

        //BOOL isAllDay = isAllDayEvent(startdate, enddate);
        BOOL isAllDay = varBool == -1 ? TRUE : FALSE;

        wsprintf(tempDate, TEXT(""));

        VariantTimeToSystemTime(startdate, &st);
        if (isAllDay) {
            doubleToSystemTimeBirthday(tempDate, startdate);
        } else {
            doubleToSystemTime(tempDate, startdate);
        }

        exceptionItem  += TEXT("<ExStart>") + wstring(tempDate) + TEXT("</ExStart>");
        wsprintf(tempDate, TEXT(""));

        if (isAllDay) {
            doubleToSystemTimeBirthday(tempDate, enddate);
        } else {
            doubleToSystemTime(tempDate, enddate);
        }

        exceptionItem += TEXT("<ExEnd>") + wstring(tempDate) + TEXT("</ExEnd>");

        long elementLong = 0;
        pAppointment->get_BusyStatus(&elementLong);
        wsprintf(tempDate, TEXT("%i"), elementLong);
        exceptionItem  += TEXT("<ExBusyStatus>") + wstring(tempDate) + TEXT("</ExBusyStatus>");

    if (pAppointment)
        pAppointment->Release();

   exceptionItem += TEXT("</ExAppointment>");
finally:

    exceptionItem += TEXT("</Exception>");
    return exceptionItem;

}
*/

/*
* Method that complete an appointment object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the object.
*
* @param pAppointment : the appointment object that contain every value
* @param ptrData      : the variable that contain the xml structure
*/

void completeAppointment(IAppointment *pAppointment, wchar_t *ptrData, IP_OUTLOOK_APP* polApp, wchar_t* path) {

    wchar_t* dummy                  = NULL;
    wchar_t* dummyEvent             = NULL;
    wstring  dummyEvent_s;
    VARIANT_BOOL isNotAllDayEvent   = TRUE;

    //
    // FOR RECURRENCE
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int x = 0, y = 0, z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;

    //VARIANT_BOOL --- AllDayEvent
    dummyEvent = getElementContent(ptrData, TEXT ("AllDayEvent"), NULL);

    if (dummyEvent != NULL &&
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 || // kept to be compliant with old client. Now TRUE is represented
         wcscmp(dummyEvent, TEXT("True")) == 0)) {  // with "1" only

        if (isWindowsMobile5()) {
            pAppointment->put_AllDayEvent(TRUE);
        } else {
            pAppointment->put_AllDayEvent(FALSE);
        }

        double locDate = NULL;

        dummyEvent = getElementContent(ptrData, TEXT ("Start"), NULL);

        DATE st;
        SYSTEMTIME systemTime;
        BOOL isUTC = FALSE;
        BOOL isAllDayFormat = isDateForAllDayEventFormat(dummyEvent);

        if (isAllDayFormat) {
            systemTimeToDoubleBirthFormat(dummyEvent, &st, TEXT("START"));

        }  else { // the date is in the usual format

            isUTC = getDateAndTimeAsIs(dummyEvent, &st);

            VariantTimeToSystemTime(st, &systemTime);
            x = systemTime.wHour;
            if (isUTC) {
                UTCToLocalTime2(systemTime);
            }
            y = systemTime.wHour;
            systemTime.wHour = 0;
            systemTime.wMinute = 0;
            systemTime.wMilliseconds = 0;
            SystemTimeToVariantTime(&systemTime, &st);
        }
        pAppointment->put_Start(st);

        dummyEvent = getElementContent(ptrData, TEXT ("End"), NULL);
        isAllDayFormat = isDateForAllDayEventFormat(dummyEvent);

        if (isAllDayFormat) {
            systemTimeToDoubleBirthFormat(dummyEvent, &st, TEXT("END"));

        }  else { // the date is in the usual format

            isUTC = FALSE;
            isUTC = getDateAndTimeAsIs(dummyEvent, &st);

            VariantTimeToSystemTime(st, &systemTime);

            if (isUTC) {
                UTCToLocalTime2(systemTime);
            }

            systemTime.wHour = 23;
            systemTime.wMinute = 59;
            systemTime.wMilliseconds = 0;
            SystemTimeToVariantTime(&systemTime, &st);
        }

        pAppointment->put_End(st);

        delete [] dummyEvent;dummyEvent=NULL;
        isNotAllDayEvent = FALSE;
    }
    else {
        pAppointment->put_AllDayEvent(VARIANT_FALSE);

    }
    delete [] dummyEvent;dummyEvent=NULL;

    //VARIANT_BOOL -- ReminderSet
    dummyEvent = getElementContent(ptrData, TEXT ("ReminderSet"), NULL);


    if (dummyEvent != NULL &&
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 ||     // kept to be compliant with old client. Now TRUE is represented
         wcscmp(dummyEvent, TEXT("True")) == 0)) {  // with "1" only
        pAppointment->put_ReminderSet(VARIANT_TRUE);

    }
    else {
        pAppointment->put_ReminderSet(VARIANT_FALSE);
    }

    delete [] dummyEvent;dummyEvent=NULL;

    // BSTR
    dummyEvent = getElementContent(ptrData, TEXT ("Subject"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_Subject ((wchar_t*)dummyEvent_s.data());
           delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Location"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_Location ((wchar_t*)dummyEvent_s.data());
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("ReminderSoundFile"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_ReminderSoundFile  ((wchar_t*)dummyEvent_s.data());
        delete [] dummyEvent;dummyEvent=NULL;
    }
    dummyEvent = getElementContent(ptrData, TEXT ("Categories"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_Categories ((wchar_t*)dummyEvent_s.data());
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Body"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
        convertNewlines(dummyEvent_s); // BUG #1562
        pAppointment->put_Body  ((wchar_t*)dummyEvent_s.data());
        delete [] dummyEvent;dummyEvent=NULL;
    }

    DATE start = 0;

    // Read the start date
    if (isNotAllDayEvent) {
        //DATE
        dummyEvent = getElementContent(ptrData, TEXT ("Start"), NULL);

        if (dummyEvent && (wcscmp(dummyEvent, TEXT("")) != 0)) {
            SYSTEMTIME systemTime;
            //
            // to handle properly the RecurrencePattern
            //
            BOOL isUTC = FALSE;
            isUTC = getDateAndTimeAsIs(dummyEvent, &start);

            VariantTimeToSystemTime(start, &systemTime);
            x = systemTime.wHour;
            if (isUTC) {
                UTCToLocalTime2(systemTime);
            }
            y = systemTime.wHour;

            systemTimeToDouble(dummyEvent, &start, NULL);

            pAppointment->put_Start(start);
            delete [] dummyEvent;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("End"), NULL);

        if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("")) != 0)) {
            DATE en;
            systemTimeToDouble(dummyEvent, &en, NULL);

            pAppointment->put_End(en);
            delete [] dummyEvent;
        }

    }
    else {
                // TODO: which is the start for an ALLDAY????!!!!!!!!!
    }

    //long
    //dummyEvent = getElementContent(ptrData, TEXT ("Duration"), NULL);
    //if (dummyEvent != NULL) {
    //    pAppointment->put_Duration(wcstol(dummyEvent, &dummy, 0));
    //    delete [] dummyEvent;dummyEvent=NULL;
    //}

    dummyEvent = getElementContent(ptrData, TEXT ("BusyStatus"), NULL);
    if (dummyEvent != NULL) {
        pAppointment->put_BusyStatus(wcstol(dummyEvent, &dummy, 0));
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Sensitivity"), NULL);
    if (dummyEvent != NULL) {
        pAppointment->put_Sensitivity(wcstol(dummyEvent, &dummy, 0));
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("ReminderOptions"), NULL);
    if (dummyEvent != NULL) {
        pAppointment->put_ReminderOptions (wcstol(dummyEvent, &dummy, 0));
        delete [] dummyEvent;dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("ReminderMinutesBeforeStart"), NULL);
    if (dummyEvent != NULL) {
        pAppointment->put_ReminderMinutesBeforeStart  (wcstol(dummyEvent, &dummy, 0));
        delete [] dummyEvent;dummyEvent=NULL;
    }

    /****************************************************/

    //Recurrence
    dummyEvent = getElementContent(ptrData, TEXT ("IsRecurring"), NULL);

    if (dummyEvent != NULL &&
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 ||    // kept to be compliant with old client. Now TRUE is represented
         wcscmp(dummyEvent, TEXT("True")) == 0)) { // with "1" only

        pAppointment->ClearRecurrencePattern ();
        /*
        * This save the appointment with no recurrence pattern and no exceptions too
        * If there are some before it deletes the old one and replace with the new just came
        */
        pAppointment->Save();

        IRecurrencePattern *pRecurrence;
        pAppointment->GetRecurrencePattern (&pRecurrence);

        //
        // Parse the recurrence tags
        //
        parseRecurrenceTags(pRecurrence, ptrData, start, x, y);

        //
        // ------------ EXCEPTIONS --------------------------
        //

        //
        // Parse the exception tags
        //
        parseExceptionTags(pRecurrence, pAppointment, ptrData, polApp, path);
        pRecurrence->Release();

    }
}


BOOL isAllDayEvent(DATE startdate, DATE enddate) {

    SYSTEMTIME ststart, stend;
    VariantTimeToSystemTime(startdate, &ststart);
    VariantTimeToSystemTime(enddate, &stend);
    BOOL ret = FALSE;

    if (ststart.wHour == 0 && ststart.wMinute == 0 &&
        stend.wHour == 23 && stend.wMinute == 59)

        ret = TRUE;

    return ret;

}




/**
* The method change the property AllDayEvent to False if set to true.
* This because changing AllDayEvent changes OID too. So
* if AllDayEvent == YES (-1) ---> AllDayEvent = No, start = 0, end = 24
*
* @param pAppointment : the appointment object that contain every value
* @param dataType     : the data type to get retrieve appointments
*
* NOTE: this function is not used on Windows Mobile 5.0 devices because
* the OID remains the same even if the property changes.
*
*/

void normalizeAppointment(IP_OUTLOOK_APP *polApp, int dataType, wchar_t* path) {


    IPOutlookItemCollection *pItems;
    IAppointment *pAppointment;
    IFolder *pFolder;

    VARIANT_BOOL variantBool = FALSE;
    wchar_t tempDate [30];
    DATE date = NULL, startdate = NULL;


    polApp->GetDefaultFolder (dataType, &pFolder);
    pFolder->get_Items (&pItems);

    int currentCountItems = 0;

    pItems->get_Count(&currentCountItems);

    BOOL isWinMob5 = isWindowsMobile5();
    for (int i = 0; i < currentCountItems; i++) {
        pItems->Item (i + 1, (IDispatch**)&pAppointment);
        pAppointment->get_AllDayEvent(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);

        if (wcscmp(tempDate, TEXT("-1")) == 0) {

            SYSTEMTIME t;

            // DATE
            // START****************
            pAppointment->get_Start(&startdate);

            VariantTimeToSystemTime(startdate, &t);
            t.wHour = 0;
            t.wMinute = 0;
            SystemTimeToVariantTime(&t, &startdate);

            IRecurrencePattern* pRec;
            pAppointment->GetRecurrencePattern(&pRec);
            wstring recurrence = getRecurrenceTags(pRec, -1, 0, 0, olFolderCalendar, startdate, TRUE);

            pAppointment->put_Start(startdate);

            //END*************
            pAppointment->get_End(&date);

            VariantTimeToSystemTime(date, &t);
            if (isWinMob5) {
                if (startdate == date && !isPocketPC()){
                    // on smartphone wm5 the modification from all day -> not all day -> all day
                    // change the normal behaviour all day handling and the end date is different.
                    // this fix doesn't permit the day change but can be problem with all day events of may days.
                    // next releases change this feature and consider all day event as is.
                    t.wHour = 23;
                    t.wMinute = 59;
                } else {

                    SYSTEMTIME s;
                    s = DT_AddDiff(nano100SecInMin, -(1), &t);
                    t.wDay              = s.wDay          ;
                    t.wDayOfWeek        = s.wDayOfWeek    ;
                    t.wHour             = s.wHour         ;
                    t.wMilliseconds     = s.wMilliseconds ;
                    t.wMinute           = s.wMinute       ;
                    t.wMonth            = s.wMonth        ;
                    t.wSecond           = s.wSecond       ;
                    t.wYear             = s.wYear         ;
                }
            } else {
                t.wHour = 23;
                t.wMinute = 59;
            }

            SystemTimeToVariantTime(&t, &date);
            pAppointment->put_End(date);
            pAppointment->put_AllDayEvent(FALSE);
            pRec->Release();
            HRESULT  resu = pAppointment->Save();
            pAppointment->ClearRecurrencePattern();
            pAppointment->GetRecurrencePattern(&pRec);
            parseRecurrenceTags(pRec, recurrence.c_str(), startdate, 0, 0);
            parseExceptionTags(pRec, pAppointment, recurrence.c_str(), polApp, path);
            pRec->Release();

        }

        pAppointment->Release ();
    }

    pFolder->Release();
    pItems->Release();
}






void copyFields(IAppointment *pAppointmentExc, IAppointment *pAppointmentCopy) {

    BSTR element = NULL;

    pAppointmentExc->get_Subject(&element);
    pAppointmentCopy->put_Subject(element);
    SysFreeString(element);

    pAppointmentExc->get_Location(&element);
    pAppointmentCopy->put_Location(element);
    SysFreeString(element);

    pAppointmentExc->get_Body(&element);
    pAppointmentCopy->put_Body(element);
    SysFreeString(element);

    DATE d;
    pAppointmentExc->get_Start(&d);
    pAppointmentCopy->put_Start(d);

    pAppointmentExc->get_End(&d);
    pAppointmentCopy->put_End(d);

    VARIANT_BOOL vbool;
    pAppointmentExc->get_AllDayEvent(&vbool);
    pAppointmentCopy->put_AllDayEvent(vbool);

    pAppointmentExc->get_ReminderSet(&vbool);
    pAppointmentCopy->put_ReminderSet(vbool);

    long elementLong = 0;
    pAppointmentExc->get_BusyStatus(&elementLong);
    pAppointmentCopy->put_BusyStatus(elementLong);

    pAppointmentExc->get_ReminderMinutesBeforeStart(&elementLong);
    pAppointmentCopy->put_ReminderMinutesBeforeStart(elementLong);

}


/*
* . Scan all the appointment and get the ones with recurrences
* . If there are any that are with exceptions remove them
* .
*/

BOOL normalizeAppointmentExceptions(IP_OUTLOOK_APP* polApp) {

    IPOutlookItemCollection *pItems;
    IAppointment *pAppointment;
    IFolder *pFolder;

    VARIANT_BOOL variantBool = FALSE;

    polApp->GetDefaultFolder (olFolderCalendar, &pFolder);
    pFolder->get_Items (&pItems);

    int currentCountItems = 0;

    pItems->get_Count(&currentCountItems);

    for (int i = 0; i < currentCountItems; i++) {
        VARIANT_BOOL isRecurring;
        pItems->Item (i + 1, (IDispatch**)&pAppointment);
        pAppointment->get_IsRecurring(&isRecurring);

        if (isRecurring >= 0) { // VARIANT_BOOL true is -1. If not recurring continue
            pAppointment->Release();
            continue;
        }

        IRecurrencePattern *pRecurrence;
        pAppointment->GetRecurrencePattern (&pRecurrence);

        IExceptions* pExceptions;
        int numExceptions;

        pRecurrence->get_Exceptions(&pExceptions);

        if (!pExceptions) { // there are no exception in this occurrence
            continue;
        }

        pExceptions->get_Count(&numExceptions);

        for (int i = 0; i < numExceptions; i++) {
            IException* pException;
            pExceptions->Item(i + 1, &pException);

            // if the recurrence is deleted get the deletion and continue to the next appointment
            VARIANT_BOOL isDeleted;
            pException->get_Deleted(&isDeleted);
            if (isDeleted == -1) {
                pException->Release();
                continue;
            }

            IAppointment* pAppointmentExc, *pAppointmentCopy, *pAppointmentNew;
            pException->get_AppointmentItem(&pAppointmentExc);

            HRESULT h = pAppointment->Copy(&pAppointmentCopy);

            // copy the fields from the pAppintmentExc to the pAppointmentCopy
            copyFields(pAppointmentExc, pAppointmentCopy);

            pAppointmentExc->Delete();
            pAppointmentExc->Release();

            pAppointmentCopy->Save();

            /*
            * There is an exception in ClearRecurrencePattern on the copy appointment.
            * it is safe to open a new appointment object
            */
            long oid = 0;
            h = pAppointmentCopy->get_Oid(&oid);
            pAppointmentCopy->Release();

            polApp->GetItemFromOid (oid, (IDispatch**)&pAppointmentNew);
            if (pAppointmentNew) {
                h = pAppointmentNew->ClearRecurrencePattern();
                pAppointmentNew->Save();
                pAppointmentNew->Release();
            } else {
                LOG.error("Appointment normalization: it is impossible to clear the pattern %i", oid);
            }
            pException->Release();

        }
        pExceptions->Release();
        pRecurrence->Release();

        pAppointment->Release();
    }

    pFolder->Release();
    pItems->Release();

    return TRUE;
}

/*
* The method calculate the CRC hash value for the appointment item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pAppointment : the appointment object that contain every value
*/

long calculateAppointmentHash (IAppointment* pAppointment) {

    wstring p;
    populateAppointmentStringItem(p, pAppointment, TRUE);

    long l = stringCrc32(p.data());

    return l;
}

VObject* AppointmentToVObject(IAppointment *pAppointment) {

    BSTR element = NULL;
    DATE startdate = NULL;
    DATE enddate   = NULL;
    VObject* vo = new VObject();
    VProperty* vp      = NULL;
    wchar_t*  dateValue = new wchar_t[20];

    vp = new VProperty(TEXT("BEGIN"), TEXT("VCALENDAR"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("VERSION"), TEXT("1.0"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("BEGIN"), TEXT("VEVENT"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    pAppointment->get_Subject(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("SUMMARY"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pAppointment->get_Location(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("LOCATION"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pAppointment->get_Body(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("DESCRIPTION"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pAppointment->get_Start(&startdate);
    dateValue[0] = 0;
    doubleToSystemTime(dateValue, startdate);
    vp = new VProperty(TEXT("DTSTART"), dateValue);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    pAppointment->get_End(&enddate);
    dateValue[0] = 0;
    doubleToSystemTime(dateValue, enddate);
    vp = new VProperty(TEXT("DTEND"), dateValue);
    vo->addProperty(vp);
    delete [] vp; vp = NULL;

    pAppointment->get_Categories(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("CATEGORIES"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    long sensitivity;
    pAppointment->get_Sensitivity(&sensitivity);
    vp = new VProperty(TEXT("CLASS"));
    if(sensitivity == olPrivate)
        vp->addValue(TEXT("PRIVATE"));
    else
        vp->addValue(TEXT("PUBLIC"));
    vo->addProperty(vp);
    delete vp; vp = NULL;


    IRecipients* pRecipients;
    pAppointment->get_Recipients(&pRecipients);
    int countRecipient = 0;
    pRecipients->get_Count(&countRecipient);

    for(int i = 0; i < countRecipient; i++) {
        IRecipient* pRecipient;
        pRecipients->Item(i + 1, &pRecipient);

        wchar_t* attedee = new wchar_t[100];
        pRecipient->get_Name(&element);
        wcscat(attedee, element);
        wcscat(attedee, TEXT(" "));
        pRecipient->get_Address(&element);
        wcscat(attedee, element);
        vp = new VProperty(TEXT("ATTENDEE"), attedee);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        delete [] attedee; attedee = NULL;
        pRecipient->Release();
    }

    pRecipients->Release();

    if(dateValue)
    {
        delete [] dateValue; dateValue = NULL;
    }

    VARIANT_BOOL bReminder;
    pAppointment->get_ReminderSet(&bReminder);

    if(bReminder == VARIANT_TRUE) {
        long minBefore;
        pAppointment->get_ReminderMinutesBeforeStart(&minBefore);
        double minStartDate = startdate * 1440;
        //subtract the alarm
        minStartDate -= minBefore;

        if(dateValue)
        {     delete [] dateValue; dateValue = NULL;        }

        dateValue = new wchar_t[30];
        doubleToSystemTime(dateValue, minStartDate/1440);

        vp = new VProperty(TEXT("AALARM"));
        vp->addValue(dateValue);                        // "RunTime"
        vp->addValue(TEXT(""));                         // "Snooze Time" (empty)
        vp->addValue(TEXT("0"));                        // "Repeat Count"
        vp->addValue(TEXT("reminder"));                 // "Audio Content"
        vo->addProperty(vp);

        delete vp; vp = NULL;

        if(dateValue)
          {     delete [] dateValue; dateValue = NULL;        }

    }

    //recurrence
    VARIANT_BOOL isRec;
    pAppointment->get_IsRecurring(&isRec);

    if(isRec == VARIANT_TRUE) {
        IRecurrencePattern* pRecPat;
        pAppointment->GetRecurrencePattern(& pRecPat);
        wchar_t* recRule = NULL;
        recRule = extractRrule(pRecPat);
        if(recRule) {
            vp = new VProperty(TEXT("RRULE"), recRule);
            vo->addProperty(vp);
            delete vp; vp = NULL;
            delete [] recRule; recRule = NULL;
        }
        pRecPat->Release();
    }

    vp = new VProperty(TEXT("END"), TEXT("VEVENT"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("END"), TEXT("VCALENDAR"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    return vo;
}

void VObjectToAppointment(IAppointment *pAppointment, VObject *vo) {

    int i = 0;

    pAppointment->ClearRecurrencePattern ();

    if(vo->containsProperty(TEXT("SUMMARY")) && vo->getProperty(TEXT("SUMMARY"))->getValue())
        pAppointment->put_Subject(vo->getProperty(TEXT("SUMMARY"))->getValue());

    if(vo->containsProperty(TEXT("LOCATION")) && vo->getProperty(TEXT("LOCATION"))->getValue())
        pAppointment->put_Location(vo->getProperty(TEXT("LOCATION"))->getValue());

    if(vo->containsProperty(TEXT("DESCRIPTION")) && vo->getProperty(TEXT("DESCRIPTION"))->getValue())
        pAppointment->put_Body(vo->getProperty(TEXT("DESCRIPTION"))->getValue());

    DATE startdate = NULL;
    DATE enddate = NULL;

    if(vo->containsProperty(TEXT("DTSTART")) && vo->getProperty(TEXT("DTSTART"))->getValue()) {

        systemTimeToDouble(vo->getProperty(TEXT("DTSTART"))->getValue(), &startdate, NULL);
        pAppointment->put_Start(startdate);
        }

    if(vo->containsProperty(TEXT("DTEND")) && vo->getProperty(TEXT("DTEND"))->getValue()) {

        systemTimeToDouble(vo->getProperty(TEXT("DTEND"))->getValue(), &enddate, NULL);
        pAppointment->put_End(enddate);
        }

    BOOL isAllDay = isAllDayEvent(startdate, enddate);

    if(isAllDay)
        pAppointment->put_AllDayEvent(VARIANT_TRUE);
    else
        pAppointment->put_AllDayEvent(VARIANT_FALSE);

    if(vo->containsProperty(TEXT("CATEGORIES")) && vo->getProperty(TEXT("CATEGORIES"))->getValue())
        pAppointment->put_Categories(vo->getProperty(TEXT("CATEGORIES"))->getValue());


    if(vo->containsProperty(TEXT("CLASS")) && vo->getProperty(TEXT("CLASS"))->getValue()) {
        wchar_t* calClass = new wchar_t[20];
        wcscpy(calClass, vo->getProperty(TEXT("CLASS"))->getValue());
        if(!wcscmp(calClass, TEXT("PRIVATE")) || !wcscmp(calClass, TEXT("CONFIDENTIAL")))
            pAppointment->put_Sensitivity(olPrivate);
        else
            pAppointment->put_Sensitivity(olNormal);

        delete [] calClass;
    }
    else
        pAppointment->put_Sensitivity(olNormal);

    //ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
    //ATTENDEE;ROLE=ATTENDEE;STATUS=CONFIRMED;VALUE=URL;TYPE=VCARD:http://www.xyz.com/~myvcard.vcf
    IRecipients* pRecipients;
    pAppointment->get_Recipients(&pRecipients);
    int countRecipient = 0;
    pRecipients->get_Count(&countRecipient);

    for (i = 0; i < countRecipient; i++) {
        pRecipients->Remove(i + 1);
    }

    for(i = 0; i<vo->propertiesCount(); i++)
        if(!wcscmp(vo->getProperty(i)->getName(), TEXT("ATTENDEE")) &&
            vo->getProperty(i)->getValue() &&
            !vo->getProperty(i)->containsParameter(TEXT("VALUE"))) {

            wchar_t* attendee = new wchar_t[300];
            wcscpy(attendee, vo->getProperty(i)->getValue());

            wchar_t separator[] = TEXT("<");
            wchar_t* pos = NULL;

            pos = wcsstr(attendee, separator);

            if(pos) {
                wchar_t* name = new wchar_t[200];
                wcscpy(name, TEXT(""));
                if((pos - attendee)-1 > 0) {
                    wcsncpy(name, attendee, (pos - attendee)-1);
                    name[pos - attendee] = TEXT('\0');
                }

                wchar_t* addr = new wchar_t[200];
                wcscpy(addr, pos+1);
                addr[wcslen(addr) - 1] = TEXT('\0');

                IRecipient* pRecipient;
                pRecipients->Add(name, &pRecipient);
                pRecipient->put_Address(addr);
                pRecipient->Release();

                delete [] name;
                delete [] addr;
            }
        }

    pRecipients->Release();

    //alarm
    //The value consists of: Run Time, Snooze Time; Repeat Count, Audio Content
    if(vo->containsProperty(TEXT("AALARM"))) {

        if (vo->getProperty(TEXT("AALARM"))->getValue()) {

            DATE runTime;
            systemTimeToDouble(vo->getProperty(TEXT("AALARM"))->getPropComponent(1), &runTime, NULL);

            long minBeforeEvent = long((startdate - runTime) * 1440);

            pAppointment->put_ReminderSet(VARIANT_TRUE);
            pAppointment->put_ReminderMinutesBeforeStart(minBeforeEvent);

            } else {
                pAppointment->put_ReminderSet(VARIANT_FALSE);
            }

    } else {
        pAppointment->put_ReminderSet(VARIANT_FALSE);
    }



    //recurrence
    if((vo->containsProperty(TEXT("RRULE")) && vo->getProperty(TEXT("RRULE"))->getValue())) {

        pAppointment->ClearRecurrencePattern();

        IRecurrencePattern* pRecPat;
        pAppointment->GetRecurrencePattern(& pRecPat);


        DATE startDate;
        pAppointment->get_Start(&startDate);
        bool ret = convertRrule(pRecPat, vo->getProperty(TEXT("RRULE"))->getValue(), startDate);

        if(!ret)
            pAppointment->ClearRecurrencePattern();

        pRecPat->Release();

    }

}
