/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
#include <windows.h>
#include "Winbase.h"

#include "pimstore.h"
#include <stdio.h>

#include "base/Log.h"
#include "spds/spdsutils.h"
#include "spds/Constants.h"
#include "spdm/spdmutils.h"
#include "pim/Utils.h"
#include "pim/FILEClient2Server.h"
#include "pim/FILEFileManagement.h"
#include "spds/FileData.h"
#include "pim/ClientSettings.h"

#include "shellapi.h"

static wstring getXMLNote (const char* ptrFileContent, const wchar_t* fileName);


/*
to get My documents foleder
SHGetSpecialFolderPath(NULL, internalPath, CSIDL_PERSONAL, FALSE);
*/

void setAllItemsFILE(Container* c, int dataType, const wchar_t* path, const wchar_t* dir) {

    LOG.debug("Enter in setAllItemsFILE... ");

    wchar_t localTemp [DIM_LOCAL_TEMP];

    wchar_t **ptrArrayOfFileName    = NULL;
    wchar_t  *ptrFileContent        = NULL;
    char* fileContent = NULL;
    char* pp = NULL;
    wchar_t   filterPattern [256];
    wchar_t   internalPath  [DIM_LOCAL_TEMP];

    int numBytes            = 0;
    int count               = 0;
    int j                   = 0;
    unsigned long sizeFile  = 0;

    BOOL recursiveInDirectory = FALSE;
    filterPattern[0] = 0;
    internalPath[0]  = 0;

    /*
    * It avoids to send object greater then MaxObjSize.
    * In next releases it must me inserted into the
    * source.
    */

    //ClientSettings cs(APPLICATION_URI);
    //cs.readConfig();
    int fileRemoved = 0;
    //
    // decide the path of the files to read
    //

    if (dataType == constFiles)
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
    else if (dataType == constNotes) {
        wsprintf(internalPath,  TEXT("%s%s"), dir, TEXT("\\"));
        wcscpy(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {
        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));
        recursiveInDirectory = TRUE;
    }

    // to retrieve num of files
    getFileList(internalPath, ptrArrayOfFileName, &count, TRUE, recursiveInDirectory, filterPattern);

    ptrArrayOfFileName = new wchar_t*[count];

    for ( int k = 0; k < count; k++) {
        ptrArrayOfFileName[k] = 0;
    }

    // to retrieve fileName
    getFileList(internalPath, ptrArrayOfFileName, &count, FALSE, recursiveInDirectory, filterPattern);

    SyncItem* syncItem = NULL;

    for (j = 0; j < count; j++) {

        // fileName with complete path (e.g. \\Program Files\\SyncClient PIM\\store\\example.bin)
        wcscpy(localTemp, ptrArrayOfFileName[j]);

        if (dataType == constFiles) {

            unsigned long sizeFile = 0;
            getFileSize(ptrArrayOfFileName[j], &sizeFile);
            if (sizeFile == 0)
                return;
            if (sizeFile > getMaxObjectSizeToSend()) {
                LOG.debug("Found file greater then maxObjectSize(%lu): sizefile = %lu", getMaxObjectSizeToSend(), sizeFile);
                fileRemoved++;
                continue;
            }
            fileContent = new char[sizeFile];
            readByteFromFile(ptrArrayOfFileName[j], (byte*)fileContent, &numBytes);
            FileData fileData;
            getFileName(ptrArrayOfFileName[j], localTemp);
            fileData.setName(localTemp);
            fileData.setSize(numBytes);
            fileData.setEnc(TEXT("base64"));
            fileData.setBody(fileContent, numBytes);
            delete [] fileContent; fileContent = NULL;
            fileContent = fileData.format();
            numBytes = strlen(fileContent);

        }
        else if (dataType == constNotes) {

            pp = readTextFromBinaryFileChar(ptrArrayOfFileName[j]);
            getFileName(ptrArrayOfFileName[j], localTemp);
            if (pp != NULL) {
                wstring note = getXMLNote(pp, localTemp);
                fileContent = toMultibyte((wchar_t*)note.data());
                numBytes = strlen(fileContent);
            }
        }
        else if (dataType == constFavorites) {

            ptrFileContent = readTextFromBinaryFile(ptrArrayOfFileName[j]);

            if (ptrFileContent != NULL) {
                translateText2Base64(&ptrFileContent);
            }
            fileContent = toMultibyte(ptrFileContent);
            numBytes = strlen(fileContent);

            wcscpy(localTemp, ptrArrayOfFileName[j]);

            wchar_t* t = wcsstr(ptrArrayOfFileName[j], internalPath);

            if (t != NULL) {
                wsprintf(localTemp, t + wcslen(internalPath));
                delete [] t; t = NULL;
            }
            if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                convertSlashAt(localTemp);
        }

        encodeKey (localTemp);

        syncItem = new SyncItem(localTemp);

        if (fileContent == NULL) {
            syncItem->setData("", 1*sizeof(char));
        }
        else {
            syncItem->setData(fileContent, numBytes);
        }

        c->addItemToAllItems(syncItem);

        if (ptrFileContent != NULL) {
            delete [] ptrFileContent; ptrFileContent = NULL;
        }
        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
        if (fileContent != NULL) {
            delete [] fileContent; fileContent = NULL;
        }
        pp = NULL;

    }

    // delete array and pointer...
    for (j = 0; j < count; j++) {
        if (ptrArrayOfFileName[j] != NULL)
            delete [] ptrArrayOfFileName[j];
    }

    if (ptrArrayOfFileName != NULL)
        delete [] ptrArrayOfFileName;

    wsprintf(localTemp, SET_ALL_ITEMS, count - fileRemoved);
    LOG.debug("Exiting from setAllItemsFILE... ");
}

void setAllItemsFILEKey(Container* c, int dataType, const wchar_t* path, const wchar_t* dir) {

    LOG.debug("Enter in setAllItemsFILEKey... ");
    wchar_t localTemp [DIM_LOCAL_TEMP];

    wchar_t **ptrArrayOfFileName    = NULL;

    wchar_t   filterPattern [256];
    wchar_t   internalPath  [DIM_LOCAL_TEMP];

    int numBytes            = 0;
    int count               = 0;
    int j                   = 0;
    unsigned long sizeFile  = 0;

    BOOL recursiveInDirectory = FALSE;
    filterPattern[0] = 0;
    internalPath[0]  = 0;

    //
    // decide the path of the files to read
    //

    if (dataType == constFiles)
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
    else if (dataType == constNotes) {
        wsprintf(internalPath,  TEXT("%s%s"), dir, TEXT("\\"));
        wsprintf(filterPattern, TEXT("%s"), PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {
        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));
        recursiveInDirectory = TRUE;
    }

    // to retrieve num of files
    getFileList(internalPath, ptrArrayOfFileName, &count, TRUE, recursiveInDirectory, filterPattern);

    ptrArrayOfFileName = new wchar_t*[count];

    for ( int k = 0; k < count; k++) {
        ptrArrayOfFileName[k] = 0;
    }

    // to retrieve fileName
    getFileList(internalPath, ptrArrayOfFileName, &count, FALSE, recursiveInDirectory, filterPattern);

    SyncItem* syncItem = NULL;

    for (j = 0; j < count; j++) {

        // fileName with complete path (e.g. \\Program Files\\SyncClient PIM\\store\\example.bin)
        wcscpy(localTemp, ptrArrayOfFileName[j]);

        if (dataType == constFiles)
            getFileName(ptrArrayOfFileName[j], localTemp);
        else if (dataType == constNotes)
            getFileName(ptrArrayOfFileName[j], localTemp);
        else if (dataType == constFavorites) {

            wcscpy(localTemp, ptrArrayOfFileName[j]);
            wchar_t* t = wcsstr(ptrArrayOfFileName[j], internalPath);

            if (t != NULL) {
                wsprintf(localTemp, t + wcslen(internalPath));
                delete [] t;
            }
            if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                convertSlashAt(localTemp);
        }

        encodeKey (localTemp);

        syncItem = new SyncItem(localTemp);
        syncItem->setData("", 1*sizeof(char));

        c->addItemToAllItems(syncItem);

        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
    }

    // delete array and pointer...
    for (j = 0; j < count; j++) {
        if (ptrArrayOfFileName[j] != NULL)
            delete [] ptrArrayOfFileName[j];
    }
    if (ptrArrayOfFileName != NULL)
        delete [] ptrArrayOfFileName;
    LOG.debug("Exiting from setAllItemsFILEKey... ");
}


/*
* The method replace the text file content with new one in xml format.
*/
static wstring getXMLNote (const char* ptrFileContent, const wchar_t* fileName) {

    char* t  = NULL;
    wstring name(fileName);

    int position  = name.find(PWI_EXTENSION);
    /*
    string s = name;
    name = string(s, position);
    */

    if (position != std::wstring::npos) {
        wstring s = name;
        name = s.substr(0,position);
    }
    encodeSpecialChar(name);

    wchar_t *cnt = toWideChar(ptrFileContent);
    wstring content(cnt);
    encodeSpecialChar(content);

    wchar_t date[64];
    doubleToSystemTime(date, getSystemTime());

    wstring xml = wstring(TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
    xml += TEXT("<note><Subject>");
    xml += name; // filename (with no .pwi extension)
    xml += TEXT("</Subject><Body>") ;
    xml += content ;
    xml += TEXT("</Body><Date>");
    xml += date;
    xml += TEXT("</Date></note>");

    return xml;
}


/*
* text pointed by ptrData is translated in base64 and assigned to ptrData
* Used only for "favorites" source.
* TO BE DELETED
*
*/
void translateText2Base64(wchar_t** ptrFileContent) {

    wchar_t* ptr = NULL;
    ptr = encodeBase64UTF8(*ptrFileContent);
    if (*ptrFileContent != NULL) {
        delete *ptrFileContent;
        *ptrFileContent = NULL;
    }
    *ptrFileContent = ptr;
}


/*
* get the data content of a file by filename and return these data into base64 format.
* It's useful to read data to set into setAllItems, setModifiedItem, setNewItems
NO MORE USED


char* getB64DataContentByFilename(wchar_t* filename) {

    unsigned long sizeFile      = 0;
    int      numBytes           = 0;
    byte*    ptrFileContent     = NULL;
    char*    base64             = NULL;
    wchar_t* ptrFileContentb64  = NULL;
    int encodeLen               = 0;
    getFileSize(filename, &sizeFile);

    if (sizeFile == 0)
        goto finally;

    ptrFileContent = new byte[sizeFile];

    // read bytes from file

    readByteFromFile(filename, ptrFileContent, &numBytes);

    if (numBytes != (int)sizeFile) {
        LOG.error("The number of byte read and size are different");
        goto finally;
    }

    encodeLen = lengthForB64(numBytes);
    base64 = new char[encodeLen + 1];

    if (base64 == NULL) {
        LOG.error("not enough memory to instantiate array...");
    }

    memset(base64, 0, encodeLen + 1);

    b64_encode(base64, ptrFileContent, numBytes);

finally:

    if (ptrFileContent != NULL)
        delete ptrFileContent; ptrFileContent = NULL;

    return base64;
}

*/

/*
* Method to understand if file are new, deleted or updated.
*/

void setModifiedItemsFILE(Container* c, int dataType, const wchar_t* path, const wchar_t* dir) {

    LOG.debug("Enter in setModifiedItemsFILE... ");

    wchar_t   localTemp     [DIM_LOCAL_TEMP];
    wchar_t   internalPath  [DIM_LOCAL_TEMP];
    wchar_t   filterPattern [DIM_LOCAL_TEMP];
    wchar_t   wtime         [DIM_LOCAL_TEMP];

    localTemp[0]        = 0;
    internalPath[0]     = 0;
    wtime[0]            = 0;
    filterPattern[0]    = 0;

    wchar_t** ptrFilenameInDir      = NULL;
    wchar_t*  ptrFileContent        = NULL;
    char* fileContent               = NULL;
    wchar_t** ptrFilenameFromFile   = NULL;
    char* pp = NULL;

    SyncItem* syncItem = NULL;

    int numBytes            = 0;
    int countFileInDir      = 0;
    int countFileInFile     = 0;
    int countNewItems       = 0;
    int countUpdatedItems   = 0;
    int countDeletedItems   = 0;

    int j                   = 1;
    int i                   = 0;
    unsigned long sizeFile  = 0;
    DATE dtime              = 0.0;
    DATE timeInFile         = 0.0;

    BOOL found = FALSE;
    BOOL recursiveInDirectory = FALSE;

    filterPattern[0] = 0;

    wstring key_s;

    if (dataType == constFiles)
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
    else if (dataType == constNotes) {
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
        wcscpy(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {
        SHGetSpecialFolderPath(NULL, internalPath, CSIDL_FAVORITES, FALSE);
        wcscat(internalPath, TEXT("\\"));
        recursiveInDirectory = TRUE;
    }
    /*
    * It avoids to send object greater then MaxObjSize.
    * In next releases it must me inserted into the
    * source.
    */

    //ClientSettings cs(APPLICATION_URI);
    //cs.readConfig();
    int fileRemoved = 0;

    // to retrieve num of files in directory
    getFileList(internalPath, ptrFilenameInDir, &countFileInDir, TRUE, recursiveInDirectory, filterPattern);
    ptrFilenameInDir = new wchar_t*[countFileInDir];

    // to retrieve fileName
    getFileList(internalPath, ptrFilenameInDir, &countFileInDir, FALSE, recursiveInDirectory, filterPattern);

    // to retrieve num of files in files.dat/notes.dat
    countFileInFile = readFilenameFromFile(dataType, path, &ptrFilenameFromFile);

    // if there is no backup file...
    if (!ptrFilenameFromFile)
        return;


    // first element in file is timestamp
    timeInFile = wcstod(ptrFilenameFromFile[0], NULL);

    //
    // to get the new or update items that are files
    //

    for (i = 0; i < countFileInDir; i++) {

        found = FALSE;

        for (j = 1; j <= countFileInFile; j++) {

            if (wcscmp(ptrFilenameInDir[i], ptrFilenameFromFile[j]) == 0) {

                found = TRUE;
                getLastWriteTime(ptrFilenameInDir[i], wtime, &dtime);

                if (dtime > timeInFile) {

                    if (dataType == constFiles) {
                        unsigned long sizeFile = 0;
                        getFileSize(ptrFilenameInDir[i], &sizeFile);
                        if (sizeFile == 0)
                            return;
                        if (sizeFile > getMaxObjectSizeToSend()) {
                            LOG.debug("Found file greater then maxObjectSize(%lu): sizefile = %lu", getMaxObjectSizeToSend(), sizeFile);
                            fileRemoved++;
                            continue;
                        }
                        fileContent = new char[sizeFile];

                        readByteFromFile(ptrFilenameInDir[i], (byte*)fileContent, &numBytes);
                        // only file name without path. (e.g. example.bin)
                        FileData fileData;
                        getFileName(ptrFilenameInDir[i], localTemp);
                        fileData.setName(localTemp);
                        fileData.setSize(numBytes);
                        fileData.setEnc(TEXT("base64"));
                        fileData.setBody(fileContent, numBytes);
                        delete [] fileContent; fileContent = NULL;
                        fileContent = fileData.format();
                        numBytes = strlen(fileContent);

                    }
                    else if (dataType == constNotes) {

                        pp = readTextFromBinaryFileChar(ptrFilenameInDir[i]);
                        getFileName(ptrFilenameInDir[i], localTemp);
                        if (pp != NULL) {
                            wstring note = getXMLNote(pp, localTemp);
                            fileContent = toMultibyte((wchar_t*)note.data());
                            numBytes = strlen(fileContent);
                        }

                    }
                    else if (dataType == constFavorites) {
                        ptrFileContent = readTextFromBinaryFile(ptrFilenameInDir[i]);

                        if (ptrFileContent != NULL) {
                            fileContent = toMultibyte(ptrFileContent);
                            numBytes = strlen(fileContent);
                        }

                        wcscpy(localTemp, ptrFilenameInDir[i]);

                        wchar_t* t = wcsstr(ptrFilenameInDir[i], internalPath);
                        if (t != NULL) {
                            wsprintf(localTemp, t + wcslen(internalPath));
                        }
                        if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                            convertSlashAt(localTemp);

                    }

                    encodeKey (localTemp);
                    key_s = wstring(localTemp);
                    //encodeSpecialChar(key_s);

                    syncItem = new SyncItem((wchar_t*)key_s.data());

                    // without CDATA

                        if (fileContent == NULL) {
                        syncItem->setData("", 1*sizeof(char));
                    }
                    else {
                        syncItem->setData(fileContent, numBytes*sizeof(char));
                    }
                    countUpdatedItems++;

                    c->addItemToUpdatedItems(syncItem);

                    if (syncItem)
                        delete syncItem;
                    syncItem = NULL;

                    if (ptrFileContent != NULL)
                        delete [] ptrFileContent;
                    ptrFileContent = NULL;
                    if (fileContent != NULL)
                        delete [] fileContent;
                    fileContent = NULL;
                    pp = NULL;

                }
                else {

                }
                break;
            }

        }

        if (found == FALSE) {

            if (dataType == constFiles) {

                unsigned long sizeFile = 0;
                getFileSize(ptrFilenameInDir[i], &sizeFile);
                if (sizeFile == 0)
                    return;
                if (sizeFile > getMaxObjectSizeToSend()) {
                    LOG.debug("Found file greater then maxObjectSize(%lu): sizefile = %lu", getMaxObjectSizeToSend(), sizeFile);
                    fileRemoved++;
                    continue;
                }
                fileContent = new char[sizeFile];

                readByteFromFile(ptrFilenameInDir[i], (byte*)fileContent, &numBytes);
                //getFileName(ptrFilenameInDir[i], localTemp);
                FileData fileData;
                getFileName(ptrFilenameInDir[i], localTemp);
                fileData.setName(localTemp);
                fileData.setSize(numBytes);
                fileData.setEnc(TEXT("base64"));
                fileData.setBody(fileContent, numBytes);
                delete [] fileContent; fileContent = NULL;
                fileContent = fileData.format();
                numBytes = strlen(fileContent);
            }
            else if (dataType == constNotes) {

                pp = readTextFromBinaryFileChar(ptrFilenameInDir[i]);
                getFileName(ptrFilenameInDir[i], localTemp);
                if (pp != NULL) {
                    wstring note = getXMLNote(pp, localTemp);
                    fileContent = toMultibyte((wchar_t*)note.data());
                    numBytes = strlen(fileContent);
                }
            }
            else if (dataType == constFavorites) {
                ptrFileContent = readTextFromBinaryFile(ptrFilenameInDir[i]);

                if (ptrFileContent != NULL) {
                    fileContent = toMultibyte(ptrFileContent);
                    numBytes = strlen(fileContent);
                }

                wcscpy(localTemp, ptrFilenameInDir[i]);

                wchar_t* t = wcsstr(ptrFilenameInDir[i], internalPath);

                if (t != NULL) {
                     wsprintf(localTemp, t + wcslen(internalPath));
                }
                if (FILE_SYSTEM_SYNC_SOURCE == TRUE) {
                    convertSlashAt(localTemp);
                }

            }

            encodeKey (localTemp);
            key_s = wstring(localTemp);
            //encodeSpecialChar(key_s);

            syncItem = new SyncItem((wchar_t*)key_s.data());

            if (fileContent == NULL) {
                syncItem->setData("", 1*sizeof(char));
            }
            else {
                syncItem->setData(fileContent, numBytes*sizeof(char));
            }
            countNewItems++;

            c->addItemToNewItems(syncItem);

            if (syncItem)
                delete syncItem;
            syncItem = NULL;

            if (ptrFileContent != NULL)
                delete [] ptrFileContent;
            ptrFileContent = NULL;

            if (fileContent != NULL)
                delete [] fileContent;
            fileContent = NULL;
            pp = NULL;
        }

    }

    //
    // to get the delete items that are files
    //

    for (j = 1; j <= countFileInFile; j++) {

        found = FALSE;
        for (i = 0; i < countFileInDir; i++) {
            if (wcscmp( ptrFilenameFromFile[j], ptrFilenameInDir[i]) == 0) {
                found = TRUE;
            }
        }
        if (found == FALSE) {

            if (dataType == constFiles || dataType == constNotes) {
                getFileName(ptrFilenameFromFile[j], localTemp);
            }
            else if (dataType == constFavorites) {
                wcscpy(localTemp, ptrFilenameFromFile[j]);
                wchar_t* t = wcsstr(ptrFilenameFromFile[j], internalPath);
                if (t != NULL) {
                    wsprintf(localTemp, t + wcslen(internalPath));
                }
                if (FILE_SYSTEM_SYNC_SOURCE == TRUE)
                    convertSlashAt(localTemp);
            }

            encodeKey (localTemp);
            key_s = wstring(localTemp);
            //encodeSpecialChar(key_s);

            syncItem = new SyncItem((wchar_t*)key_s.data());

            c->addItemToDeletedItems(syncItem);

            if (syncItem)
                delete syncItem;
            syncItem = NULL;

            countDeletedItems++;

        }
    }

    // delete array and pointer...
    for (j = 0; j < countFileInDir; j++) {
        if (ptrFilenameInDir[j] != NULL)
            delete ptrFilenameInDir[j];
    }
    if (ptrFilenameInDir != NULL)
        delete [] ptrFilenameInDir;

    for (j = 0; j < countFileInFile; j++) {
        if (ptrFilenameFromFile[j] != NULL)
            delete ptrFilenameFromFile[j];
    }
    if (ptrFilenameFromFile != NULL)
        delete [] ptrFilenameFromFile;

    LOG.debug("Exiting from setModifiedItemsFILE... ");
}

