/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
#include "Winbase.h"
#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>

#include "base/startcmd.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/SettingFunctions.h"
#include "client/SyncClient.h"
#include "pim/ContactSyncSource.h"
#include "pim/CalendarSyncSource.h"
#include "pim/NoteSyncSource.h"
#include "pim/TaskSyncSource.h"
#include "pim/BriefcaseSyncSource.h"

#include "pim/mail/MailSyncSource.h"
#include "client/MailSourceManagementNode.h"
#include "client/DMTClientConfig.h"
#include "filter/SourceFilter.h"
#include "filter/ClauseUtil.h"

#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "syncml/formatter/Formatter.h"
#include "spds/spdsutils.h"
#include "Notify.h"
#include "notify/timed_msgbox.h"
#include "pim/account.h"
#include "OutOfMemoryException.h"

#include "pim/difftime.h"

#include "events/SyncItemListenerClient.h"
#include "events/SyncListenerClient.h"
#include "events/SyncSourceListenerClient.h"
#include "events/SyncStatusListenerClient.h"
#include "events/TransportListenerClient.h"
#include "event/SetListener.h"

#include "HwndFunctions.h"
#include "customization.h"

#include "pim/CustomSyncClient.h"
#include <new.h>
#include <algorithm>
#include <string>
#include <list>
using namespace std;

//
// USE the pim/ClientSettings.h. THis must be shared with the same ClientSettings of the
// FunambolClient
//
#include "pim/ClientSettings.h"

#define OL_MAIL 610

static SyncSourceListenerClient* syncSourceListener ;

// it could be a string. it is a vector because better should be divide the sources
// and not get a string
// TODO: do we need this as a global???
//static vector<wstring> sourceNameForSlow;

/** Prototypes for local functions */
static wstring getMailUnsent(MailSyncSource& mailSource);
static wchar_t* completeMailId(const wchar_t* path, const wchar_t* partialID);
static wchar_t* findMailIdInFile(const wchar_t* fileName, const wchar_t* partialID);
//static int checkRefreshFromServer(int type);

static TCHAR* getMailMaxMsgSize();

/**
 * List of the sources to sync when no subset is specified,
 * in the default order. The list must be NULL terminated.
 */
const static wchar_t *defaultSources[] = {
        MAILS_NAME,
        CONTACTS_NAME,
        APPOINTMENTS_NAME,
        TASKS_NAME,
        NOTES_NAME,
        FAVORITES_NAME,
        FILES_NAME,
        NULL
};


/**
 * Function to handle the memory failure. It is set to be called at new failure
 * using the _set_new_handler() call in the main of the calling program.
 *
 * @param size the size of the memory required
 *
 * @throws an OutOfMemoryException with error code -1
 */
int operatorNewHandler(size_t size)
{
    throw OutOfMemoryException(size, -1);
}


/*
 * Return the sources to sync from the current config.
 */
static vector<const wchar_t*> getActiveSources(ClientSettings &cs,
                                               const wchar_t **sources)
{
    vector<const wchar_t*> ret;
    bool foundFirstSource = false;

    for (int j=0; defaultSources[j]; j++) {
        for (int i=0; sources[i]; i++){
            if (wcscmp(defaultSources[j], sources[i]) == 0) {

                char *name = toMultibyte(sources[i]);               //XXX
                SyncSourceConfig *sc = cs.getSyncSourceConfig(name);

                if (!sc) {
                    LOG.error("No config for source: %s", name);
                }
                else if(strcmp(sc->getSync(), "none") != 0) {
                    ret.push_back(sources[i]);

                    if (!foundFirstSource) {
                        HWND hwnd = HwndFunctions::getWindowHandle();
                        if (hwnd) {
                             int idSpecifiedSource = sourceNameToId(name);
                             SendMessage(hwnd, (WM_APP+4), (WPARAM) 1, (LPARAM) idSpecifiedSource );
                        }
                        foundFirstSource = true;
                    }
                }
                delete [] name;                                     //XXX
                break;
            }
        }
    }

    return ret;
}

/* Perform the synchronization */
static DWORD WINAPI performSync(ClientSettings& config,
                                const wchar_t* path,
                                vector<const wchar_t*> &sources,
                                const wchar_t** ids,
                                const char* mode,
                                BOOL cleanup)
{

    if (sources.empty()) {
        LOG.debug("No sources to sync");
        return 0;
    }

    int ret = 0;
    int mailSourceIndex = -1, result = 0;
    int days = 0, attachSize = 0;
    BOOL blockedByUser = FALSE;

    TCHAR* msxmessagesize = getMailMaxMsgSize();
    long maxMailMessageSize = (wcstol(msxmessagesize, NULL, 10) - 16384) / 1024;
    wchar_t* downloadAge = NULL;
    SourceFilter* filter = NULL;

    ArrayList syncSources;
    char* value = NULL;

    ManagementNode* node = NULL;

    LOG.setLevel(getLOGLevel());

    /*
    * To understand if the briefcase and note directory exist. If no and there are
    * no other source to sync the message is specific
    */
    BOOL briefcaseDirExists = TRUE;
    BOOL noteDirExists = TRUE;

    //
    // Mail configuration read now because it has to have a scope until the end
    //
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");
    MailSyncSourceConfig sc;
    sc = m.getMailSourceConfig(true);
    MailSyncSource mailSource = MailSyncSource (TEXT("mail"), &sc);

    for (int i=0, l=sources.size(); i<l; i++) {
        if (wcscmp(sources[i], CONTACTS_NAME) == 0) {

            ContactSyncSource s(CONTACTS_NAME, OL_CONTACTS, config.getSyncSourceConfig(CONTACTS_NAME_A));
            s.setPath(path);
            s.setMimeType(config.getSyncSourceConfig(CONTACTS_NAME_A)->getType());
            if (mode) {
                s.setSyncMode(syncModeCode(mode));
            }
            syncSources.add(s);
        }
        else if (wcscmp(sources[i], APPOINTMENTS_NAME) == 0) {

            CalendarSyncSource s(APPOINTMENTS_NAME, OL_CALENDAR,
                                 config.getSyncSourceConfig(APPOINTMENTS_NAME_A));
            s.setPath(path);
            s.setMimeType(config.getSyncSourceConfig(APPOINTMENTS_NAME_A)->getType());
            if (mode) {
                s.setSyncMode(syncModeCode(mode));
            }
            syncSources.add(s);
        }
        else if (wcscmp(sources[i], TASKS_NAME)        == 0) {

            TaskSyncSource  s(TASKS_NAME, OL_TASK, config.getSyncSourceConfig(TASKS_NAME_A));
            s.setPath(path);
            s.setMimeType(config.getSyncSourceConfig(TASKS_NAME_A)->getType());
            if (mode) {
                s.setSyncMode(syncModeCode(mode));
            }
            syncSources.add(s); //XXX
        }
        else if (wcscmp(sources[i], FAVORITES_NAME)    == 0) {
/*
            FavoritesSyncSource s(FAVORITES_NAME, OL_FAVORITES);
            s.setPath(path);
            syncSources.add(s);
*/
        }
        else if (wcscmp(sources[i], FILES_NAME)        == 0) {
            //config.readConfigSources(FILES_NAME_A);
            BriefcaseSyncSource  s(FILES_NAME, OL_BRIEFCASE, config.getSyncSourceConfig(FILES_NAME_A));
            s.setPath(path);
            if (mode) {
                s.setSyncMode(syncModeCode(mode));
            }
            wchar_t* dir = toWideChar((config.getWinSyncSourceConfig(FILES_NAME_A))->getSyncDir().c_str());
            if (existsDirectory(dir)) {
                s.setDir(dir);
                syncSources.add(s);

            } else {
                LOG.info("Briefcase dir %S doesn't exists!", dir);
                briefcaseDirExists = FALSE;
            }
            if (dir) { delete [] dir; dir = NULL; }
        }
        else if (wcscmp(sources[i], NOTES_NAME) == 0) {
            //config.readConfigSources(NOTES_NAME_A);
            NoteSyncSource s(NOTES_NAME, OL_NOTES, config.getSyncSourceConfig(NOTES_NAME_A));
            s.setPath(path);
            if (mode) {
                s.setSyncMode(syncModeCode(mode));
            }
            wchar_t* dir = toWideChar((config.getWinSyncSourceConfig(NOTES_NAME_A))->getSyncDir().c_str());
            if (existsDirectory(dir)) {
                s.setDir(dir);
                syncSources.add(s);
            } else {
                LOG.info("Note dir %S doesn't exists!", dir);
                noteDirExists = FALSE;
            }
            if (dir) { delete [] dir; dir = NULL; }
        }
        else if (wcscmp(sources[i], MAILS_NAME) == 0 && doesFunambolAccountExist()) {

            if (mode) {
                mailSource.setSyncMode(syncModeCode(mode));
            }
            wchar_t t[12];
            getFolderToSync(sc, t);

            attachSize = sc.getAttachSize();

            // we want to disable the message size filter for the inclusive items
            // but we need to use the default as for donwload all message
            if (ids)
                attachSize = -1;

            if (attachSize == -1) {
                //reserved value for the remain part of syncml msg
                // 1024 because the filter multiply for 1024
                attachSize = maxMailMessageSize;
            }
            //
            // Set the size from the filter.
            //
            sc.setAttachSize(attachSize*1024);

            mailSource.setMaxMailMessageSize(maxMailMessageSize*1024);
            //
            // the ids is an array of mail id
            //
            if (ids) {
                //
                mailSource.setIsSyncInclusive(true);
                mailSource.setSyncMode(SYNC_ONE_WAY_FROM_SERVER);
                wchar_t* idComplete = completeMailId(path, ids[0]);
                if (idComplete) {
                    char* ss = toMultibyte(idComplete);
                    filter = ClauseUtil::createSourceFilterInclusive(ss, attachSize);
                    delete [] ss;
                    delete [] idComplete;
                }
            } else {
                mailSource.setIsSyncInclusive(false);
                days = sc.getDownloadAge();
                downloadAge = getDayBefore(days);

                filter =
                    ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(), attachSize);
            }

            /*
            * From the filter logic, if bodysize == -1 the attachment are consider in this filter,
            * otherwise not. BTW In the inclusive filter they are always consider
            */
            if (sc.getBodySize() == -1) {
                mailSource.setIncludeAttachment(true);
            } else {
                mailSource.setIncludeAttachment(false);
            }
            mailSource.setPath(path);
            mailSource.setFolderToSync(t);
            mailSource.setFilter(filter);

            syncSources.add(mailSource);
        }
    }

    // only to cleanup all the items of the sourcess
    if (cleanup) {

        int l = syncSources.size();
        for (int i=0; i<l; i++) {
            const wchar_t *name = ((SyncSource*)syncSources[i])->getName();
            if (wcscmp(name, TEXT("mail")) == 0) {
                mailSource.beginSync();
                wchar_t t[12];
                // set all the folders to clear
                t[0] = 'I'; t[1] = 'O'; t[2] = 'D'; t[3] = 'S'; t[4] = 'T'; t[5] = '0';
                mailSource.setFolderToSync(t);
                mailSource.clearAll();
                config.getWinSyncSourceConfig("mail")->setLast(0);
            } else {
                char* nn = toMultibyte(name);
                ((SyncSource*)syncSources[i])->getFirstItemKey();
                if(nn) {
                    config.getWinSyncSourceConfig(nn)->setLast(0);
                    delete [] nn;
                }

            }
        }
        config.save();
        return 0;
    }

    if (syncSources.size() == 0) {
            LOG.debug("The source array to sync is empty");
        if (blockedByUser) {
            LOG.debug("%s", "No sources to sync: blockedByUser");
            ret = -20;
        } else if (briefcaseDirExists == FALSE) {
            LOG.debug("%s", "No sources to sync: briefcaseDirExists false");
            ret = -25;
        } else if (noteDirExists == FALSE) {
            LOG.debug("%s", "No sources to sync: noteDirExists false");
            ret = -26;
        } else {
            LOG.debug("%s", "No sources to sync");
            ret = -10;
        }

    } else {
        //
        // Now we can create the Sync4jClient object and kick off the sync
        //
        // FIXME: adjust CustomSyncClient to make the hard work
        // the main should be as light as possible.
        CustomSyncClient syncClient;

        if (ids) {
            syncClient.isMailInclusive = TRUE;
        }
        else {
            syncClient.isMailInclusive = FALSE;
        }

        syncSourceListener->sc = &syncClient;

        int i=0;
        int l = syncSources.size();

        const int ml = 10;
        SyncSource* s_array[ml];
        for (i=0; (i<l) && (i<ml-1); ++i) {
            const wchar_t *name = ((SyncSource*)syncSources[i])->getName();
            if (wcscmp(name, TEXT("mail")) == 0) {
                s_array[i] = &mailSource;
            } else {
                s_array[i] = (SyncSource*)syncSources[i];
            }
            syncClient.namesOfSources.push_back(name); //XXX
        }
        s_array[i] = NULL;

        ret = syncClient.sync(config, s_array);

        if (ret == 0) {
            config.save();
        }
        else if (ret > 299 && ret < 599) {
            // try to discover if there are sources that are ok...
            SyncSource* s = NULL;
            for (int i = 0; (s = s_array[i]) != NULL; i++) {
                SyncSourceReport* sr = s->getReport();
                if (sr->checkState()) {
                    //SyncSourceConfig& sc = s->getConfig();
                    //ClientSettings cs(APPLICATION_URI);
                    //cs.saveSyncSourceConfig(sc);
                    config.saveSyncSourceConfig(s->getConfig().getName());
                }
                else{
                    //this is to check if there's a problem with the email account
                    //if the email account on the server is not set we find it and
                    //trasform the error code from 506 to 588 so we can catch this
                    //error in uidlg.cpp and popup the right allert message to the
                    //user
                    const char* errorToCheck = "Error Setting Preliminary Info: Error getting Account information " ;
                    if( sr->getLastErrorCode() == 506 &&\
                        strcmp(sr->getLastErrorMsg(),errorToCheck) == 0 &&\
                        strcmp(sr->getSourceName(), "mail") == 0){
                            ret = 588;
                    }
                }
            }
        }

        // check source states and signal them to UI
        SyncSource* s = NULL;
        int sourceID = -1;
        for (int i = 0; (s = s_array[i]) != NULL; i++) {
            SyncSourceReport* sr = s->getReport();
            if(strcmp(sr->getSourceName(),CONTACTS_NAME_A) ==0)
                sourceID = SOURCE_CONTACTS;
            if(strcmp(sr->getSourceName(),APPOINTMENTS_NAME_A) ==0)
                sourceID = SOURCE_CALENDAR;
            if(strcmp(sr->getSourceName(),TASKS_NAME_A) ==0)
                sourceID = SOURCE_TASKS;
            if(strcmp(sr->getSourceName(),FILES_NAME_A) ==0)
                sourceID = SOURCE_BRIEFCASE;
            if(strcmp(sr->getSourceName(),MAILS_NAME_A) ==0)
                sourceID = SOURCE_MAIL;
            if(strcmp(sr->getSourceName(),NOTES_NAME_A) ==0)
                sourceID = SOURCE_NOTES;

            if (sr->checkState())
                SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SOURCE_STATE, sourceID, SYNCSOURCE_STATE_OK);
            else
                SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SOURCE_STATE, sourceID, SYNCSOURCE_STATE_NOT_SYNCED);
        }


        if (mailSource.getIsMailInOutbox() && ret == 0) {
             LOG.debug("performSync: there are mails in outbox");
        }

        if (mailSource.getIsMailInInbox()) {
            LOG.info("performSync: new mail in inbox.");
            /*
            // temporary removed default funambol sound. Use the usual
            // set in the ppc
            LOG.info("performSync: new mail in inbox, playing sound.");
            wstring s(path);
            s += TEXT("\\newmessage.wav");

            if (!PlaySound(s.c_str(), NULL, SND_SYNC | SND_NOSTOP)) {
                LOG.info("Could not play sound: %s", s.c_str());
                MessageBeep(MB_ICONASTERISK);
            }
            */
        }

        if (mailSource.getFailedSendMailInOutbox()) {
            LOG.error("performSync: there was an error sending an email, alerting the user.");
            wstring message = getMailUnsent(mailSource);
            MessageBox(NULL, message.c_str(), TEXT("Alert"),
                    MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
        }

        LOG.debug("Sync ended multiple sources.");
    }
    return ret;
}

/*
 * Entry point to kickoff the synchronization process, both specifying the
 * sources or not. The default is to make all the configured sources.
 *
 * @param path    : the install path of the SyncClient PIM application on
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 * @param ids :     NULL terminated array of strings containing the id of the mail
 *                  that must be synched with a INCLUSIVE filter. Note that this parameter
 *                  could be valorized only when there is only a source and this source is mail
 *
 * To avoid a server bug and send every time the capabilities to the server
 * in order to receive the NumberOfChanges tag
 * RESET the hashDevInfo flag in the registry
 */

DWORD WINAPI synchronize (
                const wchar_t* path,
                const wchar_t** sources,
                const wchar_t** ids,
                const char* mode)
{
    LOG.reset();
    int ret = 0;

    HwndFunctions::getWindowHandle();

    SyncItemListenerClient* itemListener = new SyncItemListenerClient();
    SyncListenerClient* syncListener = new SyncListenerClient();
    syncSourceListener  = new SyncSourceListenerClient();
    SyncStatusListenerClient* sstl = new SyncStatusListenerClient();
    TransportListenerClient* tl = new TransportListenerClient();

    setSyncItemListener(itemListener);
    setSyncListener(syncListener);
    setSyncSourceListener(syncSourceListener);
    setSyncStatusListener(sstl);
    setTransportListener(tl);

    //ClientSettings cs(APPLICATION_URI);
    ClientSettings::getInstance();

    string logTitle = ClientSettings::getInstance()->getAccessConfig().getUserAgent();
    logTitle += " Log";
    LOG.reset(logTitle.c_str());

    LOG.info("DeviceID: %s", ClientSettings::getInstance()->getDeviceConfig().getDevID());
    LOG.info("IP address: %s", ClientSettings::getInstance()->getIp().c_str());

    // Get the sources to sync
    const wchar_t** srcList = sources ? sources : defaultSources;
    vector<const wchar_t*> activeList = getActiveSources(*(ClientSettings::getInstance()), srcList);
    // Start the sync
    ret = performSync(*(ClientSettings::getInstance()), path, activeList, ids, mode, false);

    if (ret == -20) {
        LOG.debug("The user stops the sync to avoid slow sync of PIM");
        return ret;
    }

    if (ret == -10) {
        LOG.debug("Mail not set to be synched");
        LOG.debug("No source to sync");
        return -10;
    }

    // release listeners
    if(itemListener)
        { delete itemListener; itemListener=NULL; }
    if(syncListener)
        { delete syncListener; syncListener=NULL; }
    if(syncSourceListener)
        { delete syncSourceListener; syncSourceListener=NULL; }
    if(itemListener)
        { delete itemListener; itemListener=NULL; }
    if(itemListener)
        { delete itemListener; itemListener=NULL; }

    LOG.debug("Synchronize is returning: %d.", ret);
    return ret;
}


/*
 * NOTE: USED ONLY TEMPORANEALLY TO GET A MAIL PARAMETER THAT MUST
 * BE FIXED IN A PROPER WAY
 */
static TCHAR* getMailMaxMsgSize() {
    HKEY key = NULL;
    DWORD res;
    long err = 0;
    TCHAR *p = TEXT("mailMaxMsgSize");
    ULONG dim = 0;
    TCHAR *buf = NULL;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\") ROOT_CONTEXT_W TEXT("\\spds\\sources\\mails"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        sprintf(lastErrorMsg, "Invalid context path: %s", "Software/" ROOT_CONTEXT "/spds/sources/mails");
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            p,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            buf = new TCHAR[dim + 1];

            err = RegQueryValueEx(
                    key,
                    p,
                    NULL,
                    NULL,  // we currently support only strings
                    (UCHAR*)buf,
                    &dim
                    );
        }
    }
    //else MessageBox(NULL,  T("Error"), T("getConfigParameter"), MB_OK);

    if (!buf)
        buf = wstrdup(TEXT(""));

finally:

    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}


/*
* return code -10: no source to sync
*/

DWORD checkStartSync() {

    DWORD pid = 0;
    HANDLE hMutex = NULL;
    // Try to create the mutex
    hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );
    if(hMutex){
        DWORD err = GetLastError();
        if(err == ERROR_ALREADY_EXISTS){
            pid = getProcessPid( TEXT("startsync.exe") );
        }
        CloseHandle( hMutex );
    }
    return pid;

}

/*
* To get the complete mail id. The AdviseSink of the syncml transport returns an array of /id1, /id2
* But the complete id mapped with the server is I/id1, S/id2. So to ask the right mail is necessary to retrieve
* the complete id that is stored in the files containing the cached situation
*/
static wchar_t* findMailIdInFile(const wchar_t* fileName, const wchar_t* partialID) {

    wchar_t* idComplete = NULL;
    wchar_t line[512];
    FILE* f = _wfopen(fileName, TEXT("r"));

    if (f == NULL)
        return NULL;

    while(fgetws(line, 511, f) != NULL) {

        if (wcsstr(line, partialID ) != NULL) {
            if (idComplete = getElementContent(line, TEXT("id"), 0))
                break;

        }
    }
    fclose(f);
    return idComplete;
}

static wchar_t* completeMailId(const wchar_t* path, const wchar_t* partialID) {

    bool isLast = false;
    wchar_t fileName[255];
    wchar_t* idComplete = NULL;

    WIN32_FIND_DATA FileData;
    wchar_t filter[255];
    wsprintf(filter, TEXT("%s\\mails_*"), path);
    HANDLE hFind = FindFirstFile(filter, &FileData);

    if (hFind == INVALID_HANDLE_VALUE) {
        LOG.debug("No mails_ file found.");
        return NULL;
    }

    while (!isLast) {
        wsprintf(fileName, TEXT("%s\\%s"), path, FileData.cFileName);
        idComplete = findMailIdInFile(fileName, partialID);

        if (idComplete) { // id completed found
            break;
        } else {
            if (!FindNextFile(hFind, &FileData))
                isLast = TRUE;
        }
    }

    FindClose(hFind);
    return idComplete;

}

/*
* Return the number of email that are not sent in outbox folder
* When the connector will send the messate cause for every item
* we can get the message too
*/

static wstring getMailUnsent(MailSyncSource& mailSource) {

    wstring messageToRet;
    SyncSourceReport* rep = mailSource.getReport();
    int res = rep->getItemReportFailedCount(SERVER, COMMAND_ADD);
    int number = 0, i = 0;
    BOOL found = FALSE;
    for (i = 0; i < res; i++) {
        ItemReport* r = rep->getItemReport(SERVER, COMMAND_ADD, i);
        if (wcsstr(r->getId(), L"O/") != 0 && (r->getStatus() == 500 || r->getStatus() == 418 || r->getStatus() == 419)) {
            found = TRUE;
            if (r->getStatusMessage()) {
                LOG.debug("The outbox mail (id: %S) cannot be sent. Server says: %S", r->getId(), r->getStatusMessage());
                messageToRet = r->getStatusMessage();
                if (messageToRet.length() > 130) {
                    messageToRet = messageToRet.substr(0,130);
                    messageToRet.append(L"...");
                }

            }
            number++;
        }
    }

    if (!found) {
        res = rep->getItemReportFailedCount(SERVER, COMMAND_REPLACE);
        for (i = 0; i < res; i++) {
            ItemReport* r = rep->getItemReport(SERVER, COMMAND_REPLACE, i);
            if (wcscmp(r->getId(), L"O/") != 0 && (r->getStatus() == 500 || r->getStatus() == 418 || r->getStatus() == 419)) {
                if (r->getStatusMessage()) {
                    LOG.debug("The outbox mail (id: %S) cannot be sent. Server says: %S", r->getId(), r->getStatusMessage());
                    messageToRet = r->getStatusMessage();
                    if (messageToRet.length() > 130) {
                        messageToRet = messageToRet.substr(0,130);
                        messageToRet.append(L"...");
                    }
                }
                number++;
            }
        }

    }

    if (number == 1) {
        /*
        * Added extension to the name if there isn't one
        */
        wstring msg(messageToRet);
        std::transform(messageToRet.begin(), messageToRet.end(), msg.begin(), tolower);

        if (messageToRet == L"" || (msg.find(L"error sending") == wstring::npos &&
                                    msg.find(L"error adding") == wstring::npos)
                                ||  msg.find(L"com.funambol") != wstring::npos ) {
            messageToRet = ONE_OUT_MESSAGE_NOT_SENT;
        } else {
            messageToRet.insert(0, OUT_MESSAGE_NOT_SENT);
        }
    } else {
        messageToRet = MORE_OUT_MESSAGE_NOT_SENT;
    }

    return messageToRet;
}

DWORD WINAPI cleanupAllItems (vector<const wchar_t*> sources) {

    wchar_t path[DIM_PATH];
    getClientConfigurationInternal(NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);

    int retValue = performSync(*(ClientSettings::getInstance()), path, sources, NULL, NULL, true);
    return retValue;
}


/**
 * Reads the property 'queueList' from Win registry and returns a
 * NULL terminated array of source names to sync (new buffer).
 * The 'queueList' is cleared at the end.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync.
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *
 * @return  the NULL terminated array of source names to sync
 */
wchar_t** readQueueList() {

    wchar_t** sources    = NULL;
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    list<wstring> sourceList;
    list<wstring>::iterator it;

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CTP_QUEUE_PUSH);
    if ( !value || strcmp(value, "1") ) {
        // Not using queuePush
        goto finally;
    }

    // Read current value of 'queueList' from registry
    if (value) {
        delete [] value; value = NULL;
    }
    value = node->readPropertyValue(CTP_QUEUE_LIST);
    if (!value) {
        goto finally;
    }
    mask = atoi(value);
    if (mask <= 0) {
        goto finally;
    }


    // Create the NULL terminated array of source names
    // (first need the number of sources, to allocate the WCHAR**)
    sourceList.clear();
    if (mask &  1)  sourceList.push_back(MAILS_NAME       );
    if (mask &  2)  sourceList.push_back(CONTACTS_NAME    );
    if (mask &  4)  sourceList.push_back(APPOINTMENTS_NAME);
    if (mask &  8)  sourceList.push_back(TASKS_NAME       );
    if (mask & 16)  sourceList.push_back(NOTES_NAME       );
    if (mask & 32)  sourceList.push_back(FILES_NAME       );

    int i=0;
    sources = new wchar_t*[sourceList.size()+1];
    it = sourceList.begin();
    while (it != sourceList.end()) {
        sources[i++] = wstrdup((*it).c_str());
        it++;
    }
    sources[i] = NULL;


    // Clear the 'queueList' value on registry.
    node->setPropertyValue("queueList", "0");
    node->setPropertyValue("queuePush", "1");

finally:

    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

    return sources;
}

/**
 * Reads and updates the property 'queueList' from Win registry with
 * the list of sources passed as parameter.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync. This value will be read
 * at the end of the sync, and a new sync will automatically start (if value != 0).
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *
 * @param sources  the NULL terminated array of source names
 */
void updateQueueList(const wchar_t** sources) {

    if (!sources) {
        return;
    }
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    char buf[4];

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CTP_QUEUE_PUSH);
    if ( !value || strcmp(value, "1") ) {
        // Not using queuePush
        goto finally;
    }

    // Read current value of 'queueList' from registry
    if (value) {
        delete [] value; value = NULL;
    }
    value = node->readPropertyValue(CTP_QUEUE_LIST);
    if (!value) value = stringdup("0");
    mask = atoi(value);

    //
    // Update the bit-mask with current sources
    //
    for (int i=0; sources[i]; i++) {
        if (!wcscmp(sources[i], MAILS_NAME)) {
            mask = mask | 1;
        }
        else if (!wcscmp(sources[i], CONTACTS_NAME)) {
            mask = mask | 2;
        }
        else if (!wcscmp(sources[i], APPOINTMENTS_NAME)) {
            mask = mask | 4;
        }
        else if (!wcscmp(sources[i], TASKS_NAME)) {
            mask = mask | 8;
        }
        else if (!wcscmp(sources[i], NOTES_NAME)) {
            mask = mask | 16;
        }
        else if (!wcscmp(sources[i], FILES_NAME)) {
            mask = mask | 32;
        }
    }

    // Write 'queueList' value to registry
    sprintf(buf, "%d", mask);
    node->setPropertyValue(CTP_QUEUE_LIST, buf);
    node->setPropertyValue(CTP_QUEUE_PUSH, "1");

finally:
    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;
}

wchar_t** readPIMModifications() {

    wchar_t** sources    = NULL;
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    list<wstring> sourceList;
    list<wstring>::iterator it;

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    value = node->readPropertyValue(CP_PUSH_LIST);
    if (!value) {
        goto finally;
    }
    mask = atoi(value);
    if (mask <= 0) {
        goto finally;
    }

    // Create the NULL terminated array of source names
    // (first need the number of sources, to allocate the WCHAR**)
    sourceList.clear();
    if (mask &  2)  sourceList.push_back(CONTACTS_NAME    );
    if (mask &  4)  sourceList.push_back(APPOINTMENTS_NAME);
    if (mask &  8)  sourceList.push_back(TASKS_NAME       );

    int i=0;
    sources = new wchar_t*[sourceList.size()+1];
    it = sourceList.begin();
    while (it != sourceList.end()) {
        sources[i++] = wstrdup((*it).c_str());
        it++;
    }
    sources[i] = NULL;

    // NO: to reset the property, call 'resetPIMModifications'.
    //node->setPropertyValue(CP_PUSH_LIST, "0");

finally:

    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

    return sources;
}


void resetPIMModifications(const wchar_t** sources) {

    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    char buf[4];

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    if (sources == NULL) {
        node->setPropertyValue(CP_PUSH_LIST, "0");
        goto finally;
    }

    value = node->readPropertyValue(CP_PUSH_LIST);
    if (!value) {
        goto finally;
    }

    mask = atoi(value);

    //
    // Update the bit-mask with current sources
    //
    for (int i=0; sources[i]; i++) {

        if (!wcscmp(sources[i], CONTACTS_NAME)) {
            mask = mask & (~2);
        }
        else if (!wcscmp(sources[i], APPOINTMENTS_NAME)) {
            mask = mask & (~4);
        }
        else if (!wcscmp(sources[i], TASKS_NAME)) {
            mask = mask & (~8);
        }
    }

    sprintf(buf, "%d", mask);
    node->setPropertyValue(CP_PUSH_LIST, buf);

finally:

    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

}



wchar_t** getCompleteList(wchar_t** sLists, wchar_t** cpLists) {
    
    if (sLists == NULL && cpLists == NULL) { return NULL;    }
    if (sLists == NULL)                    { return cpLists; }
    if (cpLists == NULL)                   { return sLists;  }
    
    list<wstring> sourceList;
    list<wstring>::iterator it;
    int queue = 0, cp = 0, mask = 0, i = 0;
    for (i = 0; sLists[i]; i++) {                      
        if (wcscmp(sLists[i], MAILS_NAME) == 0)             { queue = queue | 1; } 
        else if (wcscmp(sLists[i], CONTACTS_NAME) == 0)     { queue = queue | 2; } 
        else if (wcscmp(sLists[i], APPOINTMENTS_NAME) == 0) { queue = queue | 4; } 
        else if (wcscmp(sLists[i], TASKS_NAME) == 0)        { queue = queue | 8; }
    }
    for (i = 0; cpLists[i]; i++) {                      
        if (wcscmp(cpLists[i], CONTACTS_NAME) == 0)          { cp = cp | 2; } 
        else if (wcscmp(cpLists[i], APPOINTMENTS_NAME) == 0) { cp = cp | 4; } 
        else if (wcscmp(cpLists[i], TASKS_NAME) == 0)        { cp = cp | 8; }
    }

    mask = queue | cp;
    sourceList.clear();
    if (mask &  1)  sourceList.push_back(MAILS_NAME       );
    if (mask &  2)  sourceList.push_back(CONTACTS_NAME    );
    if (mask &  4)  sourceList.push_back(APPOINTMENTS_NAME);
    if (mask &  8)  sourceList.push_back(TASKS_NAME       );

    i = 0;
    wchar_t** sources = new wchar_t*[sourceList.size()+1];
    it = sourceList.begin();
    while (it != sourceList.end()) {
        sources[i++] = wstrdup((*it).c_str());
        it++;
    }
    sources[i] = NULL;

    return sources;
} 