/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <stdio.h>

#include "base/Log.h"
#include "spds/spdsutils.h"
#include "spds/Constants.h"
#include "base/util/utils.h"
#include "vocl/WinContact.h"


#include "pim/Utils.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/POOMFileManagement.h"
#include "pim/POOMClient2Server.h"



void setAllItemsPOOM(Container* c, int dataType, wchar_t* path) {
    LOG.debug("Enter setAllItemsPOOM... ");

    IP_OUTLOOK_APP* polApp = getOutlookApp();
    if (polApp == NULL) {
        LOG.error("setAllItemsPOOM: can't open PocketOutlook session");
        return;
    }

    if (dataType == olFolderCalendar && !isWindowsMobile5()) {
        normalizeAppointment(polApp, dataType, path);
    }

    wchar_t localTemp [DIM_LOCAL_TEMP];

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    if( FAILED(polApp->GetDefaultFolder (dataType, &pFolder)) ) {
        LOG.error("setAllItemsPOOM: can't open folder: %d", dataType);
        return;
    }

    pFolder->get_Items (&pItems);

    int countItems = 0;

    pItems->get_Count(&countItems);

    SyncItem* syncItem = NULL;
    for (int j=0; j<countItems; j++) {
        long oid = 0;

        // Item values start from element number 1 not 0!!!!!
        // from dataType choose if a Contact or a Appointment
        // leave 2 populateString to simplify
        if (dataType == olFolderContacts) {
            pItems->Item (j + 1, (IDispatch**)&pContact);
            pContact->get_Oid(&oid);
            // set these objlist to write to file...
            pContact->Release ();
        }
        else if (dataType == olFolderCalendar){
            pItems->Item (j + 1, (IDispatch**)&pAppointment);
            pAppointment->get_Oid(&oid);
            pAppointment->Release();
        }
        else if (dataType == olFolderTasks){
            pItems->Item (j + 1, (IDispatch**)&pTask);
            pTask->get_Oid(&oid);
            pTask->Release();
        }
        wsprintf(localTemp, TEXT("%i"), oid);

        syncItem = new SyncItem(localTemp);

        c->addItemToAllItems(syncItem);

        if (syncItem) {
            delete syncItem;
            syncItem = NULL;
        }

    }

    pItems->Release ();
    pFolder->Release ();

    releaseOutlookApp(polApp);

    wsprintf(localTemp, SET_ALL_ITEMS, countItems);
    LOG.debug("Exiting setAllItemsPOOM... ");
 }

 /*
Set modified Items for FAST_SYNC from PPC to server
*/

void setModifiedItemsPOOM(Container* c, int dataType, wchar_t* path, bool isSIF, map<long,long> &currentOidHash) {

    LOG.debug("Enter setModifiedItemsPOOM... ");
    wchar_t localTemp [DIM_LOCAL_TEMP];

    IP_OUTLOOK_APP* polApp = getOutlookApp();
    if (polApp == NULL)
        return;

    IAppointment *pAppointment;
    ITask        *pTask;

    int countItems = 0;

    vector<long> newOid;
    vector<long> updatedOid;
    vector<long> deletedOid;

    //bool isSIF = checkSIF(dataType);

    // only if dataType is appointment
    if (dataType == olFolderCalendar && !isWindowsMobile5())
            normalizeAppointment(polApp, dataType, path);

    fillModifiedOidArray (newOid, updatedOid, deletedOid, polApp, dataType, path, currentOidHash) ;

    SyncItem* syncItem = NULL;

    // for every oid array...
    unsigned int i = 0; //-- unsigned for avoiding "signed/unsigned mismatch"  warning
    long oid = 0;
    for (i = 0; i < newOid.size(); i++) {
        wstring stringItem;

        oid = newOid[i];
        wsprintf(localTemp, TEXT("%i"), oid);

        // use the IItem interface of IPoutlookApp2 wm5
        if (dataType == olFolderContacts) { // it's a contact

            IItem* ppItem = NULL;
            polApp->GetItemFromOidEx(oid, 0, &ppItem);
            WinContact* winC = contactToWinContact(ppItem, isSIF);
            if(winC) {
                stringItem = winC->toString();
                delete winC;
            }
            if (ppItem) {
                ppItem->Release();
            }
        }
        else if (dataType == olFolderCalendar){ // it's an appointments

            polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);

            if(isSIF)
                populateAppointmentStringItem(stringItem, pAppointment);
            else {
                VObject* vo = AppointmentToVObject(pAppointment);
                if(vo) {
                    stringItem = vo->toString();
                    delete vo;
                }
            }
            pAppointment->Release ();
        }
        else if (dataType == olFolderTasks){ // it's an task

            polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
            if(isSIF)
                populateTaskStringItem(stringItem, pTask);
            else {
                VObject* vo = TaskToVObject(pTask);
                if(vo) {
                    stringItem = vo->toString();
                    delete vo;
                }
            }
            pTask->Release ();
        }
        // Set the SyncItem

        // newItems[i] = new SyncItem(localTemp);
        syncItem = new SyncItem(localTemp);


        //
        // Now Sources works in clear mode. The encoding in b64 is demanded to api side
        //

        char* data = toMultibyte((wchar_t*)stringItem.data());
        syncItem->setData(data, strlen(data)*sizeof(char));
        delete [] data; data = NULL;

        syncItem->setState(SYNC_STATE_NEW);

        c->addItemToNewItems(syncItem);

        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
        stringItem.clear();
    }

    i=0;
    oid = 0;
    for (i = 0; i < updatedOid.size(); i++) {
        wstring stringItem;
        oid = updatedOid[i];
        wsprintf(localTemp, TEXT("%i"), oid);

        // use the IItem interface of IPoutlookApp2 wm5
        if (dataType == olFolderContacts) { // it's a contact

            IItem* ppItem = NULL;
            polApp->GetItemFromOidEx(oid, 0, &ppItem);
            WinContact* winC = contactToWinContact(ppItem, isSIF);
            if(winC) {
                stringItem = winC->toString();
                delete winC;
            }
            if (ppItem) {
                ppItem->Release();
            }

        }
        else if (dataType == olFolderCalendar){ // it's an appointments
            polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);
            if(isSIF)
                populateAppointmentStringItem(stringItem, pAppointment);
            else {
                VObject* vo = AppointmentToVObject(pAppointment);
                if(vo) {
                    stringItem = vo->toString();
                    delete vo;
                }
            }
            pAppointment->Release ();
        }
        else if (dataType == olFolderTasks){ // it's a task
            polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
            if(isSIF)
                populateTaskStringItem(stringItem, pTask);
            else {
                VObject* vo = TaskToVObject(pTask);
                if(vo) {
                    stringItem = vo->toString();
                    delete vo;
                }
            }
            pTask->Release ();
        }

        syncItem = new SyncItem(localTemp);

        //
        // Now Sources works in clear mode. The encoding in b64 is demanded to api side
        //

        char* data = toMultibyte((wchar_t*)stringItem.data());
        syncItem->setData(data, strlen(data)*sizeof(char));
        delete [] data; data = NULL;

        syncItem->setState(SYNC_STATE_UPDATED);
        c->addItemToUpdatedItems(syncItem);

        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
        stringItem.clear();
    }

    i=0;
    oid = 0;
    for (i = 0; i < deletedOid.size(); i++) {
        oid = deletedOid[i];
        wsprintf(localTemp, TEXT("%i"), oid);

        syncItem = new SyncItem(localTemp);

        c->addItemToDeletedItems(syncItem);

        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }

    }

    releaseOutlookApp(polApp);

    LOG.debug("Exiting setModifiedItemsPOOM... ");

}

/*
fill array new, update, delete with OID of element to modify. This function can be used by every source.
This function call funcion that use the alghorytm to retrieve OID modified
*/
void fillModifiedOidArray (vector<long>& newOid, vector<long>& updateOid,
                           vector<long>& deleteOid,
                           IP_OUTLOOK_APP *polApp, int dataType, wchar_t* path,
                           map<long,long> &currentOidHash) {

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;


    if( FAILED(polApp->GetDefaultFolder (dataType, &pFolder)) ) {
        LOG.error("fillModifiedOidArray: can't open folder: %d", dataType);
        return;
    }

    pFolder->get_Items (&pItems);

    int currentCountItems = 0;
    vector<long> currentOid;
    vector<long> currentHash;

    pItems->Sort(TEXT ("[Oid]"), FALSE);
    pItems->get_Count(&currentCountItems);

    long previousCountItems = 0;
    vector<long> previousOid;
    vector<long> previousHash;

    // TODO:
    readFromFile(&previousCountItems, previousOid, previousHash, dataType, path);

    int i = 0;
    int j = 0;
    int lastFound = -1;

    long currentLongOid = 0;
    long currentLongHash = 0;

    for (i = 0; i < currentCountItems; i++) {

        if (dataType == olFolderContacts) {

            /*
            * Check to aim the right contact update from the version before 6.5.x in which
            * the crc and the contact sif where calculated using the IpOutlookApp interface
            * and using the IContact object. Since 6.5.x the crc and the IpOutlookApp2 interface
            * permits to get other fields like picture...
            */
            if (getContactUpdateFlag() == true) {
                currentLongOid = 0;
                pItems->Item (i + 1, (IDispatch**)&pContact);

                pContact->get_Oid(&currentLongOid);
                currentLongHash = calculateContactHash(pContact);
                // set these objlist to write to file...
                currentOid.push_back(currentLongOid);
                currentHash.push_back(currentLongHash);
                pContact->Release ();
            } else {
                currentLongOid = 0;
                IDispatch* pDisp  = NULL;
                IItem*     ppItem = NULL;
                pItems->Item (i + 1, (IDispatch**)&pDisp);

                if (pDisp == NULL) {
                    LOG.error("fillModifiedOidArray: invalid item %d", i+1);
                    continue;
                }
                HRESULT hr = pDisp->QueryInterface(IID_IItem, (LPVOID*)&ppItem);
                ppItem->get_Oid(&currentLongOid);
                WinContact* winC = contactToWinContact(ppItem, false);
                if (winC) {
                    currentLongHash = winC->getCRC();
                    // set these objlist to write to file...
                    currentOid.push_back(currentLongOid);
                    currentHash.push_back(currentLongHash);
                    currentOidHash[currentLongOid] = currentLongHash;
                    delete winC;
                    ppItem->Release();
                    pDisp->Release();
                }
                else {
                    LOG.error("fillModifiedOidArray: error converting to WinContact.");
                }
            }
        }
        else if (dataType == olFolderCalendar){ // for appointment
            long internalOid = 0;

            pItems->Item (i + 1, (IDispatch**)&pAppointment);

            pAppointment->get_Oid(&internalOid);
            currentLongHash = calculateAppointmentHash(pAppointment);
            currentLongOid = internalOid;
            // set these objlist to write to file...
            currentOid.push_back(currentLongOid);
            currentHash.push_back(currentLongHash);
            pAppointment->Release ();
        }
        else if (dataType == olFolderTasks){ // for appointment
            long internalOid = 0;
            pItems->Item (i + 1, (IDispatch**)&pTask);
            pTask->get_Oid(&internalOid);
            currentLongHash = calculateTaskHash(pTask);
            currentLongOid = internalOid;
            // set these objlist to write to file...
            currentOid.push_back(currentLongOid);
            currentHash.push_back(currentLongHash);
            pTask->Release ();
        }
        int itemFound = 0;
        j = lastFound + 1;

        while ((j < previousCountItems && itemFound == 0)) {
            long previousLongOid = previousOid[j];

            if (currentLongOid == previousLongOid) {
                itemFound = 1;
                lastFound = j;

                long previousLongHash = previousHash[j];

                if (currentLongHash != previousLongHash) {
                    // if different hash then into updateOid to pass to setModifiedItems
                    updateOid.push_back(currentLongOid);

                } else {
                    // nothing to do...
                }
            }
            j++;
        }
        if (itemFound == 0) {
            newOid.push_back(currentLongOid);
        }


    } // fine for

    i = 0;
    lastFound = -1;

    for (j = 0; j < previousCountItems; j++) {

        int itemFound = 0;
        i = lastFound + 1;

        long previousLongOid = previousOid[j];

        int count = currentOid.size();
        while ((i < count && itemFound == 0)) {
            long currentLongOid = currentOid[i];

            if (currentLongOid == previousLongOid) {
                itemFound = 1;
                lastFound = i;
            }
            i++;
        }

        if (itemFound == 0) {
            deleteOid.push_back(previousLongOid);
        }

    } // fine for

    pItems->Release ();
    pFolder->Release ();


}

/**
* Used by the getFirstItem and getNextItem of the sync source to fill the item.
* For the slow sync
*/
void fillSyncItem(SyncItem* syncItem, int dataType, bool isSIF, map<long,long> &currentOidHash) {

    IP_OUTLOOK_APP* polApp = getOutlookApp();
    if (polApp == NULL)
        return;

    IAppointment *pAppointment;
    ITask        *pTask;

    //bool isSIF = checkSIF(dataType);
    wstring stringItem;
    long oid = wcstol(syncItem->getKey(), NULL, 10);

    //
    // we start to use the IPOutlookApp2 to use the IItem object
    //
    if (dataType == olFolderContacts) { // it's a contact

        IItem* ppItem = NULL;
        polApp->GetItemFromOidEx(oid, 0, &ppItem);

        WinContact* winC = contactToWinContact(ppItem, isSIF);

        long crc = winC->getCRC();
        currentOidHash[oid] = crc;

        stringItem = winC->toString();
        if (winC) {
            delete winC;
        }
        if (ppItem) {
            ppItem->Release();
        }
    }
    else if (dataType == olFolderCalendar){ // it's an appointments

        polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);

        if(isSIF)
            populateAppointmentStringItem(stringItem, pAppointment);
        else {
            VObject* vo = AppointmentToVObject(pAppointment);
            if(vo) {
                stringItem = vo->toString();
                delete vo;
            }
        }
        pAppointment->Release ();
    }
    else if (dataType == olFolderTasks){ // it's an task

        polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
        if(isSIF)
            populateTaskStringItem(stringItem, pTask);
        else {
            VObject* vo = TaskToVObject(pTask);
            if(vo) {
                stringItem = vo->toString();
                delete vo;
            }
        }
        pTask->Release ();
    }
    //
    // Now Sources works in clear mode. The encoding in b64 is demanded to api side
    //

    char* data = toMultibyte((wchar_t*)stringItem.data());
    syncItem->setData(data, strlen(data)*sizeof(char));
    delete [] data; data = NULL;

    releaseOutlookApp(polApp);

}

int deleteAllItems(int dataType) {

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    IFolder *pFolder;
    IPOutlookItemCollection *pItems;

    IP_OUTLOOK_APP* polApp = getOutlookApp();
    if (polApp == NULL)
        return -1;

    if( FAILED(polApp->GetDefaultFolder (dataType, &pFolder)) ) {
        LOG.error("deleteAllItems: can't open folder: %d", dataType);
        return -1;
    }

    pFolder->get_Items (&pItems);
    int numItems = 0;

    pItems->get_Count(&numItems);

    LOG.info("Items to delete: %i", numItems);

    for (int i = 1; i <=numItems; i++) {
        try {
            if (dataType == olFolderContacts) { // it's a contact
                pItems->Item(1,(IDispatch **)&pContact);

                pContact->Delete();
                pContact->Release();

            }
            else if (dataType == olFolderCalendar){ // it's an appointments
                pItems->Item(1,(IDispatch **)&pAppointment);
                //pAppointment->Cancel();   it sends the cancellation to the recipients
                pAppointment->Delete();
                pAppointment->Release();

            }
            else if (dataType == olFolderTasks){ // it's an task
                pItems->Item(1,(IDispatch **)&pTask);
                pTask->Delete();
                pTask->Release();

            }
        } catch(...) {
            LOG.debug("Exception in deleting all items. Continue until num items: %i", numItems);
        }

    }

    pItems->Release ();
    pFolder->Release ();

    releaseOutlookApp(polApp);
    return numItems;

}

/*
* Call function to remove all the appointment exceptions and transform them into
* normal appointment with only delete exceptions
*/
void normalizeAllExceptions() {

    IP_OUTLOOK_APP* polApp = getOutlookApp();
    if (polApp == NULL)
        return;

    normalizeAppointmentExceptions(polApp);

    releaseOutlookApp(polApp);
}
