/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "spdm/Constants.h"

#include "pim/SettingFunctions.h"
#include <winioctl.h>
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "base/util/utils.h"
#include <uniqueid.h>
#include <commctrl.h>
#include <aygshell.h>
#include <sipapi.h>
#include <cfgmgrapi.h>

#include <string>

#define MAX_DEVID 50


#define IOCTL_HAL_GET_DEVICEID CTL_CODE(FILE_DEVICE_HAL, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)

extern "C" __declspec(dllimport) BOOL KernelIoControl(
                                                        DWORD dwIoControlCode,
                                                        LPVOID lpInBuf,
                                                        DWORD nInBufSize,
                                                        LPVOID lpOutBuf,
                                                        DWORD nOutBufSize,
                                                        LPDWORD lpBytesReturned
);

/*
* Get the unique Device Identifier from PPC 2002 - 2003
* see more at http://www.pocketpcdn.com/articles/serial_number2002.html
*
* @param deviceId     : the retrieved device id
*/

wstring GetSerialNumberFromKernelIoControl() {

    DWORD dwOutBytes = 0;
    const int nBuffSize = 2048;
    byte arrOutBuff[nBuffSize];
    BOOL bRes = TRUE;

    wstring deviceId;

    bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);

    if(!bRes) {
        /*
        * Repeat function call GetSerialNumberFromKernelIoControl to fix
        * retrieve serial number from Asus MyPal 620.
        * It's seems this function doesn't work properly if the function
        * is called only one time.
        */
        bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);
    }

    if (bRes) {

        try {
            wchar_t tmp [10];
            for (unsigned int i = 0; i<dwOutBytes; i++) {
                wsprintf(tmp, TEXT("%02X"), arrOutBuff[i]);
                deviceId += tmp;
            }

            deviceId.replace(0, 40,_T(""));
            deviceId.replace(2, 3,_T(""));
            deviceId.replace(11, 16,_T(""));
            deviceId.replace(17, 4,_T(""));

        } catch (std::exception) {

            DEVICE_ID *deviceIDInfo = (DEVICE_ID *)arrOutBuff;

            if (deviceIDInfo != NULL) {

                /* Convert PresetID into the string */
                wchar_t tmp [10];
                for (unsigned int i = 0; i<deviceIDInfo->dwPresetIDBytes; i++) {
                    wsprintf(tmp, TEXT("%02X"), arrOutBuff[deviceIDInfo->dwPresetIDOffset+i]);
                    deviceId += tmp;
                }

                /* Convert PlatformID into the string */
                for (unsigned int i = 0; i<deviceIDInfo->dwPlatformIDBytes; i++) {
                    wsprintf(tmp, TEXT("%02X"), arrOutBuff[deviceIDInfo->dwPlatformIDOffset+i]);
                    deviceId += tmp;
                }

                if (deviceId.length() > 30) {
                    deviceId.replace(2, 3,_T(""));
                    deviceId.replace(11, 12,_T(""));
                }
            } else {
                deviceId = TEXT("pocketpc-plugin");
            }
        }
        catch (...) {
            deviceId = TEXT("pocketpc-plugin");
        }
        deviceId.insert(0, TEXT("fwm-"));

        if(deviceId.length() > MAX_DEVID) {
            deviceId.erase(MAX_DEVID);
        }
    }
    else {
        deviceId = TEXT("fwm-pocketpc-plugin");
    }
    return deviceId;
}
DWORD WINAPI setClientConfiguration(wchar_t* sourceName, wchar_t* propertyName,
                                     wchar_t* propertyValue, wchar_t* errorMsg)  {

    char  spds_context [128];
    sprintf(spds_context, "%s", ROOT_CONTEXT);

    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {

        if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0) {
            node->setPropertyValue(PROPERTY_LOG_LEVEL, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0) {
            node->setPropertyValue(PROPERTY_PATH_INFO, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0) {
            node->setPropertyValue(PROPERTY_PUSH_NOTIFICATION, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0) {
            node->setPropertyValue(PROPERTY_POLLING_NOTIFICATION, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0) {
            node->setPropertyValue(PROPERTY_SERVER_NOTIFIED, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_NOTIFICATION_PORT) == 0) {
            node->setPropertyValue(PROPERTY_NOTIFICATION_PORT, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_SMS_WAP_PUSH) == 0) {
            node->setPropertyValue(PROPERTY_SMS_WAP_PUSH, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PID) == 0) {
            node->setPropertyValue(PROPERTY_PID, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_CONTACT_OLD_CRC) == 0) {
            node->setPropertyValue(PROPERTY_CONTACT_OLD_CRC, _wcc(propertyValue));
        }


    }
    if (node)
        delete node;
    node = NULL;

    return 0;
}

/*
* The method recall the other getClientConfiguration and set the variable isFromDB as true.
* This variable means that the caller is the VB and so it's necessary remove the final char '\0'
* to permit the vb to understand the return string properly
NO MORE USED

DWORD WINAPI getClientConfiguration (wchar_t* sourceName, wchar_t* propertyName,
                                     wchar_t* propertyValue) {

    return getClientConfigurationInternal (sourceName, propertyName, propertyValue, TRUE);
}
*/
/*
* Method to get the client configurations. These property are set by installing process.
* These are extra parameters not used in synchronization process
* but are needed for example to show or hide some fields in GUI interface.
*
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD getClientConfigurationInternal (wchar_t* sourceName, wchar_t* propertyName,
                                     wchar_t* propertyValue, BOOL isFromVB) {

    const int VALUESIZE = 1024;
    int nc = 0;

    char  *tmp = NULL;
    wchar_t* toRet = NULL;

    char  spds_context [128];
    sprintf(spds_context, "%s", ROOT_CONTEXT);
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    wchar_t buf[1024];

    /*
    * if sourceName is empty then is a property of root context
    */
    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {

        if (wcscmp(propertyName, PROPERTY_SPDM_REMOTE_SETTINGS) == 0) {
            tmp = node->readPropertyValue(PROPERTY_REMOTE_SETTINGS);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0) {
            tmp = node->readPropertyValue(PROPERTY_LOG_LEVEL);
        }
        //else if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0) {
        //    tmp = node->getPropertyValue(PROPERTY_ENCRYPTION);
        //}
        else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0) {
            tmp = node->readPropertyValue(PROPERTY_PATH_INFO);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0) {
            tmp = node->readPropertyValue(PROPERTY_PUSH_NOTIFICATION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_NOTIFICATION_PORT) == 0) {
            tmp = node->readPropertyValue(PROPERTY_NOTIFICATION_PORT);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0) {
            tmp = node->readPropertyValue(PROPERTY_POLLING_NOTIFICATION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0) {
            tmp = node->readPropertyValue(PROPERTY_SERVER_NOTIFIED);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_SMS_WAP_PUSH) == 0) {
            tmp = node->readPropertyValue(PROPERTY_SMS_WAP_PUSH);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_PID) == 0) {
            tmp = node->readPropertyValue(PROPERTY_PID);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_CONTACT_OLD_CRC) == 0) {
            tmp = node->readPropertyValue(PROPERTY_CONTACT_OLD_CRC);
        }
        else
            tmp = "";

        toRet = toWideChar(tmp);
        wcscpy(propertyValue, toRet);

        if (isFromVB != NULL && isFromVB == TRUE)
            propertyValue[wcslen(propertyValue)] = L' ';

        goto finally;

    }

    wsprintf(buf, TEXT(""));

    wcscpy(propertyValue, buf);
    propertyValue[wcslen(propertyValue)] = L' ';

finally:
    if (toRet)
        delete [] toRet;
    toRet = NULL;

    if(tmp)
        delete [] tmp;
    tmp = NULL;

    if (node) {delete node; node = NULL; }
    return 0;
}




// ***************************************************************************
// Function Name: GetTSPLineDeviceID
//
// Purpose: To get a TSP Line Device ID
//
// Arguments:
//  hLineApp = the HLINEAPP returned by lineInitializeEx
//  dwNumberDevices = also returned by lineInitializeEx
//  dwAPIVersionLow/High = min version of TAPI that we need
//  psTSPLineName = "Cellular Line"
//
// Return Values: Current Device ID
//
// Description:
//  This function returns the device ID of a named TAPI TSP.  The Device ID is
//  used in the call to lineOpen

static DWORD GetTSPLineDeviceID(const HLINEAPP hLineApp,
                         const DWORD dwNumberDevices,
                         const DWORD dwAPIVersionLow,
                         const DWORD dwAPIVersionHigh,
                         const TCHAR* const psTSPLineName)
{
    DWORD dwReturn = 0xffffffff;
    for(DWORD dwCurrentDevID = 0 ; dwCurrentDevID < dwNumberDevices ; dwCurrentDevID++)
    {
        DWORD dwAPIVersion;
        LINEEXTENSIONID LineExtensionID;
        if(0 == lineNegotiateAPIVersion(hLineApp, dwCurrentDevID,
            dwAPIVersionLow, dwAPIVersionHigh,
            &dwAPIVersion, &LineExtensionID))
        {
            LINEDEVCAPS LineDevCaps;
            LineDevCaps.dwTotalSize = sizeof(LineDevCaps);
            if(0 == lineGetDevCaps(hLineApp, dwCurrentDevID,
                dwAPIVersion, 0, &LineDevCaps))
            {
                BYTE* pLineDevCapsBytes = new BYTE[LineDevCaps.dwNeededSize];
                if(0 != pLineDevCapsBytes)
                {
                    LINEDEVCAPS* pLineDevCaps = (LINEDEVCAPS*)pLineDevCapsBytes;
                    pLineDevCaps->dwTotalSize = LineDevCaps.dwNeededSize;
                    if(0 == lineGetDevCaps(hLineApp, dwCurrentDevID,
                        dwAPIVersion, 0, pLineDevCaps))
                    {
                        if(0 == _tcscmp((TCHAR*)((BYTE*)pLineDevCaps+pLineDevCaps->dwLineNameOffset),
                            psTSPLineName))
                        {
                            dwReturn = dwCurrentDevID;
                        }
                    }
                    delete[]  pLineDevCapsBytes;
                }
            }
        }
    }
    return dwReturn;
}

/**
 * Get the phone's IMEI, with the prefix fwm- to be used as deviceID.
 * TODO: make a unique getDeviceId function here with conditional compilation.
 */
wstring getIMEI()
{
    DWORD dwNumDevs;
    DWORD dwAPIVersion = TAPI_API_HIGH_VERSION;
    LINEINITIALIZEEXPARAMS liep;
    HLINEAPP hLineApp = 0;
    HLINE hLine = 0;
    DWORD dwExtVersion;
    LPBYTE pLineGeneralInfoBytes = NULL;
    DWORD dwTAPILineDeviceID;
    const DWORD dwMediaMode = LINEMEDIAMODE_DATAMODEM | LINEMEDIAMODE_INTERACTIVEVOICE;
    LINEGENERALINFO lviGeneralInfo;
    LPLINEGENERALINFO plviGeneralInfo;
    LPTSTR tsSerialNumber;

    wstring ret(TEXT("fwm-smartphone-plugin"));

    // set the line init params
    liep.dwTotalSize = sizeof(liep);
    liep.dwOptions = LINEINITIALIZEEXOPTION_USEEVENT;

    if (lineInitializeEx(&hLineApp, 0, 0,  TEXT(PROVIDER), &dwNumDevs, &dwAPIVersion, &liep))
    {
        goto cleanup;
    }

    // get the device ID
    dwTAPILineDeviceID = GetTSPLineDeviceID(hLineApp, dwNumDevs, TAPI_API_LOW_VERSION, TAPI_API_HIGH_VERSION, CELLTSP_LINENAME_STRING);

    // error getting the line device ID?
    if (0xffffffff == dwTAPILineDeviceID)
    {
        goto cleanup;
    }

    // now try and open the line
    if(lineOpen(hLineApp, dwTAPILineDeviceID,
        &hLine, dwAPIVersion, 0, 0,
        LINECALLPRIVILEGE_OWNER, dwMediaMode, 0))
    {
        goto cleanup;
    }

    // set up ExTAPI
    if (lineNegotiateExtVersion(hLineApp, dwTAPILineDeviceID, dwAPIVersion, EXT_API_LOW_VERSION, EXT_API_HIGH_VERSION, &dwExtVersion))
    {
        goto cleanup;
    }

    // try to get the general info
    lviGeneralInfo.dwTotalSize = sizeof(lviGeneralInfo);

    // step 1: find out how much space we need
    if (lineGetGeneralInfo(hLine, &lviGeneralInfo))
    {
        goto cleanup;
    }

    // step 2: malloc space for all the info we need
    pLineGeneralInfoBytes = new BYTE[lviGeneralInfo.dwNeededSize];
    plviGeneralInfo = (LPLINEGENERALINFO)pLineGeneralInfoBytes;

    // step 3: call lGGI again with the appropriately sized buffer
    if(NULL != pLineGeneralInfoBytes)
    {
        plviGeneralInfo->dwTotalSize = lviGeneralInfo.dwNeededSize;
        if (lineGetGeneralInfo(hLine, plviGeneralInfo))
        {
            goto cleanup;
        }
    } else
    {
        goto cleanup;
    }

    // step 4: cast all the arguments to strings
    if(0 < plviGeneralInfo->dwSerialNumberSize)
    {
        tsSerialNumber = (WCHAR*)(((BYTE*)plviGeneralInfo)+plviGeneralInfo->dwSerialNumberOffset);
        for (unsigned int i = 0; i < wcslen(tsSerialNumber); i++) {

            //
            // for the serial number we force to use only ascii char number and letter
            //

            if(tsSerialNumber[i] >= 32 && tsSerialNumber[i] <= 47)
                tsSerialNumber[i] = (int)tsSerialNumber[i] + 33;
            else if (tsSerialNumber[i] >= 58 && tsSerialNumber[i] <= 64)
                tsSerialNumber[i] = (int)tsSerialNumber[i] + 7;
            else if (tsSerialNumber[i] >= 91 && tsSerialNumber[i] <= 96)
                tsSerialNumber[i] = (int)tsSerialNumber[i] - 6;
            else if (tsSerialNumber[i] >= 123 && tsSerialNumber[i] <= 126)
                tsSerialNumber[i] = (int)tsSerialNumber[i] - 10;
        }
        ret = TEXT("fwm-");
        ret += tsSerialNumber;
        if(ret.length() > MAX_DEVID) {
            ret.erase(MAX_DEVID);
        }
    }

cleanup:
    if (pLineGeneralInfoBytes)
    { delete [] pLineGeneralInfoBytes; pLineGeneralInfoBytes = NULL; }
    if (hLine) lineClose(hLine);
    if (hLineApp) lineShutdown(hLineApp);

    return ret;
}
