/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef __WORKER_H
#define __WORKER_H
/** @cond WMPLUGIN_NOTLISTENER */
/** @ingroup NotificationListenerDll */
/** @{ */
#include <windows.h>
#include <winsock2.h>

#include "s4jproxy.h"
#include "CTPMessage.h"

// Set to 0 to disable Client Push.
#define USE_CP      1


DWORD startWorker(SOCKET sock);
DWORD stopWorker();

const char *processPacket(const char *buf, int len);
const char *getMsg();
int getLen();

// Worker threads for stp/ctp
DWORD WINAPI stpWorker       (LPVOID lpv);
DWORD WINAPI ctpWorker       (LPVOID lpv);
DWORD WINAPI receiveWorker   (LPVOID lpv);
DWORD WINAPI heartbeatWorker (LPVOID lpv);
DWORD WINAPI cmdTimeoutWorker(LPVOID lpv);

void setServerNotified(bool val);
bool saveNonceParam(CTPMessage* authStatusMsg);
int printErrorStatus(CTPMessage* statusMsg);
int startSyncFromSAN(SyncNotification* sn);
int extractMsgLength(const char* package, int packageLen);
void hexDump(char *buf, int len);

DWORD WINAPI checkInternetConnectionWorker(LPVOID lpv);
DWORD WINAPI establishConnectionWorker    (LPVOID lpv);

HANDLE starCheckInternetConnectionWorker();
bool stopCheckInternetConnectionWorker(int exitCode) ;

/**
 * Starts the 'startClientPushWorker' thread, which starts the CP process.
 * We use a thread, because we need to return immediately the control to service.exe.
 */
void startClientPush();

/**
 * Starts the Client Push process.
 * If ClientPush.exe is already running, nothing is done. Otherwise creates
 * the process and saves its PID into registry (key CP_PID).
 * @return 0 if CP started succesfully.
 */
DWORD WINAPI startClientPushWorker(LPVOID lpv);


/**
 * Stops the Client Push process.
 * If ClientPush.exe is not running, nothing is done. Otherwise we send
 * the WM_CLOSE message to the CP window to correctly close and unregister CP.
 * Then the PID of CP is resetted in the registry.
 */
void stopClientPush();

/**
 * Checks if Client Push process is running or not.
 * PID of ClientPush.exe is stored in registry (key CP_PID), we then
 * check if that process is still running.
 * @return  the PID if CP is running, 0 if not running.
 */
DWORD isClientPushRunning();

/** @} */
/** @endcond */
#endif

