/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef __OTA_CONFIG_MESSAGE_H__
#define __OTA_CONFIG_MESSAGE_H__

#include <windows.h>
#include "pushclient.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreefactory.h"
#include "base/Log.h"
#include "pim/utils.h"
#include "base/util/utils.h"

class OTAConfigMessage  {

private:

    const char* syncURL;
    const char* userName;
    const char* password;

    const char* visibleName;
    const char* mailAddress;

    // set the uri of the source. If NULL never set
    const char* mailURI;
    const char* contactURI;
    const char* calendarURI;
    const char* taskURI;
    const char* noteURI;
    const char* briefcaseURI;

    // set the format of the source. If NULL never set
    const char* contactFormat;
    const char* calendarFormat;
    const char* taskFormat;

    // set the localURI
    const char* noteLocalURI;
    const char* briefcaseLocalURI;


public:

    /*
    * return an ota config message
    */
    OTAConfigMessage();
    ~OTAConfigMessage();

    void setUsername(const char* user);
    const char* getUsername() { return userName; }

    void setPassword(const char* pass);
    const char* getPassword() { return password; }

    void setSyncURL(const char* url);
    const char* getSyncURL() { return syncURL; }

    void setVisibleName(const char* name);
    const char* getVisibleName() { return visibleName; }

    void setMailAddress(const char* address);
    const char* getMailAddress() { return mailAddress; }

    void setMailURI(const char* uri);
    const char* getMailURI() { return mailURI; }

    void setContactURI(const char* uri);
    const char* getContactURI() { return contactURI; }

    void setCalendarURI(const char* uri);
    const char* getCalendarURI() { return calendarURI; }

    void setTaskURI(const char* uri);
    const char* getTaskURI() { return taskURI; }

    void setNoteURI(const char* uri);
    const char* getNoteURI() { return noteURI; }

    void setBriefcaseURI(const char* uri);
    const char* getBriefcaseURI() { return briefcaseURI; }

    void setContactFormat(const char* uri);
    const char* getContactFormat() { return contactFormat; }

    void setCalendarFormat(const char* uri);
    const char* getCalendarFormat() { return calendarFormat; }

    void setTaskFormat(const char* uri);
    const char* getTaskFormat() { return taskFormat; }

    void setNoteLocalURI(const char* uri);
    const char* getNoteLocalURI() { return noteLocalURI; }

    void setBriefcaseLocalURI(const char* uri);
    const char* getBriefcaseLocalURI() { return briefcaseLocalURI; }


    /*
    * return true if the action goes properly; false othewhise
    */
    static bool store(OTAConfigMessage otaMsg);

};


#endif

